//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//


#include <QAbstractButton>
#include <QColor>
#include <QMap>
#include <QPushButton>
#include <QDebug>

#include "RunControlWidget.h"
#include "adcmlib/daq_ipc.h"
#include "runcontrol.h"

#define PREFIX "RunControlWidget: "

RunControlWidget::RunControlWidget(QWidget *parent) :
    QWidget(parent)
{
    setupUi(this);
    state = ST_OFF;
    updateGui();
}

void RunControlWidget::enablePowerOff(bool value)
{
    if (value)
        pbPower->show();
    else
        pbPower->hide();
}

void RunControlWidget::on_pbPower_clicked()
{
    emit powerOff();
}

void RunControlWidget::on_pbReset_clicked()
{
    emit changeStateRequest(ST_RESET);
}

void RunControlWidget::on_pbRun_clicked()
{
    emit changeStateRequest(ST_RUN);
}

void RunControlWidget::on_pbPause_clicked()
{
    emit changeStateRequest(ST_PAUSE);
}

void RunControlWidget::setState(daq_state_t value)
{
    state = value;
    updateGui();
}

void RunControlWidget::updateGui()
{
    updateStatusLabel();
    updateButtons();
}

void RunControlWidget::updateStatusLabel()
{
//    qDebug() << "RunControlWidget: state " << daq_state_str(state);
    /*
  tlStatus->setText (daq_state_str[state]);
  tlStatus->setPaletteBackgroundColor (statusPaletteBackgroundColor());
*/
}

void RunControlWidget::updateButtons()
{
    QMap<daq_state_t, QAbstractButton *> transitionButtonMap;
    transitionButtonMap[ST_RESET] = pbReset;
    transitionButtonMap[ST_RUN] = pbRun;
    transitionButtonMap[ST_PAUSE] = pbPause;

    // highlight buttons
    for (auto toState : transitionButtonMap.keys()) {
        QAbstractButton *toButton = transitionButtonMap[toState];
        toButton->setEnabled(RunControl::isTransitionAllowed(state, toState));
        if (toState == RunControl::highlightState(state)) {
//            toButton->setStyleSheet("QAbstractButton { background-color : green; }");
            toButton->setStyleSheet("QAbstractButton { color : green; }");
        } else {
            toButton->setStyleSheet("QAbstractButton {}");
        }
    }

}
