
#include <cstdlib>
#include <iostream>
#include <exception>
#include <stdexcept>

#include <QApplication>
#include <QMessageBox>
#include <QTextCodec>
#include <QSplashScreen>
#include "QStyleFactory"
#include <QtCore>

#include "mlink/exc.h"
#include "adcm_app/AdcmApp.h"
#include "mainwindow.h"
#include "qlog/qlog.h"
#include "ProgramArguments.h"
#include "oemstrings.h"

#ifndef DEBUG_STARTUP_DEBUG_LEVEL
static const int DEBUG_STARTUP_DEBUG_LEVEL = 0;
#endif

int main( int argc, char ** argv )
{
    logger.setDebugLevel(DEBUG_STARTUP_DEBUG_LEVEL);

    QTextCodec::setCodecForLocale(QTextCodec::codecForName("utf8"));
    qInfo() << QString("Starting %1").arg(argv[0]);
    //    qInstallMsgHandler(myMessageOutput);

    QApplication a( argc, argv );
//    if(QStyleFactory::keys().contains("GTK+"))
//        a.setStyle(QStyleFactory::create("GTK+"));
//    else if(QStyleFactory::keys().contains("Plastique"))
//        a.setStyle(QStyleFactory::create("Plastique"));
//    else if(QStyleFactory::keys().contains("Windows"))
//        a.setStyle(QStyleFactory::create("Windows"));

    QSplashScreen splash(QPixmap(":/icons/adcm-splash.png"));
//    splash.show();
    splash.showMessage(QString("Loading translations"));
//    a.processEvents();
    QCoreApplication::setOrganizationName("AFI Electronics");
    QCoreApplication::setOrganizationDomain("local");
    QCoreApplication::setApplicationName("ADCM");

//    // translation file for Qt
//    QTranslator qt( 0 );
//    qt.load( QString( "qt_" ) + QTextCodec::locale(), "." );
//    a.installTranslator( &qt );


//    // translation file for application strings
//    QTranslator myapp( 0 );
//    myapp.load( QString( "adcm_" ) + QTextCodec::locale(), "." );
//    a.installTranslator( &myapp );

    ProgramArguments programArguments;
    programArguments.parseCommandLine();
    if (programArguments.print_version_and_exit) {
        qInfo().noquote() << OEMStrings::name() << OEMStrings::version();
        exit(0);
    }

    MainWindow *w = nullptr;
    try {
        splash.showMessage(qApp->tr("Starting ADCM application"));
        auto adcmApp(new AdcmApp(&a));
        w = new MainWindow(adcmApp, programArguments);
        splash.showMessage(qApp->tr("Starting GUI"));
//        a.setMainWidget(w);
        w->show();
        splash.finish(w);
        w->initialize_gui();
        a.connect( &a, SIGNAL( lastWindowClosed() ), &a, SLOT( quit() ) );
        int ret = a.exec();
        delete w;
        delete adcmApp;
        return ret;
    }
    catch (mlink::exc &e) {
        QString str = QString("Application error: ")+"exception " + QString::fromLocal8Bit(e.what());
        qCritical() << str;
        QMessageBox::critical(w, "Error", str, QMessageBox::Cancel,  Qt::NoButton);
        return EXIT_FAILURE;
    }
    catch (std::runtime_error &e) {
        QString str = QString("Application error: ")+"std::runtime_error " + QString::fromLocal8Bit(e.what());
        qCritical() << str;
        QMessageBox::critical(w, "Error", str, QMessageBox::Cancel,  Qt::NoButton);
        return EXIT_FAILURE;
    }
    catch (std::exception &e) {
        QString str = QString("Application error: ")+"std::exception " + QString::fromLocal8Bit(e.what());
        qCritical() << str;
        QMessageBox::critical(w, "Error", str, QMessageBox::Cancel,  Qt::NoButton);
        return EXIT_FAILURE;
    }
}
