//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "mainwindow.h"

#include <cstdio>
#include <cstdlib>
#include <cstdarg>
#include <cmath>
#include <iostream>
#include <stdexcept>
#include <chrono>

#include <QAction>
#include <QApplication>
#include <QCheckBox>
#include <QComboBox>
#include <QDebug>
#include <QDesktopServices>
#include <QFileDialog>
#include <QLabel>
#include <QList>
#include <QMainWindow>
#include <QMessageBox>
#include <QMutex>
#include <QProgressDialog>
#include <QPushButton>
#include <QRadioButton>
#include <QSettings>
#include <QSpinBox>
#include <QStatusBar>
#include <QTabWidget>
#include <QTextStream>
#include <QTimer>
#include <QToolTip>
#include <QThread>
#include <QValidator>

#include <qwt_scale_engine.h>

#include "oemstrings.h"
#include "version_str.h"
#include "adcmraw/adcm_mem.h"
#include "config/ConfigManager.h"

//#include "bitop.h"
#include "fittersettingsdialog.h"
#include "DeconvConfigDialog.h"
#include "adcmlib/runstats.h"
#include "dsplib/dsplib.h"
#include "dsplib/fir.h"
//#include "SetupDataOutputDialog.h"
#include "EvStorageConfigDialog.h"
#include "RawStorageConfigDialog.h"
#include "RunConfigDialog.h"
#include "AdcConfigDialog.h"
#include "ChannelControlWidget.h"

#include "adcmlib/adcm_dec.h"
#include "adcmlib/AdcmChbuf.h" // adcm_ev_t
#include "adcmlib/adcm_ev.h"
#include "allstats.h"
#include "adcmlib/ch_label.h"
#include "adcmlib/util.h"
#include "adcinstance.h"


//#include "regioemu.h"
#include "adcmdevice.h"

#include "sysmon.h"
#include "reports.h"
#include "sysinfo.h"

#include "calavgdialog.h"
#include "calzerodialog.h"
#include "DecodeManager.h"
#include "FrameList.h"
#include "EventListFiller.h"
#include "RawDataWriter.h"

#include "adcmlib/runstats.h"
#include "adcmlib/daq_ipc.h"

#include "qutil.h"
//#include "mymutexlocker.h"

#include "runcontroladcm.h"
#include "waveformmodeladc.h"
#include "WaveformController.h"

#include "RemoteControlServer.h"
#include "RunControlWidget.h"
//#include "qxw/WaveformView.h"
//#include "qxw/WaveformViewAdvanced.h"
#include "WaveformViewAdcm.h"
#include "HistExplorerView.h"
#include "ProgramArguments.h"
#include "adcm_app/AdcmApp.h"

#include "ui_mainwindowbase.h"

using namespace std::chrono; // steady_clock, duration_cast, milliseconds

#ifdef DEBUG
# undef MAINDEBUG
# define MAINDEBUG 1
#endif

#ifndef MAINDEBUG
# define MAINDEBUG 0            /* dummy */
#endif

#ifndef DECODER_PROFILING
static const int DECODER_PROFILING = 0;
#endif

//static const int MAINWINDOW_DEBUG_LEVEL = 1;
//static const int MWDL = MAINWINDOW_DEBUG_LEVEL; /* shortcut */

const double temperature_high_critical = 80;
const bool default_autosave_value = true;
const bool default_autosavegui_value = true;

//const double autosave_time = 5.0;
const double update_display_time_ms = 333;
const double update_histograms_time = 0.7; //TODO remove it
const double updateBaseStatTime = 1;
const double simulate_event_time = 1.0;

static int disable_cfg_slots = 0;          // when setting GUI boxes

//enum { CONVLEN = ADCM_MAX_N_SAMP + FIRMAXLEN };

// Deprecated
//// Persistant decoder data
//const int save_ec_default = 0x5A5A5A5A;
//unsigned int save_ec = save_ec_default; // saved event number for display update

//static bool display_average_state = false;

MainWindow::MainWindow(AdcmApp *app, const ProgramArguments &programArguments, QWidget* parent)
    : QMainWindow(parent),
    adcmApp(app),

    configManager(adcmApp->configManager),
    adcInstance(adcmApp->adcInstance),
    decodeManager(adcmApp->decodeManager),
    filler(adcmApp->filler),
    writeInstance(adcmApp->writeInstance),
    remoteControlServer(adcmApp->remoteControlServer),

      idLabel(new QLabel(this)),
      statusLabelDeviceState(new QLabel(this)),
      temperatureLabel(new QLabel(this)),
      defaultConfigFileName("adcm_config.ini"),
      defaultConfigDir(QStandardPaths::writableLocation(QStandardPaths::HomeLocation)),
      waveformModel_ADC(new WaveformModel_ADC(this)),
      waveformController(new WaveformController(waveformModel_ADC, this)),
//      timerAutoSave(new QTimer (this)),
      timerBaseStatistics(new QTimer (this)),
      timerRawWriter(new QTimer(this)),
      timerEventWriter(new QTimer(this)),
      timer_sim(new QTimer (this)),
      timerUpdateDisplay(new QTimer (this)),
      timerUpdateHistograms(new QTimer (this)), //TODO remove it
//      savedcfg(new AllConfig),
//      lastAutoSaveConfig(new AllConfig),
      deconvConfigDialog(new DeconvConfigDialog(this)),
      adcConfigDialog(new AdcConfigDialog(this)),
      calAvgDialog(new CalAvgDialog(this)),
      calZeroDialog(new CalZeroDialog(this)),
      fitProfileDialog(new FitterSettingsDialog(this)),
    evStorageConfigDialog(new EvStorageConfigDialog(this)),
    rawStorageConfigDialog(new RawStorageConfigDialog(this)),
    runConfigDialog(new RunConfigDialog(this)),
//      allstats(new (struct allstats_t)),
      prev_tv(QTime::currentTime())
{
    setObjectName("main GUI thread");
    updateWindowTitle("");
    setupUi(this); // setupUi calls connectSlotsByName()
    waveformController->setView(waveformViewAdcm);
//    connect(waveformModel_ADC, &WaveformModel_ADC::dataChanged,
//            waveformController, &WaveformController::setData);
    table_cc->horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
    statusBar()->addPermanentWidget(statusLabelDeviceState, 1);
    statusBar()->addPermanentWidget(idLabel, 9);
    statusBar()->addPermanentWidget(temperatureLabel, 1);
    if (!DECODER_PROFILING)
        tabWidget2->removeTab(tabWidget2->indexOf(tabPerformance));

    updateStatusLabel();

    connectSignals();
    initialize();
    runControl.setAutoRun(programArguments.autorun);
    initialConfigFileName = programArguments.config;
#ifdef DEBUG
//    conf_mem_t conf_mem;
//    conf_mem.setBoard(0, conf_mem_board_t(ADCMG_BOARD_ID, AdcmHardwareConfig()));
//    configManager->setAdcConfMem(conf_mem);
#endif
}

void MainWindow::connectSignals()
{
    if (DECODER_PROFILING)
        connect(this, &MainWindow::performanceInfoModified, textBrowserPerformance, &QTextBrowser::setText);

    // AdcConfig
    connect(configManager, &ConfigManager::configModified, this, &MainWindow::setWindowModified);
    connect(configManager, &ConfigManager::fileNameChanged, this, &MainWindow::setConfigFileName);
    connect(configManager, &ConfigManager::adcConfigChanged, this, &MainWindow::setAdcConfig);
    connect(configManager, &ConfigManager::adcConfigChanged, calZeroDialog, &CalZeroDialog::setAdcConfig);
    connect(configManager, &ConfigManager::adcConfigChanged, calAvgDialog, &CalAvgDialog::setAdcConfig);
    connect(configManager, &ConfigManager::adcConfigChanged, adcConfigDialog, &AdcConfigDialog::setAdcConfig);
    connect(adcConfigDialog, &AdcConfigDialog::adcConfigChanged, configManager, &ConfigManager::setAdcConfig);
    connect(configManager, &ConfigManager::adcConfigChanged, table_cc, &ChannelControlWidget::setAdcConfig);
    connect(table_cc, &ChannelControlWidget::adcConfigChanged, configManager, &ConfigManager::setAdcConfig);

    // DecoderConfig
    connect(configManager, &ConfigManager::decoderConfigChanged, this, &MainWindow::setDecoderConfig);
    connect(configManager, &ConfigManager::decoderConfigChanged, deconvConfigDialog, &DeconvConfigDialog::setDecoderConfig);
    connect(deconvConfigDialog,  &DeconvConfigDialog::decoderConfigChanged, configManager, &ConfigManager::setDecoderConfig);

    // StorageConfig
    connect(configManager, &ConfigManager::evStorageConfigChanged, this, &MainWindow::updateEventWriterTimer);
    connect(configManager, &ConfigManager::evStorageConfigChanged, evStorageConfigDialog, &EvStorageConfigDialog::setData);
    connect(evStorageConfigDialog, &EvStorageConfigDialog::dataChanged, configManager, &ConfigManager::setEvStorageConfig);

    connect(configManager, &ConfigManager::rawStorageConfigChanged, this, &MainWindow::updateRawWriterTimer);
    connect(configManager, &ConfigManager::rawStorageConfigChanged, rawStorageConfigDialog, &RawStorageConfigDialog::setData);
    connect(rawStorageConfigDialog, &RawStorageConfigDialog::dataChanged, configManager, &ConfigManager::setRawStorageConfig);

    connect(timerRawWriter, &QTimer::timeout, writeInstance, &RawDataWriter::endBurst);
    connect(timerEventWriter, &QTimer::timeout, decodeManager, &DecodeManager::endBurst);

    // RunConfig
    connect(configManager, &ConfigManager::runConfigChanged, this, &MainWindow::setRunConfig);
    connect(checkBoxHistogramFilling, &QCheckBox::toggled, decodeManager, &DecodeManager::setHistEnabled);

    connect(configManager, &ConfigManager::runConfigChanged, runConfigDialog, &RunConfigDialog::setData);
    connect(runConfigDialog, &RunConfigDialog::dataChanged, configManager, &ConfigManager::setRunConfig);

//    qRegisterMetaType<AllConfig>();
//    connect(this, SIGNAL(configModified(AllConfig)), writeInstance, SLOT(configModified(AllConfig)));
//    connect(this, SIGNAL(configModified(AllConfig)), SLOT(slot_configModified(AllConfig)));

    // conf_mem
    qRegisterMetaType<conf_mem_t>();
    connect(adcInstance, &AdcInstance::confMemChanged, this, &MainWindow::setAdcConfMem);
    connect(configManager, &ConfigManager::adcConfMemChanged, calZeroDialog, &CalZeroDialog::setAdcConfMem);
    connect(configManager, &ConfigManager::adcConfMemChanged, calAvgDialog, &CalAvgDialog::setAdcConfMem);
    connect(configManager, &ConfigManager::adcConfMemChanged, table_cc, &ChannelControlWidget::setAdcConfMem);

    connect(this, &MainWindow::startReadout, adcInstance, &AdcInstance::startReadout);
    connect(this, &MainWindow::stopReadout, adcInstance, &AdcInstance::stopReadout);
    qRegisterMetaType<ReadoutStats>();
    connect(adcInstance, &AdcInstance::readoutStatsChanged, this, &MainWindow::readoutStatsChanged);


    connect(adcInstance, &AdcInstance::hitCountsChanged, table_cc, &ChannelControlWidget::onHitCountsChanged);
    connect(adcInstance, &AdcInstance::connected, this, &MainWindow::onDeviceConnected);
    connect(adcInstance, &AdcInstance::disconnected, this, &MainWindow::onDeviceDisconnected);
    connect(adcInstance, &AdcInstance::statusTextChanged, this, &MainWindow::onDeviceStatusTextChanged);
    connect(adcInstance, &AdcInstance::readoutStateChanged, this, &MainWindow::onDeviceReadoutStateChanged);
    connect(adcInstance, &AdcInstance::timeElapsed, this, &MainWindow::updateRuntime);
    connect(adcInstance, &AdcInstance::byteCountChanged, this, &MainWindow::updateRunBytes);
    connect(adcInstance, &AdcInstance::tempChanged, this, &MainWindow::updateTemperature);

    // CalAvg
    qRegisterMetaType<CalAvgChannelConfig>();
    connect(configManager, &ConfigManager::calAvgConfigChanged, calAvgDialog, &CalAvgDialog::setCalAvgConfig);
    connect(calAvgDialog, &CalAvgDialog::calAvgConfigChanged, configManager, &ConfigManager::setCalAvgConfig);

    // CalZero
    qRegisterMetaType<CalZeroConfig>();
    connect(configManager, &ConfigManager::calZeroConfigChanged, calZeroDialog, &CalZeroDialog::setCalZeroConfig);
    connect(calZeroDialog, &CalZeroDialog::calZeroConfigChanged, configManager, &ConfigManager::setCalZeroConfig);

    // FitProfile
    qRegisterMetaType<FitProfile>();
    connect(configManager, &ConfigManager::fitProfileChanged, fitProfileDialog, &FitterSettingsDialog::setData);
    connect(fitProfileDialog, &FitterSettingsDialog::modified, configManager, &ConfigManager::setFitProfile);

    connect(filler, &EventListFiller::progressChanged, this, &MainWindow::decoderProgress_changed);
    connect(filler, &EventListFiller::eventsProceedUpdated, this, &MainWindow::eventsProceed_changed);
    qRegisterMetaType<adcm_decstat_t>();
    connect(decodeManager, &DecodeManager::summedStatisticsUpdated, this, &MainWindow::onDecoderStatisticsUpdated);
    qRegisterMetaType<HistList>();
    connect(decodeManager, &DecodeManager::histListModified, this, &MainWindow::onHistListModified);
    connect(adcmApp, &AdcmApp::histogramsChanged, histExplorerView, &HistExplorerView::onHistogramsChanged);
    connect(decodeManager, &DecodeManager::monitorEvent, this, &MainWindow::displayDecodedEvent);

    // Run Control connections
    //connect(this, SIGNAL(transitionTaskComplete(daq_state_t)), &runControl, SLOT(transitionTaskComplete(daq_state_t))); //emit was commented
    connect(RunControlWidget1, &RunControlWidget::changeStateRequest, &runControl, &RunControlAdcm::changeStateForced);
    connect(RunControlWidget1, &RunControlWidget::powerOff, this, &MainWindow::powerOff);
    connect(&runControl, &RunControlAdcm::transitionTask, this, &MainWindow::transitionTask);
    connect(&runControl, &RunControlAdcm::stateChanged, RunControlWidget1, &RunControlWidget::setState);
    connect(&runControl, &RunControlAdcm::stateChangeEnabled, RunControlWidget1, &RunControlWidget::setEnabled);

//    connect(histExplorerView, &HistExplorerView::clearHist, decodeManager, &DecodeManager::clearHist);
//    connect(histExplorerView, &HistExplorerView::clearAllHists, decodeManager, &DecodeManager::clearAllHists);
    connect(histExplorerView, &HistExplorerView::clearHist, adcmApp, &AdcmApp::clearHist);
    connect(histExplorerView, &HistExplorerView::clearAllHists, adcmApp, &AdcmApp::clearAllHists);

    connect(this, &MainWindow::startRemoteControl, remoteControlServer, &RemoteControlServer::startListen);
    connect(remoteControlServer, &RemoteControlServer::startRequest, this, &MainWindow::rcStartRequest);
    connect(remoteControlServer, &RemoteControlServer::stopRequest, this, &MainWindow::rcStopRequest);
    connect(this, &MainWindow::updateRemoteState, remoteControlServer, &RemoteControlServer::runStateUpdated);
}

void MainWindow::initialize()
{
    // set default settings
    QSettings settings;
    settings.setValue("autosave", default_autosave_value);
    settings.setValue("autosavegui", default_autosavegui_value);

    QFileInfo fi("/mnt/live/linuxrc");
    const bool is_live = fi.exists();
    RunControlWidget1->enablePowerOff(is_live);
    //  tlStatus->hide();
    //  tlSA->hide();
    //adjustSize();

    runControl.readSettings();
    //    runControl.enable();
    //    runControl.changeState(ST_INIT);
    emit startRemoteControl(true);
}

void MainWindow::initialize_gui()
{
//    clearStats ();
    update_mainhist_view();
    clear_wf_view();

    disable_cfg_slots = 0;

//    config_modified(); // force re-read config.xml
    // set startup defaults

//    Q_ASSERT(d->cfg.isValid());

//    {
//        int wst1 = decodethread->get_req_seq();
//        int wst2=wst1;
//        //        qDebug() << QString("Starting decodethread (seq=%1)").arg(wst1);
//        decodethread->start ();
//        //        qDebug() << "Waiting decodethread to start...";
//        while(wst1 == (wst2=decodethread->get_req_seq())) {
//            qApp->processEvents();
//        }
//        //        qDebug() << QString("decodethread started (seq=%1)").arg(wst2);
//    }
//    {
//        int wst1 = writethread->get_req_seq();
//        int wst2=wst1;
//        //        qDebug() << QString("Starting writethread (seq=%1)").arg(wst1);
//        writethread->start ();
//        //        qDebug() << "Waiting writethread to start...";
//        while (wst1==(wst2=writethread->get_req_seq())) {
//            qApp->processEvents();
//        }
//        //        qDebug() << QString("writethread started (seq=%1)").arg(wst2);
//    }
//    decodethread->start();
//    writethread->start();
//    qDebug() << "Threads started";

    // update now
    //  update_mainhist_view();

//    connect (timerAutoSave, SIGNAL (timeout ()), SLOT (autoSave ()));
//    timerAutoSave->setSingleShot(false);
//    timerAutoSave->start ((autosave_time*1e3));

    connect (timerBaseStatistics, &QTimer::timeout, this, &MainWindow::updateBaseStats);
    timerBaseStatistics->setSingleShot(false);
    timerBaseStatistics->start ((updateBaseStatTime*1e3));

    connect (timerUpdateDisplay, &QTimer::timeout, this, &MainWindow::update_display);
    timerUpdateDisplay->setSingleShot(true);
    timerUpdateDisplay->start ((update_display_time_ms));

    //    connect (timerUpdateScope, SIGNAL (timeout ()), SLOT (update_scope_event ()));
    //    timerUpdateScope->setSingleShot(true);
    //    timerUpdateScope->start ((int)(update_scope_time*1e3));

    connect (timer_sim, &QTimer::timeout, this, &MainWindow::simulate_event);
    timer_sim->setSingleShot(false);
    timer_sim->start ((simulate_event_time*1e3));

    connect (timerUpdateHistograms, &QTimer::timeout, this, &MainWindow::update_mainhist_view); //TODO remove it
    timerUpdateHistograms->start ((update_histograms_time*1e3)); //TODO remove it

//    qDebug() << "Timers started";
    //  recreateHistograms ();
    //    qDebug() << "Histograms initialized";
//    display_refresh_run_statistics ();

    //    qDebug() << "MainWindow init complete";

    QSettings s;
    bool val = s.value("histogramFilling", true).toBool();
    checkBoxHistogramFilling->setChecked(val);
    decodeManager->setHistEnabled(val);

//    onDeviceDisconnected();
//    Q_ASSERT(d->cfg.isValid());
}

void MainWindow::status (const QString &str)
{
    QStatusBar *bar = statusBar ();
    if (!bar)
        return;
    bar->showMessage(str);
}

QString MainWindow::getSaveFileName()
{
    QString fileName = cacheConfigFileName;
    if (fileName.isEmpty())
        fileName = defaultConfigDir + QDir::separator() + defaultConfigFileName;

    fileName = QFileDialog::getSaveFileName (
                this, "Save setup",
                fileName, "INI files (*.ini);;All files (*)");
    return fileName;
}

void MainWindow::doSaveWork(const QString &fileName)
{
    bool state = configManager->saveConfig(fileName);
    if (!state) {
        QMessageBox::warning (
            this, "Save...",
            QString("Can't save configuration file '%1'").arg(fileName),
            QMessageBox::Cancel, Qt::NoButton);
    }
}

///////////////////////// Start

void MainWindow::powerOff()
{
    if (QMessageBox::warning (this, "Shutdown",
                              "System will be shut down.",
                              QMessageBox::Ok, QMessageBox::Cancel) != QMessageBox::Ok)
        return;
    int ret=system("poweroff");
    Q_UNUSED(ret)
}

void MainWindow::transitionTask(daq_state_t toState)
{
//    qDebug() << QString("MainWindow::transitionTask: starting task for state %1").arg(daq_state_str(toState));

    localWaitCursor lwc(this);
    qApp->processEvents();

    //  usleep(200000);
    //  qApp->processEvents();
    //  emit transitionTaskComplete(toState);
    //  return;


    taskret_t taskret = TASK_ERROR;
    switch (toState) {
    case ST_OFF:
        taskret = task_off ();
        break;
    case ST_RESET:
        taskret = task_reset ();
        break;
    case ST_RUN:
        if(checkStopCondition()){
            toState = ST_PAUSE;
            qWarning()<<"Can't start readout due to stop conditions.";
            break;
        } else {
            taskret = task_run ();
            break;
        }
    case ST_PAUSE:
        taskret = task_pause ();
        break;
    case ST_ERROR:
        taskret = task_error ();
        break;
    case ST_FAILURE:
        taskret = task_failure ();
        break;
    default:
        qCritical() << (QString("No task for state %1").arg(daq_state_str(toState)));
        taskret = TASK_OK;
    }
    if (taskret == TASK_OK)
        runControl.transitionTaskComplete(toState);
    else
        runControl.transitionTaskFailed(toState);
    emit updateRemoteState(daq_state_str(runControl.state()));
}

void MainWindow::adcMatchWindowChanged(const AdcConfig &adcConfig)
{
    const double step = adcConfig.step;
    const double winL = -1*adcConfig.prewin * step;
    const double winR = winL + adcConfig.matchwin * step;
    double waveformScaler = 1e-3;
    waveformViewAdcm->setWin(winL*waveformScaler, winR*waveformScaler);
}

void MainWindow::setAdcConfMem(const conf_mem_t& conf_mem)
{
    cacheAdcConfMem = conf_mem;
    disable_cfg_slots++;
    setupTrigger_setupAction->setVisible(conf_mem.hasTriggerControl());
    updateWaveformViewFilters(conf_mem, cacheDecoderConfig);
    disable_cfg_slots--;
}

void MainWindow::setAdcConfig(const AdcConfig &adcConfig)
{
    cacheAdcConfig = adcConfig;
    disable_cfg_slots++;
    adcMatchWindowChanged(adcConfig);
    display_refresh_trig_control(adcConfig);
    DecPermChange();
    TrigRestore ();
    disable_cfg_slots--;
}

void MainWindow::setDecoderConfig(const DecoderConfig &decoderConfig)
{
    cacheDecoderConfig = decoderConfig;
    disable_cfg_slots++;
    updateWaveformViewFilters(cacheAdcConfMem, cacheDecoderConfig);
    DecRestore(cacheDecoderConfig);
    DecPermChange();
    disable_cfg_slots--;
}

void MainWindow::setRunConfig(const RunConfig &runConfig)
{
    cacheRunConfig = runConfig;
}

void MainWindow::updateEventWriterTimer(const StorageConfig &stor)
{
    cache_stor_dev = stor;
    if(!stor.enable || stor.mode)
        timerEventWriter->stop();
    else {
        timerEventWriter->setSingleShot(false);
        timerEventWriter->start(stor.period*1e3);
    }
}

void MainWindow::updateRawWriterTimer(const StorageConfig &stor)
{
    cache_stor_raw = stor;
    if(!stor.enable || stor.mode)
        timerRawWriter->stop();
    else {
        timerRawWriter->setSingleShot(false);
        timerRawWriter->start(stor.period*1e3);
    }
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    emit startRemoteControl(false);
    QMainWindow::closeEvent(event);
}

// Autoconnected Menu Actions

void MainWindow::on_fileNewAction_triggered ()
{
    if (QMessageBox::warning (this, "Reset All Settings...",
                              "Do you want to reset all the settings to default values and remove calibration data?",
                              QMessageBox::Yes, QMessageBox::No|QMessageBox::Escape|QMessageBox::Default
                              ) != QMessageBox::Yes) return;

    configManager->newConfig();

    runControl.readSettings();
    runControl.enable();
    runControl.changeState(ST_RESET);
}

void MainWindow::on_fileOpenAction_triggered ()
{
    QString newFileName = QFileDialog::getOpenFileName (
        this, "Open configuration",
        cacheConfigFileName.isEmpty() ? defaultConfigDir : cacheConfigFileName,
        "INI files (*.ini);;All files (*)");

    if (newFileName.isEmpty())
        return;

    ConfigStatus status = configManager->loadConfig(newFileName, false);  // no merge
    if (!status) {
        QString text;
        text = status.errorText().join("\n");
        QMessageBox::warning (
            this, "Open...",
            QString("Can't load configuration file<br><pre>%1</pre><p><pre>%2</pre>")
                .arg(newFileName)
                .arg(text),
            QMessageBox::Cancel, Qt::NoButton);
    }
}

void MainWindow::on_fileSaveAction_triggered ()
{
    QString fileName = cacheConfigFileName;
    if (fileName.isEmpty())
        fileName = getSaveFileName();
    if (fileName.isEmpty())
        return;
    doSaveWork(fileName);
}

void MainWindow::on_fileSaveAsAction_triggered ()
{
    QString fileName = getSaveFileName();
    if (fileName.isEmpty())
        return;
    doSaveWork(fileName);
}

void MainWindow::on_fileExitAction_triggered ()
{
    //  AutoSave();
    close ();
}

void MainWindow::on_setupFitter_setupAction_triggered ()
{
    fitProfileDialog->show();
    fitProfileDialog->raise();
}

void MainWindow::on_setupDecoder_setupAction_triggered()
{
    QSize s = size();
    deconvConfigDialog->resize(QSize(s.width() * 4 / 5, s.height() * 4 / 5));
    deconvConfigDialog->show();
    deconvConfigDialog->raise();
}

void MainWindow::on_fileRawFileOutput_triggered()
{
    rawStorageConfigDialog->show();
    rawStorageConfigDialog->raise();
}

void MainWindow::on_fileDecEvFileOutput_triggered()
{
    evStorageConfigDialog->show();
    evStorageConfigDialog->raise();
}

void MainWindow::on_actionRunConfiguration_triggered()
{
    runConfigDialog->show();
    runConfigDialog->raise();
}

void MainWindow::on_actionAdcConfiguration_triggered()
{
    adcConfigDialog->show();
    adcConfigDialog->raise();
}

void MainWindow::on_statisticsBaselineOffsetAction_triggered()
{
    calZeroDialog->show ();
    calZeroDialog->raise ();
}

void MainWindow::on_statisticsPulseShapeAction_triggered()
{
    calAvgDialog->show ();
    calAvgDialog->raise ();
}

void MainWindow::on_optionsOptionsAction_triggered()
{
#if 0
    Q_D(const MainWindow);
    SettingsDialog settingsDialog(d->adcConfMem, this);
    SettingsData data;

    data.copyFromCfg(d->cfg);
    settingsDialog.setData(data);
    runControl.writeSettings();
    if (settingsDialog.exec() == QDialog::Accepted) {
        AllConfig newConfig(d->cfg);
        settingsDialog.getData().copyToCfg(&newConfig);
        runControl.readSettings();
        if(d->cfg != newConfig)
            setAllConfig(newConfig);
    }
//    CfgApply();
//    RestoreAll();
#endif
}

void MainWindow::on_helpAboutAction_triggered ()
{
    QString astr = QString("<p>") + OEMStrings::name() + QString("<br>")+
            OEMStrings::version() + "<br><br>" +
            OEMStrings::copyright() +"<br><br>" +
            OEMStrings::address() +"<br><br>" +
            "<a href=\""+OEMStrings::www()+"\">"+OEMStrings::www()+"</a>"+
            "</p>";
    QMessageBox::about (this, "About", astr);

}

void MainWindow::on_helpAboutQtAction_triggered ()
{
    QApplication::aboutQt();
}

// Autoconnected slots

void MainWindow::on_rbDecRec_clicked()
{
    setDecoderMode(DM_REC);
}

void MainWindow::on_rbDecAvg_clicked()
{
    setDecoderMode(DM_AVG);
}

void MainWindow::on_rbDecOffset_clicked()
{
    setDecoderMode(DM_ZERO);
}

void MainWindow::on_rbDecOff_clicked()
{
    setDecoderMode(DM_OFF);
}

void MainWindow::on_checkBoxHistogramFilling_toggled(bool checked)
{
    QSettings s;
    s.setValue("histogramFilling", checked);
}

void MainWindow::on_groupBox_dt_cut_toggled( bool checked )
{
    if (cacheDecoderConfig.dt_cut_en == checked)
        return;
    cacheDecoderConfig.dt_cut_en = checked;
    configManager->setDecoderConfig(cacheDecoderConfig);
}

void MainWindow::on_awl_valueChanged( int value )
{
    if (cacheDecoderConfig.dt_cut_l == value)
        return;
    cacheDecoderConfig.dt_cut_l = value;
    configManager->setDecoderConfig(cacheDecoderConfig);
}

void MainWindow::on_awr_valueChanged( int value )
{
    if (cacheDecoderConfig.dt_cut_r == value)
        return;
    cacheDecoderConfig.dt_cut_r = value;
    configManager->setDecoderConfig(cacheDecoderConfig);
}

void MainWindow::on_tabWidget2_currentChanged( int )
{
    status("");
    //  std::cout << title <<std::endl;
    if (histograms_tab_shown())
        update_mainhist_view();
}

void MainWindow::on_pushButtonClearDecStat_clicked()
{
    sumdecst.clear();
    rateDecSt.clear();
    rateDecStLast.clear();
    rateDecStTime = QTime();
}

///////////////////////// End
// slot
void MainWindow::updateWaveformViewFilters(const conf_mem_t &conf_mem, const DecoderConfig &dec)
{
    for (auto c : dec.channels.keys())
        waveformController->setFilterData(c, dec.channels[c].deconv.fir);
    int bits = conf_mem.getMasterAdcBits();
    for (int c=0; c<conf_mem.nch; c++)
        waveformController->setFullScale(c, 1<<(bits-1));
}

//--------------------------------------------------------------------------
//
//  Functions for setting GUI boxes from config parameters
//
void MainWindow::RestoreAll ()
{
    disable_cfg_slots++;        // disable CfgApply()
    // FIXME FIXME FIXME    QToolTip::setGloballyEnabled(d->cfg.flags & cfg_flag_tooltips);
    runControl.readSettings();

    StorRestore ();
//    display_refresh_channel_controls ();
//    display_update_channel_controls_visibility();
    histExplorerView->updateView();
    update_info();
    disable_cfg_slots--;
}

/** depends on d->cfg.dec, d->cfg.adc */
void MainWindow::DecPermChange()
{
    int avgok = 1, zerok = 1, firok = 1;
    int zerSt = 0, avgst = 0, recSt = 0;

    const auto &adc = cacheAdcConfig;
    const auto &dec = cacheDecoderConfig;
    const auto &calzero = dec.calzero;
    const auto &calavg = dec.calavg;
//    const int n = d->cfg.adc.size();
//    if (d->cfg.dec.calzero.nent.size() < n) zerok = false;
//    if (d->cfg.dec.calavg.nent.size() < n) avgok = false;
//    if (d->cfg.dec.fir.size() < n) firok = false;

    for (auto c : adc.channels.keys()) {
        if (! adc.channels[c].flags.bit.en) continue;
        if (!(adc.channels[c].flags.bit.m || adc.channels[c].flags.bit.s)) continue;

        if (zerok && calzero.channels[c].isEmpty())
            zerok = 0;
        if (avgok && calavg.channels[c].isEmpty())
            avgok = 0;
        if (firok && (!dec.channels[c].deconv.fir.validate() || dec.channels[c].deconv.fir.isEmpty()))
            firok = 0;
        //	qDebug() << (1, "ch %2d, zer: %d, avg: %d, fir: %d", c, avgok, zerok, firok);
    }

    if (zerok && avgok) {
        if (firok)
            recSt = 2;
        else
            recSt = 1;
    } else
        recSt = 1;

    if (zerok) {
        if (avgok)
            avgst = 0;
        else
            avgst = 2;
    } else
        avgst = 1;
    if (!zerok)
        zerSt = 2;

    //  qDebug() << (0, "zer: %d, avg: %d, fir: %d\n", avgok, zerok, firok);
    if (zerSt == 2)
        rbDecOffset->setStyleSheet("QAbstractButton { color : blue; }");
    else
        rbDecOffset->setStyleSheet("QAbstractButton {}");

    if (avgst == 2)
        rbDecAvg->setStyleSheet("QAbstractButton { color : blue; }");
    else if (avgst == 1)
        rbDecAvg->setStyleSheet("QAbstractButton { color : red; }");
    else
        rbDecAvg->setStyleSheet("QAbstractButton {}");

    if (recSt == 2)
        rbDecRec->setStyleSheet("QAbstractButton { color : blue; }");
    else if (recSt == 1)
        rbDecRec->setStyleSheet("QAbstractButton { color : red; }");
    else
        rbDecRec->setStyleSheet("QAbstractButton {}");
}

void MainWindow::TrigRestore ()
{
    const auto & conf_mem = cacheAdcConfMem;
    const auto & adc = cacheAdcConfig;
    //  sbRW = adc.recwin[0];
    // calculate bounds
    const int maxrecwin = ADCM_MAX_N_SAMP; // adc.adc_cfg_get_recwin_max();
    const int lat_max = std::min(maxrecwin, conf_mem.adc_get_latency_max());
    const int mw_max = std::min(maxrecwin, conf_mem.adc_get_matchwin_max());
    const double step = adc.step;
    const double winL = -1.0 * adc.prewin * step;
    const double winR = winL + adc.matchwin * step;

    sbPW->setMinimum (- conf_mem.adc_get_prewin_max() * step);
    sbPW->setMaximum (- conf_mem.adc_get_prewin_min() * step);
    sbPW->setValue (winL);
    sbPW->setSingleStep (step);

    sbMW->setMinimum (conf_mem.adc_get_matchwin_min() * step);
    sbMW->setMaximum (mw_max * step);
    sbMW->setValue (winR-winL);
    sbMW->setSingleStep (step);

    sbLat->setMinimum (conf_mem.adc_get_latency_min() * step);
    sbLat->setMaximum (lat_max * step);
    sbLat->setValue (adc.latency * step);
    sbLat->setSingleStep (step);

//    awr->setSingleStep (step);
    adcMatchWindowChanged(adc);
//    display_refresh_trig_control();
}

void MainWindow::StorRestore ()
{
}

void MainWindow::DecRestore(const DecoderConfig &dec)
{
    groupBox_dt_cut->setChecked(dec.dt_cut_en);
    awl->setValue((int)dec.dt_cut_l);
    //    awl->setSingleStep (step);
    awr->setValue((int)dec.dt_cut_r);
    switch(dec.mode) {
    case DM_OFF :
        rbDecOff->setChecked(true);
        break;
    case DM_ZERO :
        rbDecOffset->setChecked(true);
        break;
    case DM_AVG :
        rbDecAvg->setChecked(true);
        break;
    case DM_REC :
        rbDecRec->setChecked(true);
        break;
    }
}

void MainWindow::on_sbPW_valueChanged (double value)
{
    if (disable_cfg_slots)
        return;
    AdcConfig adcConfig(cacheAdcConfig);
    double step = adcConfig.step;
    adcConfig.prewin = -value/step;
    configManager->setAdcConfig(adcConfig);
}

void MainWindow::on_sbMW_valueChanged (double value)
{
    if (disable_cfg_slots)
        return;
    AdcConfig adcConfig(cacheAdcConfig);
    double step = adcConfig.step;
    adcConfig.matchwin = value/step;
    configManager->setAdcConfig(adcConfig);
}

void MainWindow::on_sbLat_valueChanged (int value)
{
    if (disable_cfg_slots)
        return;
    AdcConfig adcConfig(cacheAdcConfig);
    double step = adcConfig.step;
    adcConfig.latency = value / step;
    configManager->setAdcConfig(adcConfig);
}

void MainWindow::on_spinBoxSamplingLength_valueChanged(int)
{
    // TODO
//    Q_D(const MainWindow);
//    if (disable_cfg_slots)
//        return;
//    AdcConfig adcConfig(d->cfg.adc);
//    adcConfig;
//    setAdcConfig(adcConfig);
}

//--------------------------------------------------------------------------

void MainWindow::reset_threads()
{
    QMetaObject::invokeMethod(adcInstance, "reset");
    QMetaObject::invokeMethod(filler, "reset");
    QMetaObject::invokeMethod(writeInstance, "reset");
}

taskret_t MainWindow::task_off ()
{
    return TASK_OK;
}

taskret_t MainWindow::task_reset ()
{
    emit stopReadout ();
    clearStats();
    updateBaseStats();
    reset_threads();
//    CfgApply();
//    RestoreAll();
    return TASK_OK;
}

taskret_t MainWindow::task_run ()
{
    emit startReadout();
    return TASK_OK;
}

taskret_t MainWindow::task_pause ()
{
    emit stopReadout ();
    return TASK_OK;
}

taskret_t MainWindow::task_error ()
{
    qWarning() << "--- ERROR ---";
    emit stopReadout ();
    return TASK_OK;
}

taskret_t MainWindow::task_failure ()
{
    qWarning() << "--- FAILURE ---";
    emit stopReadout ();
    return TASK_OK;
}

void MainWindow::updateStatusLabel()
{
    {
        QString text = QString("%1").arg(cacheConnectedState ? cacheReadoutState ? "Readout" : "Pause" : "Disconnected");
        statusLabelDeviceState->setText(text);
        QPalette p = statusLabelDeviceState->palette();
        p.setBrush(QPalette::Background, cacheConnectedState ? cacheReadoutState ? Qt::darkGreen : Qt::darkYellow : Qt::darkRed);
        p.setBrush(QPalette::Foreground, Qt::white);
        statusLabelDeviceState->setPalette(p);
        statusLabelDeviceState->setAutoFillBackground(true);
    }
    QString statusText;
    if (cacheConnectedState)
        statusText = adcInstance->getDeviceIdStrList().join(" | ");
    else
        statusText = cacheDeviceStatusText;
    idLabel->setText(statusText);
//    idLabel->setEnabled(d->connectedState);
}

void MainWindow::onDeviceConnected()
{
    cacheConnectedState = true;
    updateStatusLabel();
    runControl.enable();
//    init_table_st();
//    recreateHistograms();
//    clearstats();
//    reset_threads();
//    CfgApply();
//    RestoreAll();
    runControl.changeState(ST_RESET);
}

void MainWindow::onDeviceDisconnected()
{
    cacheConnectedState = false;
    updateStatusLabel();
    runControl.disable();
}

void MainWindow::onDeviceStatusTextChanged(const QString &str)
{
    cacheDeviceStatusText = str;
    updateStatusLabel();
}

void MainWindow::onDeviceReadoutStateChanged(bool state)
{
    cacheReadoutState = state;
    updateStatusLabel();
}

/*
void MainWindow::update_wst ()
{
    char str[20];
    double mb;

    worker_state wst = WRK_OFF;
    if (writethread) {
        wst = writethread->getstate ();

        mb = (double) (writethread->getnwritten ()) / 1048576;
        if (mb < 1e1)
            sprintf (str, "%.3f", mb);
        else if (mb < 1e2)
            sprintf (str, "%.2f", mb);
        else if (mb < 1e3)
            sprintf (str, "%.1f", mb);
        else
            sprintf (str, "%d", (int) mb);
        // TODO
    }
}
*/

void MainWindow::decoderProgress_changed(double progress)
{
    //    qDebug() << "decoderProgress_changed" << progress;
    pbDECW->setValue ((int)(100. * progress));
}

void MainWindow::eventsProceed_changed(quint64 evNumbers)
{
    eventsProceed = evNumbers;
    if (runControl.state() == ST_RUN && checkStopCondition()) {
        runControl.changeState (ST_PAUSE);
    }
}

void MainWindow::updateRuntime(double seconds)
{
    runSeconds = seconds;
    // check if run has to be stopped
    if (runControl.state() == ST_RUN && checkStopCondition()) {
        runControl.changeState (ST_PAUSE);
    }
}

void MainWindow::updateRunBytes(quint64 bytes)
{
    runBytes = bytes;
    // check if run has to be stopped
    if (runControl.state() == ST_RUN && checkStopCondition()) {
        runControl.changeState (ST_PAUSE);
    }
}

void MainWindow::updateBaseStats ()
{
    QString str;
    double mb = runBytes / 1024. / 1024.;
    if (mb < 10240)
        str = QString("MB:%1").arg(mb, 13, 'f', 3);
    else
        str = QString("GB:%1").arg(mb/1024., 13, 'f', 3);
    tlrunsize->setText (str);

    double rt = runSeconds;
    int hour = (int) (rt / 3600);
    int min = (int) ((rt - hour * 3600) / 60);
    double sec = (rt - hour * 3600 - min * 60);

    str = QString("Time:%1:%2:%3")
              .arg(hour, 3)
              .arg(min, 2, 10, QChar('0'))
              .arg(sec, 4, 'f', 1, QChar('0'));
    tlruntime->setText (str);

    QTime tv = QTime::currentTime();
    double dmb = mb - prev_mb;
    double dt = 1e-3*prev_tv.msecsTo(tv);
    prev_tv = tv;
    prev_mb = mb;
    double rate;
    if (dt > 0)
        rate = dmb / dt;
    else
        rate = 0;
    str = QString("MB/s: %1").arg(rate, 10, 'f', 3);
    tlrunrate->setText (str);
}

/*
 *
 * Scope display options
 *
 */

void MainWindow::clear_wf_view()
{
    waveformViewAdcm->clear();
}

void MainWindow::display_refresh_trig_control(const AdcConfig &adc)
{
    bool state = !adc.is_testmode() && adc.is_co_trigger();
    //  groupBox_trig_windows->setEnabled(state);
    sbPW->setEnabled(state);
    sbMW->setEnabled(state);
    if (state) groupBox_dt_cut->show();
    else groupBox_dt_cut->hide();
    //  groupBox_dt_cut->setEnabled(state);
}

//static double scope_update_timestamp = 0;

//void MainWindow::display_refresh_scope_contents ()
//{
//    if (!oscilloscope_tab_shown()) return;
//    //  waveformView->setAveraging(display_average_state);
//    //  waveformView->setWaveformData(waveformModel_ADC.get());
//}

void MainWindow::simulate_event()
{
#ifdef DEBUG
    // simulate when no hardware present
    if (runControl.state() != ST_OFF) return;
    WaveformDataSimulator sim;
    WaveformDataAll wd(sim.run());
    waveformModel_ADC->set(wd);
#endif /* DEBUG */
}
/*
void MainWindow::update_scope_event ()
{
    update_scope();
    int t = (int)(update_scope_time*1e3);
    if (checkBoxWaveformEnableAverage->isChecked()) t = 5;
    timerUpdateScope->start(t);
}
*/

void MainWindow::displayDecodedEvent(const AdcmEvent &ev)
{
    if (ev.getFlags() & ADCM_EV_INVAL) {
        qWarning() << "displayDecodedEvent(): invalid event";
        return;
    }
    //  qDebug() <<  (4, "update_scope(): got event #%d, ts:%08X", ev->ec, ev->ts);
    //    int ec = ev.getEventNumber();
    //    if (ec == save_ec) return;
    //    save_ec = ec;

    waveformModel_ADC->addEvent(ev, cacheAdcConfMem);
}

void MainWindow::updateRawStorageStats()
{
    const qint64 maxValue = 100;
    const auto & stor = cache_stor_raw;
    qint64 bytes_total=0;
    qint64 bytes_avail=0;
    get_disk_space(stor.fileName, &bytes_total, &bytes_avail);
    qint64 value = 0;
    if (bytes_total > 0)
        value = maxValue * (bytes_total - bytes_avail) / bytes_total;

    bool enable = stor.enable && (bytes_total > 0);
    textLabel_disk_ev->setEnabled(enable);
    progressBar_disk_ev->setEnabled(enable);
    progressBar_disk_ev->setMaximum(maxValue);
    progressBar_disk_ev->setValue(value);
}

void MainWindow::updateEvStorageStats()
{
    const qint64 maxValue = 100;
    const auto & stor = cache_stor_dev;
    qint64 bytes_total=0;
    qint64 bytes_avail=0;
    get_disk_space(stor.fileName, &bytes_total, &bytes_avail);
    qint64 value = 0;
    if (bytes_total > 0)
        value = maxValue * (bytes_total - bytes_avail) / bytes_total;
    bool enable = stor.enable && (bytes_total > 0);
    textLabel_disk_raw->setEnabled(enable);
    progressBar_disk_raw->setEnabled(enable);
    progressBar_disk_raw->setMaximum(maxValue);
    progressBar_disk_raw->setValue(value);
}

void MainWindow::updateCPUStats()
{
    int cpu_user = sysmon.get("cpu_user").toInt();
    int cpu_nice = sysmon.get("cpu_nice").toInt();
    int cpu_sys = sysmon.get("cpu_sys").toInt();
    int cpu_idle = sysmon.get("cpu_idle").toInt();
    //  int cpu_iowait = sysmon.get("cpu_iowait").toInt();
    //  int cpu_irq = sysmon.get("cpu_irq").toInt();
    //  int cpu_softirq = sysmon.get("cpu_softirq").toInt();

    int busy = cpu_nice+cpu_user+cpu_sys;
    int all = busy+cpu_idle;
    int value = 0;
    int total = 100;
    if (all > 0) {
        value = total * busy / all;
    }
    progressBar_cpu_load->setEnabled(total > 0);
    progressBar_cpu_load->setMaximum(total);
    progressBar_cpu_load->setValue(value);
}

void MainWindow::update_display ()
{
//    updateWindowTitle();

    updateCPUStats();
    updateRawStorageStats();
    updateEvStorageStats();

    timerUpdateDisplay->start(update_display_time_ms);
}

void MainWindow::onDecoderStatisticsUpdated(const adcm_decstat_t &st)
{
//    if (save_decst_nw == st.nw) return;
    sumdecst += st;
    rateDecSt += st;

//    save_decst_nw = st.nw;
    if(rateDecStTime.isValid()){
        if (rateDecStTime.elapsed() > sbDecStatInterval->value()*1000){
            rateDecStLast = rateDecSt;
            rateDecStLastMs = rateDecStTime.restart();
            rateDecSt.clear();
        }
    } else {
        rateDecStTime.start();
        rateDecStLastMs=0;
    }
    QString decoderstats = make_decst_report(st, sumdecst, rateDecStLast, rateDecStLastMs);
    teinfo->setText(decoderstats);
    QString perfText;
    QTextStream stream(&perfText);
    stream << st.prof;
    QString perfTextHtml = "<pre>"+perfText+"<pre>";
    if (!perfText.isEmpty())
        emit performanceInfoModified(perfTextHtml);
}

//void MainWindow::append_decoder_statistics()
//{
//    const adcm_decstat_t &st = decodethread->get_decstat ();
//    if (save_decst_nw == st.nw) return;
//    sumdecst.add(st);
//    save_decst_nw = st.nw;
//}

// Decoder info page
//void MainWindow::display_refresh_run_statistics ()
//{
//    QString decoderstats;
//    if (decodethread) {
//        const adcm_decstat_t &ds = decodethread->get_decstat();
//        decoderstats = make_decst_report(ds, sumdecst);
//    }
//    teinfo->setText (decoderstats);
//}

void MainWindow::onHistListModified(const HistList &hl)
{
//    QStringList sl;
//    for (auto & i : hl)
//        sl << i.getname();
//    qDebug() << __PRETTY_FUNCTION__ << sl.join(", ");
//    histExplorerView->recreateHistograms(hl);
}

//void MainWindow::onHistogramsChanged(const HistList &hl)
//{
//    QStringList sl;
//    for (auto & i : hl)
//        sl << i.getname();
//    qDebug() << __PRETTY_FUNCTION__ << sl.join(", ");
//    histExplorerView->recreateHistograms(hl);
//}

void MainWindow::rcStartRequest()
{
    runControl.changeState(ST_RUN);
}

void MainWindow::rcStopRequest()
{
    runControl.changeState(ST_PAUSE);
}

void MainWindow::clearStats ()
{
    eventsProceed = 0;
    runSeconds = 0;
    runBytes = 0;
    prev_mb = 0;
    rst.clear();

    QMetaObject::invokeMethod(filler, "clear");

    update_mainhist_view();

    // clear oscilloscope data
    clear_wf_view();

    // clear main statistics
//    if (allstats)
//        memset (allstats, 0, sizeof (struct allstats_t));
    sumdecst.clear();
    rateDecSt.clear();
    rateDecStLast.clear();
    rateDecStTime = QTime();
}

void MainWindow::update_mainhist_view()
{
    histExplorerView->updateView();
    //  TimerStarterOnDestruct(timerUpdateHistograms, (int)(update_histograms_time*1e3), QTimerSingleShot);
}

void MainWindow::readoutStatsChanged(const ReadoutStats &readoutStats)
{
    constexpr double Mi = 1024 * 1024;
    QStringList text;
    constexpr auto format = "<tr><td>%1</td><td><pre>%2 %3</pre></td></tr>";
    text += "<h3>Readout stats</h3>";
    text += "<table>";
    text += QString(format).arg("Block").arg(readoutStats.statsNumBytesReceived / Mi, 5, 'f', 1).arg("MiB");
    text += QString(format).arg("Rate").arg(readoutStats.throughput_Bps / Mi, 5, 'f', 1).arg("MiB/s");
    text += QString(format).arg("Poll time").arg(readoutStats.readout_time_poll_ms, 5, 'f', 1).arg("ms");
    text += QString(format).arg("(framelist)").arg(readoutStats.readout_time_framelist_ms, 5, 'f', 1).arg("ms");
    text += QString(format).arg("(emit data)").arg(readoutStats.readout_time_emit_data_ms, 5, 'f', 1).arg("ms");
    text += "</table>";
    label_RunStats->setText(text.join(""));
}

QString MainWindow::test_report_str()
{
    QString msg;
    const quint64 n_frame_err = sumdecst.n_frame_err();
    const quint64 n_data_err = sumdecst.n_data_err();
    const quint64 n_dsp_err = sumdecst.n_dsp_err();
    const quint64 n_err = n_frame_err+n_data_err+n_dsp_err;
    msg += "<p>";
    if (n_err) {
        msg += "<font color=\"red\">";
        if (n_frame_err) msg += tr("Frame errors: %1 <br>").arg(n_frame_err);
        if (n_data_err) msg += tr("Data errors: %1 <br>").arg(n_data_err);
        if (n_dsp_err) msg += tr("DSP errors: %1 <br>").arg(n_dsp_err);
        msg += "</font>";
    } else {
        msg += "Ok";
    }
    return msg;
}

bool MainWindow::changeStateSynchronous(daq_state_t toState)
{
    runControl.changeState (toState);
    // state change is syncronous within single thread - signals are executed sequentially.
    return true;
}

bool MainWindow::histograms_tab_shown() const
{
    return tabWidget2->currentWidget() == tabHistograms;
}

bool MainWindow::oscilloscope_tab_shown() const
{
    return tabWidget2->currentWidget() == tabSettings;
}

bool MainWindow::live_statistics_tab_shown() const
{
    return oscilloscope_tab_shown();
}
void MainWindow::update_info()
{
    sysinfo si;
    textLabel_info->setText(si.text());
    textLabel_info->adjustSize();
}

bool MainWindow::checkStopCondition()
{
    const auto & run = cacheRunConfig;
    if (!run.stopCondition) return false;
    if (run.stopByTimer && runSeconds >= run.stopTimerValue) return true;
    if (run.stopByEventCount && eventsProceed >= run.stopEventCountValue) return true;
    return false;
}

void MainWindow::readGuiSettings(const QSettings *settings)
{
    Q_ASSERT(settings);
    const QString path = "MainWindow/";
    restoreGeometry(settings->value(path + "geometry").toByteArray());
    restoreState(settings->value(path + "windowState").toByteArray());
    splitter->restoreGeometry(settings->value(path + "splitter.geometry").toByteArray());
    tabWidget2->setCurrentIndex(settings->value(path + "tabWidget2.currentIndex").toInt());
    // TODO    histExplorerView->setCurrentHist(settings.value(path + "histExplorerView.currentHist").toString());
}

void MainWindow::writeGuiSettings(QSettings *settings)
{
    Q_ASSERT(settings);
    settings->beginGroup("MainWindow");
    settings->setValue("geometry", saveGeometry());
    settings->setValue("windowState", saveState());
    settings->setValue("splitter.geometry", splitter->saveGeometry());
    settings->setValue("tabWidget2.currentIndex", tabWidget2->currentIndex());
    settings->setValue("histExplorerView.currentHist", histExplorerView->currentHistName());
    settings->endGroup();
    settings->sync();
}

void MainWindow::updateTemperature(QVector<double> v)
{
    QStringList sl;
    for (auto t : v) {
        sl.push_back(QString("%1 %2C").arg(t, 0, 'f', 1).arg(QChar(0x02da)));
        if (t > temperature_high_critical) {
            temperatureOverflow++;
            if (temperatureOverflow == 3) {
                qCritical() << QString("Board temperature critical: %1").arg(sl.last());
                rcStopRequest();
            }
        } else {
            temperatureOverflow=0;
        }
    }
    temperatureLabel->setText(sl.join(", "));
}

void MainWindow::setConfigFileName(const QString &s)
{
    cacheConfigFileName = s;
    updateWindowTitle(s);
}

void MainWindow::updateWindowTitle(const QString &)
{
    QString appName = QApplication::applicationName();
    QString fileName = QFileInfo(cacheConfigFileName).fileName();
    QString shortName = fileName;
//    QDir dir;
//    if (fileName.startsWith(dir.canonicalPath()))
//        shortName = dir.relativeFilePath(fileName);
//    else
//        shortName = dir.cleanPath(fileName);
    QString title;
    if (!shortName.isEmpty())
        title = shortName + " [*] - " + appName;
    else
        title = "[*] " + appName;
    if (windowTitle() != title)
        setWindowTitle(title);
//    setWindowTitle(QString());
//    setWindowFilePath(shortName);
}

void MainWindow::setDecoderMode (decoder_mode_t mode)
{
    if ((mode < DM_OFF) || (mode > DM_REC))
        return;                     // other buttons pressed

    if (mode == DM_ZERO) {
        calZeroDialog->show ();
        calZeroDialog->raise ();
    }

    if (mode == DM_AVG) {
        calAvgDialog->show ();
        calAvgDialog->raise ();
    }
#if 0
    if ((mode == DM_ZERO) && d->cfg.enabled_ch_has_calzero ()) {

        switch (QMessageBox::warning (this, "Baseline Calibration",
                                      "Do you want to reset before new baseline calibration?"
                                      "Only enabled channels are affected.", "&Yes", "&No", "Cancel", 0, 2)) {
        case 0:
        {
            CalZeroConfig newConfig(d->cfg.dec.calzero);
            newConfig.clear_enabled(d->cfg.adc);
            setCalZeroConfig(newConfig);
        }
            break;
        case 1:
            break;
        default:
            return;
        }
    }
#endif
#if 0
    if ((mode == DM_AVG) && d->cfg.enabled_ch_has_calavg ()) {

        switch (QMessageBox::warning (this, "Average Shape Calibration",
                                      "Do you want to reset before new average waveform shape calibration? "
                                      "Only enabled channels are affected.", "&Yes", "&No", "Cancel", 0, 2)) {
        case 0:
        {
            CalAvgConfig newConfig(d->cfg.dec.calavg);
            newConfig.clear_enabled(d->cfg.adc);
            setCalAvgConfig(newConfig);
        }
            break;
        case 1:
            break;
        default:
            return;
        }
    }
#endif
    if (cacheDecoderConfig.mode != mode) {
        cacheDecoderConfig.mode = mode;
        configManager->setDecoderConfig(cacheDecoderConfig);
    }

    // set run mode
    adcm_runmode_t newRunMode = (cacheDecoderConfig.mode == DM_ZERO) ? ADCM_RUNMODE_BASECAL : ADCM_RUNMODE_RUN;
    AdcConfig adcConfig(cacheAdcConfig);
    adcConfig.runmode = newRunMode;
    configManager->setAdcConfig(adcConfig);
}
