
#include <cstdio>
#include <cstddef>
#include <cerrno>

#ifdef __USE_POSIX
#include <unistd.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <linux/if.h>
#endif

#include "sysinfo.h"

sysinfo::sysinfo()
{
    valid = false;
}

QString sysinfo::text() const
{
    check_cache();
    QString s;
    //  QString sharename;
    //  sharename = QString("\\\\")+ip+QString("\\data");
    /*
  s = QString("<p>");
  s += QString("<table>");
  s += QString("<tr><td>IP</td><td>")+ip+QString("</td></tr>");
  s += QString("<tr><td>MAC</td><td>")+mac+QString("</td></tr>");
  s += QString("</table>");
  s += QString("</p>\n");
*/
    //  s = QString("<div align=center><pre>MAC %1\nIP %2</pre></div>").arg(mac, ip);
    s = QString("MAC %1\nIP %2").arg(mac, ip);
    //  qDebug(s);
    return s;
}

void sysinfo::check_cache() const
{
    if (!valid) update();
}

// Get hostname, IP, MAC
void sysinfo::update () const
{
    valid = true;

#ifdef __USE_POSIX

    int sock, ret;
    struct ifreq ifr;
    char macstr[18];
    char ipstr[16];
    struct sockaddr_in *sin_ptr;
    char hostnamestr[1024];


    gethostname (hostnamestr, 1024);
    hostname = hostnamestr;

    sock = socket (PF_INET, SOCK_DGRAM, IPPROTO_IP);
    if (!sock)
        return;
    strcpy (ifr.ifr_name, "eth0");
    ret = ioctl (sock, SIOCGIFADDR, &ifr);
    //    printf("test eth0: %d, errno=%d\n", ret, errno);
    // if IP is not available, try next one
    if (ret && (errno == EADDRNOTAVAIL || errno == ENODEV)) {
        strcpy (ifr.ifr_name, "eth1");
        ret = ioctl (sock, SIOCGIFADDR, &ifr);
        //    printf("test eth1: %d, errno=%d\n", ret, errno);
    }
    if (ret && (errno == EADDRNOTAVAIL || errno == ENODEV)) {
        strcpy (ifr.ifr_name, "eth2");
        ret = ioctl (sock, SIOCGIFADDR, &ifr);
        //    printf("test eth2: %d, errno=%d\n", ret, errno);
    }
    // if IP is still not available, it may be the case that bridge configuration is activated.
    if (ret && (errno == EADDRNOTAVAIL || errno == ENODEV)) {
        strcpy (ifr.ifr_name, "br0");
        ret = ioctl (sock, SIOCGIFADDR, &ifr);
        //    printf("test br0: %d, errno=%d\n", ret, errno);
    }
    if (ret) {
        fprintf (stderr, "get_inet_param: ioctl(SIOCGIFADDR, %s) failed: %d, %s\n", ifr.ifr_name, errno, strerror (errno));
        //    goto errclose;
        // don't go error, just do nothing if interface is not found.
    } else {
        sin_ptr = (struct sockaddr_in *) &ifr.ifr_addr;
        unsigned int ipsum = sin_ptr->sin_addr.s_addr;
        sprintf (ipstr, "%d.%d.%d.%d", ipsum & 0xFF, (ipsum >> 8) & 0xFF, (ipsum >> 16) & 0xFF, ipsum >> 24);
        ip = ipstr;
        if (ioctl (sock, SIOCGIFHWADDR, &ifr)) {
            fprintf (stderr, "get_inet_param: ioctl(SIOCGIFHWADDR) failed: %s\n", strerror (errno));
            //      goto errclose;
        } else {
            unsigned char *ptr = (unsigned char *) ifr.ifr_ifru.ifru_hwaddr.sa_data;
            sprintf (macstr, "%02X%02X%02X%02X%02X%02X", *ptr, *(ptr + 1), *(ptr + 2), *(ptr + 3), *(ptr + 4), *(ptr + 5));
            mac = macstr;
        }
    }
    close (sock);
    //  printf("MAC: %s\n", mypar.mac);
    //  printf("IP: %s\n", mypar.ip);
#endif
}
