/*
**    Copyright 2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "th1_gui.h"

#include <QDebug>
#include <QFileDialog>
#include <QSettings>
#include <QDateTime>
#include <QRegExp>
#include <QSettings>
#include <QTextStream>
#include <QDir>
#include <QFile>

#include "dsplib/th1.h"

bool th1_gui::save_to_file (const th1 &h, QWidget *parent, const QString &dir)
{
    QString subdir;
    if (dir.isNull()){
        subdir = QDateTime::currentDateTime().toString("yyyy-MM-dd_hh-mm-ss");
    } else {
        subdir = dir;
    }

    QSettings settings;
    settings.beginGroup("Histograms");
    QString allHistDirPath = settings.value("dir").toString();
    if(allHistDirPath.isEmpty()){
        allHistDirPath = QFileDialog::getExistingDirectory(parent, "Open Histograms Directory","",
                                                           QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
        if(allHistDirPath.isEmpty())
            return false;
        else
            settings.setValue("dir", allHistDirPath);
    }

    QDir histDir;
    if(!subdir.isEmpty()){
        histDir = QDir(allHistDirPath+QDir::separator()+subdir);
    } else {
        histDir = QDir(allHistDirPath);
    }

    if(!histDir.exists() && !histDir.mkpath(".")){
        qWarning()<<"Can't create directory "<<histDir.absolutePath();
        return false;
    }

    QString filename(h.getname()+".txt");
    QRegExp rx("[/]");
    filename.replace(rx, "_");
    filename = histDir.absolutePath()+QDir::separator()+filename;

    QFile f(filename);
    if (!f.open(QIODevice::WriteOnly)) {
        qWarning() << QString("Can't write histogram '%1' to file '%2'")
                          .arg(h.getname())
                          .arg(filename);
        return false;
    }
    QTextStream ts(&f);
    //    ts.setEncoding(QTextStream::UnicodeUTF8);
    auto py = h.get_py();
    for (unsigned int i = 0; i < h.get_nbins(); i++)
        ts << h.bincenter(i) << "\t" << py[i] << "\n";

    f.close();
    return true;
}
