/*
**    Copyright 2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "ProgramArguments.h"

#include <QCommandLineParser>
#include <QTextStream>

ProgramArguments::ProgramArguments()
{

}

void ProgramArguments::parseCommandLine()
{
    QCommandLineParser parser;
    parser.setApplicationDescription(QCoreApplication::applicationName());
    const QCommandLineOption helpOpt = parser.addHelpOption();

    QCommandLineOption debugOpt({"d", "debug"} ,
                                     "set debug level",
                                     "0");
    parser.addOption(debugOpt);

    QCommandLineOption versionOpt({"v", "version"} ,
                                "print version and exit");
    parser.addOption(versionOpt);

    QCommandLineOption noautoOpt({"n", "noauto"} ,
                                  "Disable automatic run start");
    parser.addOption(noautoOpt);

    QCommandLineOption configOpt({"c", "confpath"} ,
                                 "Set configuration directory");
    parser.addOption(configOpt);

    QCommandLineOption deviceOpt({"m", "dev"} ,
                                 "Set mlink device file");
    parser.addOption(deviceOpt);

//    QCommandLineOption bufsizeOpt({"b", "bufsize"} ,
//                                 "Set buffer size, MiB");
//    parser.addOption(bufsizeOpt);

    parser.process(QCoreApplication::arguments());

    if (parser.isSet(helpOpt)) {
        show_help_and_exit = true;
    }
    if (parser.isSet(versionOpt)) {
        print_version_and_exit = true;
    }
    if (parser.isSet(debugOpt)) {
        bool ok = false;
        quint32 v = parser.value(debugOpt).toUInt(&ok);
        if (ok) {
            debug = v;
        } else {
            show_help_and_exit = true;
        }
    }
    if (parser.isSet(noautoOpt)) {
        autorun = false;
    }
    if (parser.isSet(configOpt)) {
        config = parser.value(configOpt);
    }
//    if (parser.isSet(bufsizeOpt)) {
//        // TODO
//    }
    if (show_help_and_exit) {
        parser.showHelp();
        exit(2);
    }
}
