/*
**    ADC low-level interface library
**
**    Copyright 2003-2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef ADCMDEVICE_H
#define ADCMDEVICE_H

#include "regio/mlinkdevice.h"
#include "adcmraw/adcmregs.h"
#include "adcm16.h"
#include "adcmraw/adcm_mem.h"
#include "conf_runtime.h"
#include "config/calzeroconfig.h"
#include "config/adcconfig.h"

class CalZeroConfig;
class AdcHwCounters;
namespace mlink {
class RawPacketList;
}
class AdcmDevice: protected mlink::MlinkDevice, protected AdcmRegs
{
public:
    static bool decomp_recwin(unsigned int recwin, unsigned int *flen, unsigned int *fcnt, unsigned int flen_max, unsigned int fcnt_max);

    explicit AdcmDevice(QSharedPointer<mlink::RegIOMLink> adcmLink);
    QString getDeviceIdStr() const;
    // General Control and Status
    void adc_reset();
    bool adc_start();
    bool adc_stop();
    QString readFirmwareIds();
    void setClock();
//    int adc_get_testpass(adcm_testpass_t *fpass, adcm_testpass_t *ffail) const;
    void setBufferSize();
    void poll();
    mlink::RawPacketList popDataPackets();
    //    qint64 read(char * data, qint64 maxBytes);
    unsigned int adc_dma_cycle () const;
    int adc_nrun () const;              // cycle count

//    void setCalZero(const CalZero &r) { if (r == calZero) return; calZero = r; }
//    void setAdcConfig(const AdcConfig &r) { if (r == adcConfig) return; adcConfig = r; }
    void setup(const CalZeroConfig &_calZero, const AdcConfig &_adcConfig);

    AdcHwCounters adc_get_counters();
    QVector<double> readTemperatures();

    int nch() const;

    static float adc_gain_val_to_db(int value);
    static int adc_gain_val_to_raw(float db_value);

    conf_mem_t getConfMem() const { return conf_mem; }

protected:

    conf_mem_t discover();
    void writeClock(quint64 value);
    quint64 readClock();
    double ADC_TS_FREQ() const;

    typedef enum { ADC_OFF = 0, ADC_READY = 2, ADC_RUN = 3, ADC_ERROR = 4 } adc_state_t;

    static const char *adc_state_str (int st);

    bool isRunning() const;
    int state_online() const;
    void adc_setreg_inrun(int reg, int data);
    void adc_write_ctrl_reg();
//    bool test_regs();
    void clearcounts ();

    // per-channel settings
    void adc_setgain (int ch, float gain); // dB
    void adc_setthr (int ch, int thr);
    void adc_sethyst (int ch, int hyst);
    void adc_settrig (int ch, adcm_channel_flags_t trig);

    // General settings
    //    void adc_setrunmode (adcm_runmode_t runmode);
    void adc_settrigbits (adcm_trigbit_t trigbits);
    void adc_setwindow (int searchback, int searchwin, int latency);
    void set_timestamp_is_utc(bool value);
    void adc_set_trig_mult (unsigned int mult_min, unsigned int mult_max);
    void adc_set_fragm_len (unsigned int fragm_len);
    void adc_set_fragm_count (int ch, unsigned int fragm_count);
    int adc_get_fragm_len_max (unsigned int *fragm_len_max) const;
    int adc_get_fragm_count_max (unsigned int *fragm_count_max) const;

    conf_mem_t read_adc_setup_memory();

    CalZeroConfig calZero;
    AdcConfig adcConfig;
    adc_state_t ast;
    //    const unsigned int *dma_buf;   // address
    //    unsigned long dma_size;     // size, bytes
    //    unsigned long dma_used;     // bytes, dma buffer usage
    //    unsigned long prev_dma_used;
    unsigned long nrun;         // run number
    conf_runtime_t conf_runtime;
    qint64 ts_write_ts;
    conf_mem_t conf_mem;
    quint32 ts_free_prev; // prev values of HW Counters
    quint32 ts_busy_prev; // for free/busy time
};

#endif /* ADCMDEVICE_H */
