/*
**    Copyright 2010-2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef _ADCM_CHBUF_IMPL_H
#define _ADCM_CHBUF_IMPL_H

#include <vector>
#include <cstddef>
#include <cstdint>
#include "adcm16.h"

class QTextStream;

typedef int16_t AdcmCbufSample;
typedef std::vector<AdcmCbufSample> AdcmCbufWaveform;

typedef int32_t AdcmConvSample;
typedef std::vector<AdcmConvSample> AdcmConvWaveform;

struct adcm_peak_t {
    //    void clear() { *this = adcm_peak_t(); }
    double pt_coarse = 0;            // coarse peak time
    double pa_coarse = 0;            // coarse peak amplitude
    double pw_coarse = 0;            // coarse peak width
    double fasttime = 0;             // peak coarse time
    double fastnorm = 0;             // coarse time fit norm
    double pt = 0;                   // peak fine time
    double pa = 0;                   // peak amplitude
    double pw = 0;                   // peak width
    double pchi2 = 0;                // normalized by a and len
    int pniter = 0;
    double *w = nullptr;  // pointer to cbuf.w[?]
    size_t wlen = 0;
};

typedef std::vector<adcm_peak_t> adcm_peakbuf_t;

struct AdcmConvolvedData {
    void clear() { *this = AdcmConvolvedData(); }
    AdcmConvWaveform w;
    adcm_peakbuf_t peaks;
    int offs = 0;  // FIR offset
    int i1 = 0;
    int i2 = 0;    // peak search range
//    int ip = 0;    // peak position
//    double ap = 0; // peak amplitude
};

typedef union {
    uint32_t all;
    struct {
        unsigned int finished: 1;
        unsigned int has_data: 1;
        unsigned int has_thr: 1;
        unsigned int has_clip: 1;
        unsigned int has_pu: 1;
        unsigned int dt_cut: 1;
        unsigned int unused_bit_6: 1;
        unsigned int eval_sine: 1;

        unsigned int eval_thr: 1;
        unsigned int eval_bl: 1;
        unsigned int eval_int: 1;
        unsigned int eval_clip: 1;
        unsigned int eval_pu: 1;
        unsigned int eval_tfine: 1;
        unsigned int eval_rms: 1;
        unsigned int eval_tfast: 1;

        unsigned int unused_bits_16_31: 16;
    } bit;
} adcm_cbuf_flags_t;

typedef union {
    uint32_t all;
    struct {
        unsigned int unused_bits_0_16: 16;
        unsigned int bl_nodata: 1;
        unsigned int int_nodata: 1;
        unsigned int bl_out: 1;
        unsigned int pks_nodata: 1;
        unsigned int tfit_err: 1;
        unsigned int tfit_off: 1;
        unsigned int bad_len: 1;
        unsigned int unused_bit_23: 1;
        unsigned int unused_bits_24_31: 8;
    } bit;
} adcm_cbuf_error_flags_t;

QTextStream &operator<<(QTextStream &stream, const adcm_cbuf_flags_t &flags);
QTextStream &operator<<(QTextStream &stream, const adcm_cbuf_error_flags_t &f);

class adcm_chbuf_t
{
public:
    void clear() {
        flags.all = 0;
        ch = 0;
        ts = 0;
        tp = 0;
        b = 0;
        i = 0;
        dc = 0;
        rms = 0;
        w_bits = 0;
        w_latency = 0;
        w_calzero = 0;
        w.resize(0);
        conv.clear();
    }
    size_t len() const { return w.size(); }

    adcm_channel_flags_t trig = {};
    adcm_cbuf_flags_t flags = {};
    adcm_cbuf_error_flags_t error_flags = {};
    unsigned int ch = 0; // channel number
    uint64_t ts = 0;
    // decoded parameters
    int tp = 0;            // threshold position
    float b = 0;                    // baseline
    float i = 0;                    // not integral, but mean value of signal
    float dc = 0;
    float rms = 0;
    unsigned short w_bits = 0;
    int w_latency = 0;
    float w_calzero = 0;
    AdcmCbufWaveform w; // input signal
    AdcmConvolvedData conv; // convolved
};

#endif /* _ADCM_CHBUF_IMPL_H */
