/*
**    Copyright 2006-2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef ADCM_DECSTAT_H
#define ADCM_DECSTAT_H

#include <QtCore/QtGlobal>
#include <QMetaType>
#include <array>

#include "adcm16.h"
#include "decoder/decst_t.h"

class QTextStream;
class QDebug;

struct decst_prof_t {
    decst_prof_t();
    void clear();
    static const char *label(decst_items_t n);
    decst_prof_t &operator+=(const decst_prof_t &r);
//    QString toString() const;
    decst_t_t ta = {}; // average
    decst_t_t tm = {}; // minimum
    decst_t_t tl = {}; // last event
    unsigned int n = 0;
    friend QTextStream &operator<<(QTextStream &s, const decst_prof_t &prof);
//    friend QDebug operator<<(QDebug s, const decst_prof_t &st);
};

typedef quint64 stcnt_t;

// accumulated channel statistics
struct adcm_chstat_t {
    adcm_chstat_t &operator+=(const adcm_chstat_t &r);

    std::array<quint64, ADCM_N_CH> all = {};   // total pulses
    std::array<quint64, ADCM_N_CH> blnd = {};  // ADCM_CH_BL_NODATA
    std::array<quint64, ADCM_N_CH> intnd = {}; // ADCM_CH_INT_NODATA
    std::array<quint64, ADCM_N_CH> blout = {}; // ADCM_CH_BL_OUT baseline out of limits
    std::array<quint64, ADCM_N_CH> clip = {};  // ADCM_CH_EVAL_CLIP clipping detected
    std::array<quint64, ADCM_N_CH> pu = {};    // ADCM_CH_EVAL_PU pile-up detected
};

class adcm_decstat_t
{
public:
    void clear();
    quint64 n_frame_err() const;
    quint64 n_data_err() const;
    quint64 n_dsp_err() const;
    adcm_decstat_t &operator+=(const adcm_decstat_t &r);

    // find_frame_forw():
    quint64 nw = 0;            // number of words processed
    quint64 overruns = 0;
    unsigned int lastpos = 0;       // last processed word index (not byte!)
    quint64 n_resync = 0;      // number of stream resyncronisations (bad data skipped)
    quint64 nf_crc = 0;        // frames with crc error
    quint64 nf_seq = 0;        // frames with sequence error (lost frames?)
    quint64 nf = 0;            // total number of frames found
    quint64 nf_echo = 0;       // echo frames (type 0)
    quint64 nf_dadc = 0;       // data frames (type 0x100)
    quint64 nf_creq = 0;       // control request frames (type 0x101)
    quint64 nf_cack = 0;       // control acknowledge frames (type 0x102)
    quint64 nf_stat = 0;       // status frames (type 0x103)
    quint64 nf_unkn = 0;       // unknown frame type
    unsigned int praw_err = 0;      // error position, rel. 0
    unsigned int pf_err = 0;        // error position, rel. to frame (data)

    // adcm_decode_data_blocks():
    quint64 nblk_all = 0;      // total data blocks
    quint64 nblk_adc_0 = 0;      // total adc data blocks (type 0)
    quint64 nblk_adc_1 = 0;      // total adc data blocks (type 1)
    quint64 nblk_adc_2 = 0;      // total adc data blocks (type 2)
    //    quint64 nblk_stat;     // total statistical data blocks
    quint64 nblk_bad = 0;      // bad data blocks (corrupted data)
    quint64 nblk_unkn = 0;     // data blocks with unknown data type id
    quint64 nblk_crc = 0;      // data blocks with bad CRC
    unsigned int pblk_err = 0;      // error position, rel. to data block

    quint64 nblk_adc_len = 0;  // adc data blocks with length error
    quint64 nblk_adc_roll = 0; // adc data blocks with roll error
    quint64 nblk_adc_dup = 0;  // adc data blocks with duplicate channel data
    quint64 nblk_adc_evnum = 0;// adc data blocks with bad event number
    quint64 nblk_adc_ok = 0;   // good adc data blocks

    // Signal processing
    quint64 dsp_total = 0;     // total pulses processed (0 if not decoding pulses)
    quint64 dsp_good_waveform = 0;     // total waveforms processed (always nonzero)
    quint64 dsp_bad_len = 0;   // waveform length error
    quint64 dsp_thr_nf = 0;    // threshold position expected but not found
    quint64 dsp_bl_range = 0;  // baseline insuff. data
    quint64 dsp_int_range = 0; // integral insuff. data
    quint64 dsp_clip_pos = 0;  // positive clipping
    quint64 dsp_clip_neg = 0;  // negative clipping
    quint64 dsp_pk_range = 0;  // peak search range bad
    quint64 dsp_fit_fail = 0;  // fit failed
    quint64 dsp_fit_bad = 0;   // fit bad
    quint64 dsp_fit_ok = 0;    // fit ok
    quint64 dsp_sine_ok = 0;   // sine fit ok

    // Event manager
    quint64 ev_total = 0;    // total events processed
    quint64 ev_dt_cut = 0;    // event dropped by dt cut

    // messages
    quint64 frame_errors_suppressed = 0;
    quint64 data_errors_suppressed = 0;
    quint64 dsp_errors_suppressed = 0;

    // profiling timers
    struct decst_prof_t prof;

    // channel statistics
    struct adcm_chstat_t cs;

    friend QTextStream &operator<<(QTextStream &s, const adcm_decstat_t &st);
//    friend QDebug operator<<(QDebug s, const adcm_decstat_t &st);
};

Q_DECLARE_METATYPE(adcm_decstat_t)

//  void print_decoder_frame_err (adcm_decstat_t * decst, char *fmt, ...);
//  void print_decoder_data_err (adcm_decstat_t * decst, char *fmt, ...);
//  void print_decoder_dsp_err (adcm_decstat_t * decst, char *fmt, ...);

//void decst_print_prof (const struct decst_prof_t &p);

class AdcmEvent;

void decst_add_ev_chstat (adcm_decstat_t * decst, const AdcmEvent &ev);

#endif // ADCM_DECSTAT_H
