#ifndef ADCMREGS_H
#define ADCMREGS_H

#include <QtGlobal>

class AdcmRegs {

public:

    static unsigned int adc_reg_ch_thr (unsigned int ch);
    static unsigned int adc_reg_ch_hyst (unsigned int ch);
    static unsigned int adc_reg_ch_fragm (unsigned int ch);
    static unsigned int adc_reg_ch_fragmlen (unsigned int ch);
    static unsigned int adc_reg_ch_ctrl (unsigned int ch);
    static unsigned int adc_reg_ch_ctrl_s (unsigned int ch);
    static unsigned int adc_reg_ch_auxctrl (unsigned int ch);
    static unsigned int adc_reg_ch_cnt_compat (unsigned int ch);
    static unsigned int adc_reg_ch_cnt (unsigned int ch);
    static unsigned int adc_reg_status (int board);

};

const size_t ADCM_BOARD_NREG = 512;

enum {
    // Control registers
    ADCM16_CTRL_REG           = 0x40,
    ADCM16_CTRL_DEFAULT     = 0x0000,

    ADCM16_WIN_REG            = 0x41,
    ADCM16_STATUS_REG         = 0x42, /* read-only */
    ADCM16_WIN_DEFAULT      = 0xFFFF,

    ADCM16_TRIGMULT_REG       = 0x43,
    ADCM16_FRAGMLEN_REG       = 0x44,
    ADCM16_FRAGMMAX_REG       = 0x45, /* read-only */
    ADCM16_SERIAL_ID_HI_REG   = 0x46, /* read-only */
    ADCM16_MAX_NFRAG_REG      = 0x47, /* read-only */


    ADCM16_EXT_LAT_REG  = ADCM16_WIN_REG,
    ADCM16_EXT_MATCHWIN_REG   = 0x48,
    ADCM16_EXT_PREWIN_REG     = 0x49,
    ADCM16_CH_DPM_KS_REG      = 0x4A, /* read-only */
    ADCM16_TEMPERATURE_REG    = 0x4B, /* read-only */
    ADCM16_FW_VER_REG         = 0x4C, /* read-only */
    ADCM16_FW_REV_REG         = 0x4D, /* read-only */
    ADCM16_SERIAL_ID_REG      = 0x4E, /* read-only */
    ADCM16_PROPERTIES_REG     = 0x4F, /* read-only */

    // Timestamp registers
    ADCM16_TS_A_REG64         = 0x50,
    ADCM16_TS_SET_REG64       = 0x54,
    ADCM16_TS_B_REG64         = 0x58,
    ADCM16_TS_READ_REG64      = 0x5C
};

typedef union {
    quint16 all;
    struct {
        unsigned int reserved1 : 1;
        unsigned int timestamp_is_utc : 1;
        unsigned int reserved : 8;
        unsigned int trigbits : 4;
        unsigned int test : 1;
        unsigned int run  : 1;
    } __attribute__ ((packed)) bit;
} adcm_ctrl_reg_t;

typedef union {
    quint16 all;
    struct {
        unsigned int searchback : 5;
        unsigned int searchwin : 6;
        unsigned int latency : 5;
    } __attribute__ ((packed)) bit;
} adcm_win_reg_t;

typedef union {
    unsigned int all;
    struct {
        unsigned int serial:8;
        unsigned int id:8;
    } __attribute__ ((packed)) bit;
} adcm_status_reg_t;

typedef union {
    quint16 all;
    struct {
        unsigned int cmpen: 1;
        unsigned int s: 1;
        unsigned int m: 1;
        unsigned int test: 1;
        unsigned int zero : 1;
        unsigned int invert : 1;
        unsigned int unused : 2;
        unsigned int gain : 8;
    } __attribute__ ((packed)) bit;
} adcm_chctrl_reg_t;

#endif /* ADCMREGS_H */
