/*
**    Copyright 2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef TREEELEMENT_H
#define TREEELEMENT_H

#include <QList>

class TreeElement
{
public:
    TreeElement(const TreeElement &) = delete;
    TreeElement(const TreeElement &&) = delete;
    TreeElement &operator=(const TreeElement &) = delete;
    TreeElement &operator=(const TreeElement &&) = delete;
    explicit TreeElement(TreeElement *parent = nullptr)
        : _parent(parent) {
        if (parent) {
            parent->addChild(this);
        }
    }
    virtual ~TreeElement() {
        if (parent()) {
            parent()->removeChild(this);
        }
    }
    void addChild(TreeElement *p) {
        _children.push_back(p);
    }
    void removeChild(TreeElement *p) {
        _children.removeOne(p);
    }
    QList<TreeElement *> children() const { return _children; }
    TreeElement *parent() const { return _parent; }
private:
    TreeElement *_parent = nullptr;
    QList<TreeElement *> _children;
};

#endif // TREEELEMENT_H
