//
//    Copyright 2013-2019 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef ADCCONFIG_H
#define ADCCONFIG_H

#include "adcm16.h"
#include <QVector>
#include <QMap>
#include <QMetaType>

class QTextStream;
class ConfigStatus;

typedef enum {
    ADCM_RUNMODE_RUN  = 0x8000u,
    ADCM_RUNMODE_BASECAL = 0x4000u
} adcm_runmode_t;

class AdcChannelConfig
{
public:
    AdcChannelConfig();
    bool operator==(const AdcChannelConfig &other) const;
    bool operator!=(const AdcChannelConfig &other) const {return !operator==(other);}
    double gain = 0;         //!< preamp gain, dB
    double thr = 50;          //!< digital threshold, mV
    adcm_channel_flags_t flags;   //!< bit field
//    unsigned int recwin = 120; //!< full recording window (incl. prewin), step-ns clocks

    friend QTextStream &operator<<(QTextStream &s, const AdcChannelConfig &a);
};
Q_DECLARE_METATYPE(AdcChannelConfig)

QTextStream &operator<<(QTextStream &s, const adcm_channel_flags_t &flags);

using AdcChannelConfigMap = QMap<int, AdcChannelConfig>;
//Q_DECLARE_METATYPE(AdcChannelConfigMap)

class conf_mem_t;
class AdcConfig
{
public:
    static AdcChannelConfigMap defaultChannels();

    void resize(int n);
    ConfigStatus isValid() const;
    bool operator==(const AdcConfig &other) const;
    bool operator!=(const AdcConfig &other) const {return !operator==(other);}
    bool isBaselineCalibrationMode() const;
//    static double mv_to_value_scaler ();
    double thr_mv_min () const;
    double thr_mv_max () const;
    void setThr(int ch, double value);
    void setGain(int ch, double value);
    int adc_rawdata_maxval(int ch) const;
    int adc_rawdata_midval(int ch) const;
    int adc_rawdata_minval(int ch) const;
    bool is_testmode() const;
    bool is_co_trigger() const;
    bool is_ext_trigger() const;
    bool is_simple_trigger() const;
    bool ch_enabled_offset_calibration (int ch) const;
    bool ch_enabled_shape_calibration (int ch) const;
    bool ch_enabled_reconstruction (int ch) const;
//    unsigned int adc_cfg_get_recwin_max() const;
//    void adjust_adc_recwin();
    bool cfg_channel_may_have_data(int ch) const;
    int fragm_count(int ch) const;
    int recwin(int ch) const;

    AdcChannelConfigMap channels = defaultChannels();
    int bits = 2;                     //!< ADC resolution, bits
    double step = 1;                  //!< ADC time step, ns
    int prewin = 20;                   //!< pre-threshold recording window, step-ns clocks
    int matchwin = 40;                 //!< coincidence match window, step-ns clocks
    int latency = 30;                  //!< data latency, step-ns clocks
    adcm_runmode_t runmode = ADCM_RUNMODE_RUN;       //!< run mode bit field
//    bool run_enable;              //
    adcm_trigbit_t trigbits = { ADCM_TRIGBIT_INT };// ADCM_TRIGBIT_INT; // FIXME: use bit fields       //!< trigger bits
    unsigned int mult_min = 1;        //!< minimal multiplicity
    unsigned int mult_max = ADC_NCH;        //!< maximal multiplicity
    unsigned int fragm_len = 120;
    unsigned int fragm_count_m = 1;
    unsigned int fragm_count_s = 1;
//    unsigned int fragm_len_max = 120;
//    unsigned int fragm_count_max = 1;
//    unsigned int ch_dpm_ks = 8;       //!< channel ring memory size in kilosamples
    unsigned int timestamp_is_utc = 1;         //!< no timestamp reset on cycle start, use UTC time

protected:
//    void updateTriggerBits();
    friend QTextStream &operator<<(QTextStream &s, const AdcConfig &adcConfig);
};

Q_DECLARE_METATYPE(AdcConfig)

#endif // ADCCONFIG_H
