//
//    Copyright 2013 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef ALLCONFIG_H
#define ALLCONFIG_H

#include <QMetaType>

#include "adcconfig.h"
#include "decoderconfig.h"
#include "histconfig.h"
#include "runconfig.h"
#include "storageconfig.h"
#include "ConfigStatus.h"

class AllConfig
{
public:
    AllConfig();
    ConfigStatus isValid() const;
    bool operator==(const AllConfig &other) const;
    bool operator!=(const AllConfig &other) const {return !operator==(other);}

    double calavg_integrate(int ch, int start, int end) const;
    double calavg_baseline(unsigned int ch) const;
    double calavg_integral(unsigned int ch) const;
    double calavg_preintegral(unsigned int ch) const;
    double calavg_postintegral(unsigned int ch) const;

//    bool enabled_ch_has_calzero() const;
    bool enabled_ch_has_calavg () const;
    bool enabled_ch_has_fir () const;

    static int thr_mv_to_value (const CalZeroConfig &calZero, const AdcConfig &adc, unsigned int ch, double mv);
//    int thr_mv_to_value (unsigned int ch, double mv) const;
//    double thr_value_to_mv (unsigned int ch, unsigned int val) const;

    static int get_dec_fitside(double step_ns, const DecoderConfig &dec, int ch);

//    int dt_cut_good(double dt) const;

    int version;                //!< file version
    AdcConfig adc;
    StorageConfig stor_raw;
    StorageConfig stor_dev;
    DecoderConfig dec;
    HistConfig hist;
    RunConfig run;
};

Q_DECLARE_METATYPE(AllConfig)

#endif // ALLCONFIG_H
