/*
**    Copyright 2013 Ivan Filippov
**    Copyright 2013-2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "inibased.h"

#include <QDebug>

IniBased::IniBased(IniBased *parent, const QString &section)
    : TreeElement(parent),
    sectionName(section)
{
    while (sectionName.endsWith("/"))
        sectionName.chop(1);
}

void IniBased::writeValue(QSettings *settings, const QString &key, const QVariant &data) const
{
//    bool empty = false;
//    switch (data.type()) {
//    case QMetaType::QVariantList:
//        empty |= data.canConvert(QMetaType::QVariantList) && data.toList().isEmpty();
//        break;
//    case QMetaType::QVariantMap:
//        empty |= data.canConvert(QMetaType::QVariantMap) && data.toMap().isEmpty();
//        break;
//    default:;
//    }
//    if (empty)
//        settings->setValue(path() + key, "");
//    else
        settings->setValue(path() + key, data);
}
#if 0
void IniBased::writeValue(QSettings *settings, const QString &key, const QVector<int> &data) const
{
    //    qDebug() << objectName() << "writeValue" << "path:" << path() << "key:" << key << "data:" << data;
    QStringList sl;
    for (auto & item : data) {
        sl.push_back(QString::number(item));
    }
    settings->setValue(path() + key, sl);
}

void IniBased::writeValue(QSettings *settings, const QString &key, const QVector<unsigned int> &data) const
{
    QStringList sl;
    for (auto & item : data) {
        sl.push_back(QString::number(item));
    }
    settings->setValue(path() + key, sl);
}

void IniBased::writeValue(QSettings *settings, const QString &key, const QVector<double> &data) const
{
    QStringList sl;
    for (auto & item : data) {
        sl.push_back(QString::number(item, 'e', 16));
    }
    settings->setValue(path() + key, sl);
}
#endif
#if 0
void IniBased::readValue(const QSettings *settings, const QString &key, bool &data)
{
    const QString &fullKey = path() + key;
    if (!settings->contains(fullKey))
        return;
    auto tmp = settings->value(fullKey, data);
    if (tmp.isValid())
        data = tmp.toBool();
}

void IniBased::readValue(const QSettings *settings, const QString &key, qint32 &data)
{
    const QString fullKey = path() + key;
    if (!settings->contains(fullKey))
        return;
    bool ok;
    auto tmp = settings->value(fullKey, data).toInt(&ok);
    if (ok)
        data = tmp;
}

void IniBased::readValue(const QSettings *settings, const QString &key, quint32 &data)
{
    const QString &fullKey = path() + key;
    if (!settings->contains(fullKey))
        return;
    bool ok;
    auto tmp = settings->value(fullKey, data).toUInt(&ok);
    if (ok)
        data = tmp;
}

void IniBased::readValue(const QSettings *settings, const QString &key, quint64 &data)
{
    const QString &fullKey = path() + key;
    if (!settings->contains(fullKey))
        return;
    bool ok;
    auto tmp = settings->value(fullKey, data).toULongLong(&ok);
    if (ok)
        data = tmp;
}

void IniBased::readValue(const QSettings *settings, const QString &key, double &data)
{
    const QString &fullKey = path() + key;
    if (!settings->contains(fullKey))
        return;
    bool ok;
    auto tmp = settings->value(fullKey, data).toDouble(&ok);
    if (ok)
        data = tmp;
}

void IniBased::readValue(const QSettings *settings, const QString &key, QString &data)
{
    const QString &fullKey = path() + key;
    if (!settings->contains(fullKey))
        return;
    auto tmp = settings->value(fullKey, data);
    if (tmp.isValid())
        data = tmp.toString();
}

void IniBased::readValue(const QSettings *settings, const QString &key, QStringList &data)
{
    const QString &fullKey = path() + key;
    if (!settings->contains(fullKey))
        return;
    auto tmp = settings->value(fullKey, data);
    if (tmp.isValid())
        data = tmp.toStringList();
}

void IniBased::readValue(const QSettings *settings, const QString &key, QVariant &data)
{
    const QString &fullKey = path() + key;
    if (!settings->contains(fullKey))
        return;
    auto tmp = settings->value(fullKey, data);
    if (tmp.isValid())
        data = tmp;
}

void IniBased::readValue(const QSettings &settings, const QString &key, QVector<int> &data)
{
    const QString &fullKey = path() + key;
    if (!settings.contains(fullKey))
        return;
    QStringList list;
    auto tmp = settings.value(fullKey, list);
    if (!tmp.isValid())
        return;
    list = tmp.toStringList();

    QVector<int> v;
    for (auto & s : list) {
        bool ok;
        v.push_back(s.toInt(&ok));
        if (!ok)
            return;
    }
    data = v;
}

void IniBased::readValue(const QSettings &settings, const QString &key, QVector<unsigned int> &data)
{
    const QString &fullKey = path() + key;
    if (!settings.contains(fullKey))
        return;
    QStringList list;
    auto tmp = settings.value(fullKey, list);
    if (!tmp.isValid())
        return;
    list = tmp.toStringList();

    QVector<unsigned int> v;
    for (auto & s : list) {
        bool ok;
        v.push_back(s.toUInt(&ok));
        if (!ok)
            return;
    }
    data = v;
}

void IniBased::readValue(const QSettings &settings, const QString &key, QVector<double> &data)
{
    const QString &fullKey = path() + key;
    if (!settings.contains(fullKey))
        return;
    QStringList list;
    auto tmp = settings.value(fullKey, list);
    if (!tmp.isValid())
        return;
    list = tmp.toStringList();

    QVector<double> v;
    for (auto & s : list) {
        bool ok;
        v.push_back(s.toDouble(&ok));
        if (!ok)
            return;
    }
    data = v;
}
#endif

void IniBased::save(QSettings *settings) const
{
    writeToIni(settings);
    for (auto i : children()) {
        const auto *p = dynamic_cast<const IniBased *>(i);
        Q_ASSERT(p);
        if (p)
            p->save(settings);
    }
}

void IniBased::load(const QSettings &settings)
{
    readFromIni(settings);
    for (auto i : children()) {
        auto *p = dynamic_cast<IniBased *>(i);
        Q_ASSERT(p);
        if (p)
            p->load(settings);
    }
}

QStringList IniBased::childKeys(const QSettings &settings, const QString &key) const
{
    QStringList list;
    QString arrayPath = path() + key + "/";
    while(arrayPath.startsWith("/"))
        arrayPath.remove(0, 1);
    while (arrayPath.endsWith("/"))
        arrayPath.chop(1);

    QString arrayPathSlash = arrayPath;
    if (!arrayPathSlash.endsWith("/"))
        arrayPathSlash += "/";
    QStringList allKeys = settings.allKeys();
    for (auto & k : allKeys) {
        if (!k.startsWith(arrayPathSlash))
            continue;
        QString rhs = k.section(arrayPathSlash, 1, 1);
        QString indexStr = rhs.section("/", 0, 0);
        if (!list.contains(indexStr))
            list += indexStr;
    }
    return list;
}

QList<int> IniBased::childKeysNumeric(const QSettings &settings, const QString &key) const
{
    QList<int> list;
    for (auto & k : childKeys(settings, key)) {
        bool ok;
        int i = k.toInt(&ok);
        if (ok)
            list.push_back(i);
    }
    return list;
}

QString IniBased::path() const
{
    QString s = sectionName + "/";
    if (parent()) {
        auto *p = dynamic_cast<const IniBased *>(parent());
        if (p)
            s.prepend(p->path());
    }
    return s;
}

QMap<QString, QString> IniBased::parseMap(const QString &str)
{
    QMap<QString, QString> tmp;
    QString s(str);
    if (s.startsWith("\"") && s.endsWith("\"")) {
        s.remove(0, 1);
        s.chop(1);
    }
    QStringList list = s.split(";");
    for (auto i : list) {
        i = i.trimmed();
        if (i.startsWith("{") && i.endsWith("}")) {
            i.remove(0, 1);
            i.chop(1);
            QStringList l = i.split(",");
            if (l.size() != 2)
                continue;
            QString k = l[0].trimmed();
            QString v = l[1].trimmed();
            tmp[k] = v;
        }
    }
    return tmp;
}
