//
//    Copyright 2013 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef WAVEFORMDECODER_H
#define WAVEFORMDECODER_H

#include "genericdecoder.h"
#include "decst_t.h"
#include "adcmlib/adcm_dec.h"

class AdcmEvent;
class adcm_chbuf_t;
class adcm_decstat_t;
struct ftimer_t;
//class adcm_ws_t;
struct cb_t;

/**
 * Generic waveform decoder for AdcmEvent
 */

class WaveformDecoder  : public GenericDecoder
{
public:
    WaveformDecoder(QObject *parent = nullptr);
    WaveformDecoder(const WaveformDecoder &) = delete; // copy prohibited
    WaveformDecoder &operator=(const WaveformDecoder &) = delete; // copy prohibited
    ~WaveformDecoder () override;
    static adcm_eval_flags_t resolve_decoder_stages_dependencies (const adcm_eval_flags_t &f);
    static adcm_eval_flags_t decoder_stages_for_mode (decoder_mode_t mode, int fitalgo);
    void decode(AdcmEvent*, adcm_decstat_t*);
    virtual int simulate (AdcmEvent *) { return 0; }
    static void adcm_decoder_clear_stat (adcm_decstat_t * st);
//    static void adcm_decoder_print_stat (const adcm_decstat_t &st);

protected:
    virtual void decode_work(AdcmEvent*, adcm_decstat_t*, decst_t_t &, ftimer_t &);
    void decode_prepare(const AdcmEvent &, const adcm_eval_flags_t &);
    void decode_finished(const AdcmEvent &, adcm_decstat_t*);
    static int cb_peak_rule(int *d, int ip);
//    int makewb (const AdcmEvent &ev);
//    int makewb_suboffs ();
    void subtract_calzero(AdcmEvent *ev);
//    void adcm_ev_subtract_offset(AdcmEvent *ev);
    int adcm_ev_find_thrpos (AdcmEvent * ev, adcm_decstat_t * decst);
    bool adc_find_integration_range (AdcmEvent * ev, unsigned int ch, int r1, int r2, int *fi1, int *fi2);
    int adcm_ev_find_baseline (AdcmEvent * ev, adcm_decstat_t * decst);
    int adcm_ev_find_integral (AdcmEvent * ev, adcm_decstat_t * decst);
    int adcm_ev_find_clip (AdcmEvent * ev, adcm_decstat_t * decst);
    int adcm_ev_find_rms (AdcmEvent * ev, adcm_decstat_t * decst);
    int adcm_ev_validate_ch (AdcmEvent * ev, adcm_decstat_t * decst);
//    int cb_find_pileup (int ch, adcm_chbuf_t *cb, adcm_decstat_t *decst);

    void clearmsg ();

    static void print_decoder_dsp_err (adcm_decstat_t * decst, const char *fmt, ...);

//    adcm_ws_t * const ws;
    adcm_eval_flags_t stages;
};


#endif // WAVEFORMDECODER_H
