//
//    Copyright 2013 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef WAVEFORMDECODERPEAKS_H
#define WAVEFORMDECODERPEAKS_H

#include "waveformdecoder.h"

class AdcmEvent;
class adcm_decstat_t;
class adcm_chbuf_t;
class FitterGaus;

/**
 * Peak searching decoder.
 * Uses FIR filter to transform arbitrary fixed form signals to gaussians
 * Peak fitting method used for parametrization
 * Result data is peak amplitude, time and peak width (same as egde width in input signal)
 */

class WaveformDecoderPeaks: public WaveformDecoder {
public:
    WaveformDecoderPeaks(QObject *parent = nullptr);
    WaveformDecoderPeaks(const WaveformDecoderPeaks &) = delete;
    WaveformDecoderPeaks &operator=(const WaveformDecoderPeaks &) = delete;
    ~WaveformDecoderPeaks () override;
    int simulate (AdcmEvent * ev) override;

protected:
    void decode_work(AdcmEvent*, adcm_decstat_t*, decst_t_t &, ftimer_t &) override;
//    int convolve_ch(adcm_decstat_t * decst, int c);
    void update_peaksearch_range(AdcmEvent * ev, adcm_decstat_t * decst);
    void make_convolve(AdcmEvent * ev) const;
    void find_peak(adcm_chbuf_t *cb, adcm_decstat_t *decst) const;
    void peaksearch(AdcmEvent * ev, adcm_decstat_t * decst);
//    int find_pileup (AdcmEvent * ev, adcm_decstat_t * decst);
    int find_fasttime (int method, AdcmEvent * ev, adcm_decstat_t * decst);
    int find_tfast (AdcmEvent * ev, adcm_decstat_t * decst);
    int find_finetime (int method, AdcmEvent * ev, adcm_decstat_t * decst);
    int find_tfine (AdcmEvent * ev, adcm_decstat_t * decst);

    bool find_dt_cut_condition(AdcmEvent *ev, adcm_decstat_t *decst);

    FitterGaus * const fitterGaus;
};

#endif // WAVEFORMDECODERPEAKS_H
