//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//


#ifndef DSPLIB_CONVOLVER_H
#define DSPLIB_CONVOLVER_H

#include <vector>

class Convolver
{
public:
    Convolver();
    // Convolution
    // double, double
    static std::vector<double> convolve (const std::vector<double> &v1, const std::vector<double> &v2);
    // double, int
    static std::vector<double> convolve (const std::vector<double> &v1, const std::vector<int> &v2);
    static std::vector<double> convolve (const std::vector<int> &v1, const std::vector<double> &v2) { return convolve(v2, v1); }
    // double, short
    static std::vector<double> convolve (const std::vector<double> &v1, const std::vector<short> &v2);
    static std::vector<double> convolve (const std::vector<short> &v1, const std::vector<double> &v2) { return convolve(v2, v1); }
    // int, int
    static std::vector<int> convolve (const std::vector<int> &v1, const std::vector<int> &v2);
    // short, short
    static std::vector<int> convolve (const std::vector<short> &v1, const std::vector<short> &v2);
    // int, short
    static std::vector<int> convolve (const std::vector<short> &v1, const std::vector<int> &v2);
    static std::vector<int> convolve (const std::vector<int> &v1, const std::vector<short> &v2) { return convolve(v2, v1); }

protected:
    // size of output is l1+l2
    // Integer+Integer -> Integer
    static void convolve_int_int (const int *x1, int l1, const int *x2, int l2, int *out); //!< deprecated
    // Integer+ShortInteger -> Integer
    static void convolve_int_short (const int *x1, int l1, const short int *x2, int l2, int *out);
    // Double+Double -> Double
    static void convolve_double_double (const double *x1, int l1, const double *x2, int l2, double *out); //!< deprecated
    // Integer+Double -> Double
    static void convolve_int_double (const int *x1, int l1, const double *x2, int l2, double *out);
    // Short+Double -> Double
    static void convolve_short_double (const short *x1, int l1, const double *x2, int l2, double *out);
};

#endif // DSPLIB_CONVOLVER_H
