//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//


#ifndef DSPLIB_FFTSPECTRUM_H
#define DSPLIB_FFTSPECTRUM_H

#include <vector>

class TimeDomainData {
  public:
        TimeDomainData(): t0(0), samplingFreq(1) {}
        TimeDomainData(const std::vector<double> &_v, double _t0 = 0, double _samplingFreq = 1): v(_v), t0(_t0), samplingFreq(_samplingFreq) {}
        size_t size() const { return v.size(); }
        double operator[](size_t n) const { return v[n]; }
        void print() const;
        std::vector<double> v;
        double t0;
        double samplingFreq;
};

class FFTSpectrumElement {
  public:
        FFTSpectrumElement(): freq(0), power(0), phase(0) {}
        FFTSpectrumElement(double f, double p, double h): freq(f), power(p), phase(h) {}
        FFTSpectrumElement(const FFTSpectrumElement &r): freq(r.freq), power(r.power), phase(r.phase) {}
        bool operator < (const FFTSpectrumElement &r) const { return power < r.power; }
        double freq;
        double power;
        double phase;
};

// Magnitude peak descriptor
class FFTSpectrumPeak {
  public:
        FFTSpectrumPeak(): freq(0), power(0) { }
        bool operator < (const FFTSpectrumPeak &r) const { return power < r.power; }
    double freq; // frequency
    double power; // magnitude
};

inline bool FFTSpectrumPeakLt(const FFTSpectrumPeak &l, const FFTSpectrumPeak &r) { return r<l; }

class FFTSpectrum : public std::vector<FFTSpectrumElement> {
  public:
        void print() const;
        double getPeak() const;
        std::vector<FFTSpectrumPeak> getPeaks(double threshold = 0.1) const;
        FFTSpectrum& operator+=(const FFTSpectrum &r) { *this = *this + r; return *this;}
        FFTSpectrum operator+(const FFTSpectrum &r) const;
        FFTSpectrum operator/(double n) const;
        FFTSpectrumElement getApproximation(double frequency) const;
        FFTSpectrum getFFTSpectrum(const TimeDomainData &d);
};

#endif // DSPLIB_FFTSPECTRUM_H
