/*
**    Copyright 2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
#ifndef GENERICSTATISTICS_H
#define GENERICSTATISTICS_H

#include <cmath>
#include <vector>

class GenericStatistics {
public:
    void clear() { *this = GenericStatistics(); }
    double count() const { return _count; }
    double mean() const { return _mean; }
    double variance() const { return (_count < 1) ? 0 : _sumvar/_count; }
    double stddev() const { return sqrt(variance()); }
    template<class T>
    void fill(const std::vector<T> &v) {
        for (auto i: v)
            fill(i);
    }
    void fill(double d) {
        // Welford's online algorithm
        _count++;
        double delta = d - _mean;
        _mean += delta / _count;
        double delta2 = d - _mean;
        _sumvar += delta * delta2;
    }
    GenericStatistics &operator+=(const GenericStatistics &r) {
        if (_count + r._count > 0)
            _mean = (_mean * _count + r._mean * r._count) / (_count + r._count);
        _count += r._count;
        _sumvar += r._sumvar;
        return *this;
    }
private:
    double _count = 0;
    double _mean = 0;
    double _sumvar = 0;  // sum of squared distances from the mean
};

#endif // GENERICSTATISTICS_H
