#ifndef TH1_H
#define TH1_H

#include <QString>
#include <QSharedData>
#include <vector>
#include "GenericStatistics.h"

//class th1 : public QSharedData {
class th1 {

public:

    enum {FIT=0x0001, AUTORANGE=0x0002, NO_AUTORANGE=0x0004, FIT_RESOL=0x0008};
    static constexpr double autorange_threshold = 2.0;
    static constexpr double autorange_ratio = 0.025;

    th1() = default;
    th1(const QString &_title, const QString &_xLabel, int fnbins, double fmin, double fmax, int foptions = 0);
    th1(const QString &_title, int fnbins, double fmin, double fmax, int foptions = 0);
//    void add(const th1 *add);

    bool isEmpty() const;
    // options
    int get_options() const {return options;}
    bool autorange_enabled() const;
    bool fit_enabled() const;
    bool fit_resol_enabled() const;
    void set_autorange_enable(bool state) const;
    void set_fit_enable(bool state) const;
    void set_title(const QString &s) { title = s; }
    void set_x_label(const QString &s) { xLabel = s; }
    void set_y_label(const QString &s) { yLabel = s; }
    QString get_title() const { return title; }
    QString get_x_label() const { return xLabel; }
    QString get_y_label() const { return yLabel; }
    GenericStatistics getStatistics() const;

    unsigned int get_nbins () const;
    std::vector<double> get_px () const {
        std::vector<double> px;
        for (unsigned int i = 0; i < nbins; i++) {
            px.push_back(binright(i));
        }
        return px;
    }
    std::vector<double> get_py () const { return py; }
    double get_bin (unsigned int index) const;
    void fill (double x);
    void filln (double x, double count);
    void clear ();
    void setrange (double inmin, double inmax);
    void getrange (double *outmin, double *outmax) const;
    GenericStatistics simplestats() const;
    int fitgaus (double *value, double *mean, double *sigma, double *fnorm=0) const;
    QString getname() const;

    // statistical values
    unsigned int get_nent () const;
    unsigned int get_over () const;
    unsigned int get_under () const;
    double get_sum () const;
    int get_modseq () const;
    double get_xmin () const;
    double get_xmax () const;
    double binwidth() const;
    double binleft (int n) const;
    double binright (int n) const;
    double bincenter (int n) const;

    int findbin (double x) const;
    bool find_autorange_bins(unsigned int *bfirst, unsigned int *blast) const; // bfirst >= 0, blast <= nbins
    void update_autorange() const;

    th1 &operator+=(const th1 &r);

private:
    void update_cache();
    bool isEqualRange(const th1 &r) const;
    bool isEqualRange(size_t _nbins, double _xmin, double _xmax) const;
    void resize(const th1 &r);
    void resize(size_t _nbins, double _xmin, double _xmax);

    int mod = 0; // Number of modifications
    std::vector<double> py = std::vector<double>(1);
    unsigned int under = 0, over = 0, nent = 0;
    double xmin = 0, xmax = 1;
    double cache_xrange = 1;
    double cache_binwidth = 1;
    QString title;
    QString xLabel;
    QString yLabel;
    unsigned int nbins = 1;
    // autorange data
    mutable double ar_min = 0, ar_max = 1;
    mutable unsigned int options = 0;
    GenericStatistics statistics;
};

#endif /* TH1_H */
