/*
**    Copyright 2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "hist_manager_types.h"
#include "adcm16.h"
#include "adcmlib/ch_label.h"

static const int HIST_MAX_ADC_CHANNELS = ADCM_N_CH;
static const int HIST_CHANNEL_OFFSET = HIST_MAX_ADC_CHANNELS;

//hist_index_t make_hist_index(hist_type_t htype, int channel)
//{
//    hist_index_t index;
//    index.t = htype;
//    index.c = channel;
////    int index = htype * HIST_CHANNEL_OFFSET + channel;
//    return index;
//}

bool hist_index_t::operator<(const hist_index_t &r) const
{
    int my_index = t * HIST_CHANNEL_OFFSET + c;
    int r_index = r.t * HIST_CHANNEL_OFFSET + r.c;
    return my_index < r_index;
}

QString hist_title(hist_type_t t)
{
    const QString &str_M = ustr_ch_label_M;
    const QString &str_S = ustr_ch_label_S;
    switch (t) {
    case HIST_TYPE_INVALID: return QString();
    case HIST_TYPE_dT: return QString("Time T%1-T%2 fine").arg(str_S).arg(str_M);
    case HIST_TYPE_dTmax: return QString("Time T%1-T%2 coarse").arg(str_S).arg(str_M);
    case HIST_TYPE_Tmr: return QString("Time reconstruction quality %1").arg(str_M);
    case HIST_TYPE_Tsr: return QString("Time reconstruction quality %1").arg(str_S);
    case HIST_TYPE_Ts: return QString("Time %1 rel. trigger").arg(str_S);
    case HIST_TYPE_wm: return QString("Filt. pulse width W%1").arg(str_M);
    case HIST_TYPE_ws: return QString("Filt. pulse width W%1").arg(str_S);
    case HIST_TYPE_slideev: return ("Raw pulses / event");
    case HIST_TYPE_pulsev: return ("Fitted pulses / event");
    case HIST_TYPE_niter: return ("Fitter iterations / event");
    case HIST_TYPE_evdist: return ("Channel time, rel. Spill");
    case HIST_TYPE_decodetime: return ("Event decode time");
    case HIST_TYPE_T: return QString("Time T%1-T%2 (per %1 channel)").arg(str_S).arg(str_M);
    case HIST_TYPE_Int: return ("Integral");
    case HIST_TYPE_bl: return ("BaseLine");
    case HIST_TYPE_dT_Ev: return ("Time interval (event to event)");
    default: return QString();
    }
}

bool hist_is_multichannel(hist_type_t t)
{
    switch (t) {
    case HIST_TYPE_INVALID: return false;
    case HIST_TYPE_dT: return false;
    case HIST_TYPE_dTmax: return false;
    case HIST_TYPE_Tmr: return false;
    case HIST_TYPE_Tsr: return false;
    case HIST_TYPE_Ts: return false;
    case HIST_TYPE_wm: return false;
    case HIST_TYPE_ws: return false;
    case HIST_TYPE_slideev: return false;
    case HIST_TYPE_pulsev: return false;
    case HIST_TYPE_niter: return false;
    case HIST_TYPE_evdist: return true;
    case HIST_TYPE_decodetime: return false;
    case HIST_TYPE_T: return true;
    case HIST_TYPE_Int: return true;
    case HIST_TYPE_bl: return true;
    case HIST_TYPE_dT_Ev: return false;
    default: return false;
    }

}
