/*
**    Copyright 2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef HIST_MANAGER_TYPES_H
#define HIST_MANAGER_TYPES_H

#include <QMetaType>
#include <QString>

typedef enum {
    HIST_TYPE_INVALID,
    HIST_TYPE_dT, // Time Ts-Tm fine
    HIST_TYPE_dTmax, // Time Ts-Tm coarse
    HIST_TYPE_Tmr, // Rec. quality Tm
    HIST_TYPE_Tsr, // Rec. quality Ts
    HIST_TYPE_Ts, //Time Ts
    HIST_TYPE_wm, // Filt. pulse width Wm
    HIST_TYPE_ws, // Filt. pulse width Ws
    HIST_TYPE_slideev, // Raw pulses / event
    HIST_TYPE_pulsev, // Fitted pulses / event
    HIST_TYPE_niter, // Fitter iterations / event
    HIST_TYPE_evdist, // Channel time, rel. Spill
    HIST_TYPE_decodetime, // Event decode time
    HIST_TYPE_T, // Ch Time rel. Master
    HIST_TYPE_Int, // Integral
    HIST_TYPE_bl, // BaseLine
    HIST_TYPE_dT_Ev, // Time Ev(i)-Ev(i-1)
} hist_type_t;

Q_DECLARE_METATYPE(hist_type_t)

struct hist_index_t
{
    hist_index_t(hist_type_t htype = HIST_TYPE_INVALID, int channel = 0)
        : t(htype), c(channel) { }
    bool operator<(const hist_index_t &r) const;
    hist_type_t t = HIST_TYPE_INVALID;
    int c = 0;
};

Q_DECLARE_METATYPE(hist_index_t)

QString hist_title(hist_type_t t);
bool hist_is_multichannel(hist_type_t t);

#endif // HIST_MANAGER_TYPES_H
