/*
**    Copyright 2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "adcm_raw_types.h"

// data block length in words
size_t adcm_block_len (const adcm_data_header_t * h)
{
    switch (h->id) {
    case 0 :
    case 2 :
    {
        const adcm_data_header_0_t *ah = reinterpret_cast<const adcm_data_header_0_t *>(h);

        size_t hdrlen = (sizeof (adcm_data_header_0_t) + 3) / 4;
        size_t trllen = (sizeof (adcm_data_trailer_0_t) + 3) / 4;
        size_t datalen = 0;
        if (ah->spw > 0)
            datalen = ((ah->ns/2*2)+ ah->spw - 1) / ah->spw;
        return (hdrlen + datalen + trllen); // hdr, data, trailer
    }
    case 1:                      // stat
    {
        const adcm_data_header_1_t *ah = reinterpret_cast<const adcm_data_header_1_t *>(h);

        size_t hdrlen = (sizeof (adcm_data_header_1_t) + 3) / 4;
        size_t trllen = (sizeof (adcm_data_trailer_t) + 3) / 4;
        size_t datalen = (ah->wpc * ah->nc);
        return (hdrlen + datalen + trllen); // hdr, data, trailer
    }
    default:                     // generic
        return (sizeof (adcm_data_header_t) + sizeof (adcm_data_trailer_t));
    }
}
