//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef QLOG_H
#define QLOG_H

#include <QtGlobal>
#include <QString>

class QLogPrivate;

class QLog
{
private:
    friend class Cleanup;
    class Cleanup
    {
    public:
        ~Cleanup();
    };
    static QLog *_instance;
    static Cleanup _Cleanup;

    QLog(const QLog &);              // intentionally undefined
    QLog & operator=(const QLog &);  // intentionally undefined

public:
    enum {
        Alert = 10,    /* action must be taken immediately */
        Crit = 20,     /* critical conditions              */
        Err = 30,      /* error conditions                 */
        Warning = 40,  /* warning conditions               */
        Notice = 50,   /* normal but significant condition */
        Info = 60,     /* informational                    */
        Debug = 70     /* debug-level messages             */
    };

    static QLog &getInstance();
    void setDebugLevel(int debugLevel);
    int getDebugLevel() const;
    void err(const char *str) const;
    void err(const QString &str) const;
    void warn(const char *str) const;
    void warn(const QString &str) const;
    void notice(const char *str) const;
    void notice(const QString &str) const;
    void info(const char *str) const;
    void info(const QString &str) const;
#ifdef QT_DEBUG
    void debug(int level, const char *str) const;
    void debug(int level, const QString &str) const;
#else
    inline void debug(int level, const QString &str) const {}
#endif

protected:
    void log_pri(int priority, int debugLevel, const char *str) const;
    void log_pri(int priority, int debugLevel, const QString &str) const;

private:
    QLog();
    ~QLog();
    QLogPrivate * const d;
};

extern QLog &logger;

#endif // QLOG_H
