//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//


#ifndef QXW_QXWPLOT_H
#define QXW_QXWPLOT_H

#include <vector>
#include <QMap>
#include <qwt_plot.h>
#include <qwt_plot_curve.h>
#include <QSharedPointer>

class QwtPlotMarker;

typedef QSharedPointer<QwtPlotCurve> QwtPlotCurvePtr;
typedef QSharedPointer<QwtPlotMarker> QwtPlotMarkerPtr;

class QxwCurveSet: public QMap<int, QwtPlotCurvePtr>
{
private:
    //    QxwCurvePtr &operator[](const int & key);
    //    QxwCurvePtr operator[](const int & key) const;
};
typedef QMap<int, QwtPlotMarkerPtr> QxwMarkerSet;

class QwtPlotZoomer;
class QwtPlotPanner;
class QwtPlotGrid;

class QxwPlot : public QwtPlot
{
    Q_OBJECT
public:
#if QT_VERSION >= 0x040000
    explicit QxwPlot(QWidget* parent = 0);
#else
    explicit QxwPlot(QWidget* parent = 0, const char* name = 0);
#endif
    virtual ~QxwPlot();

    virtual void clear();

    void setCurveBrush(int id, const QBrush &brush);
    void setCurvePen(int id, const QPen &pen);
    void setCurveStyle(int id, QwtPlotCurve::CurveStyle style);
    void setCurveSymbol(int id, const QwtSymbol &symbol);

    void setBrush(const QBrush &brush);
    void setPen(const QPen &pen);
    void setStyle(QwtPlotCurve::CurveStyle style);
//    void setSymbol(const QwtSymbol &symbol);

    void setYMarkers(const std::vector<double> &v);
    void setXMarkers(const std::vector<double> &v);
    void updateLegendItems();
    void enableLegend(bool enable = true);
    void disableLegend(bool disable = true);

protected:
    virtual void setupZoomer();
    virtual QRectF getDataBoundRect() const;
    virtual void clearCurves();
    virtual void clearXMarkers();
    virtual void clearRefMarkers();
    QwtPlotCurvePtr getCurve(int id);
    bool curveAttached(int id) const;
    bool curveExists(int id) const;
    bool markrefExists(int id) const;
    bool xmarkerExists(int id) const;


    QxwCurveSet curves;
    QxwMarkerSet xMarkers;
    QxwMarkerSet yMarkers;
    typedef QSharedPointer<QwtPlotGrid> QwtPlotGridPtr;
    QwtPlotGridPtr grid;
//    typedef QSharedPointer<QwtPlotZoomer> QwtPlotZoomerPtr;
    QwtPlotZoomer * zoomer;
//    typedef QSharedPointer<QwtPlotPanner> QwtPlotPannerPtr;
    QwtPlotPanner * panner;

signals:

public slots:
protected slots:
    void onLegendChecked(QwtPlotItem *plotItem, bool on);
};

#endif // QXW_QXWPLOT_H
