//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include <map>
#include "WaveformAverager.h"

void WaveformAverager::add(const WaveformDataAll &d)
{
    for(WaveformDataAll::const_iterator it = d.begin(); it != d.end(); ++it) {
        int key = (*it).first;
        const WaveformData &w = (*it).second;
        WaveformDataAll::iterator ii = wd.find(key);
        if (ii == wd.end()) { // copy
            wd[key] = w;
            count[key] = 1;
        } else { // accumulate
            WaveformData &wavg = (*ii).second;
            if (wavg.p.size() < w.p.size())
                wavg.p.resize(w.p.size());
            for(size_t i=0;i<w.p.size(); i++) {
                wavg.p[i].y += w.p[i].y;
            }
            count[key] += 1;
        }
    }
    /*
     // vector-based
    foreach_const(d, it) {
        const WaveformData &w = (*it);
        int key = w.c;
        bool found = false;
        foreach(wd, jt) {
            WaveformData &wavg = (*jt);
            if (wavg.c == key) {
                found = true;
                // accumulate
                if (wavg.p.size() < w.p.size())
                    wavg.p.resize(w.p.size());
                for(int i=0;i<w.p.size(); i++) {
                    wavg.p[i].y += w.p[i].y;
                }
                count[key] += 1;
                break;
            }
        }
        if (!found) {
            // copy
            wd.push_back(w);
            count[key] = 1;
        }
    }
    */
}

void WaveformAverager::clear()
{
    count.clear();
    wd.clear();
}

WaveformDataAll WaveformAverager::get() const
{
    WaveformDataAll result(wd);

    for (WaveformDataAll::iterator it = result.begin(); it != result.end(); ++it) {
        int key = (*it).first;
        WaveformData &w = (*it).second;
        //        WaveformData &w = (*it);
        //        int key = w.c;
        WaveformCountAll::const_iterator ci = count.find(key);
        if (ci == count.end()) continue;
        quint64 tmp_count = (*ci).second;
        if (tmp_count == 0) continue;
        for(size_t i=0;i<w.p.size(); i++) {
            w.p[i].y /= tmp_count;
        }
    }
    return result;
}
