//
//    Copyright 2010-2019 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef QXW_WAVEFORMDATA_H
#define QXW_WAVEFORMDATA_H

#include <vector>
#include <QVector>
#include <map>
#include <QString>

class point_t
{
public:
    point_t() = default;
    point_t(double px, double py): x(px), y(py) {}
    double x = 0;
    double y = 0;
};

class WaveformData {
public:
    WaveformData(const QString &dataName = QString(), int dataChannel = 0);
    WaveformData(const QVector<double> &vy, const QString &dataName = QString(), int dataChannel = 0);
    WaveformData(const std::vector<double> &vy, const QString &dataName = QString(), int dataChannel = 0);
    WaveformData(const QVector<double> &vx, const QVector<double> &vy, const QString &dataName = QString(), int dataChannel = 0);
    WaveformData(const std::vector<double> &vx, const std::vector<double> &vy, const QString &dataName = QString(), int dataChannel = 0);

    std::vector<double> getPointsY() const;
    void simulate(unsigned int length = 20);
    void print() const;
    void setName(const QString &str) { name = str; }

    int c = 0; // channel
    std::vector<point_t> p; // points
    std::vector<double> t; // time marks
    double baseline = 0; // baseline offset value
    QString name;
};

typedef std::map<int, WaveformData> WaveformDataAll; // index is channel number

class WaveformDataSimulator
{
public:
    WaveformDataAll run();
};

#endif // QXW_WAVEFORMDATA_H
