//
//    Copyright 2010 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include <QEvent>

#include <qwt_plot.h>
#include <qwt_plot_marker.h>

#include "qxw/WaveformView.h"
#include "qxw/WaveformData.h"
#include "qxw/WaveformPlot.h"

#include "ui_WaveformViewUi.h"


WaveformView::WaveformView(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::WaveformViewUi)
{
    ui->setupUi(this);
}

WaveformView::~WaveformView()
{
    delete ui;
}

void WaveformView::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

WaveformPlot *WaveformView::plot()
{
    return ui->waveformPlot;
}

void WaveformView::init()
{
    //  ui->frameControls->hide();
    currentTimeMarker.attach(plot());
    currentTimeMarker.setLineStyle(QwtPlotMarker::VLine);
    currentTimeMarker.setLinePen(QPen(QColor(Qt::green)));
}

void WaveformView::setAutoscale( bool b )
{
    setup.autoscale = b;
}

void WaveformView::clear()
{
    plot()->clear();
}

/*
void WaveformView::simulate()
{
  wfm.simulate();
  ui->waveformPlot->setAllData(wfm.get());
  stats->print();
}
*/

void WaveformView::setWaveformData( const WaveformDataAll &w )
{
    //  wfm.set(w);
    plot()->setAllData(w);
    //  stats->print();
}

void WaveformView::setCurrentTimeMarker(double v)
{
    currentTimeMarker.setXValue(v);
}

void WaveformView::setXMarkers(const std::vector<double> &v)
{
    plot()->setXMarkers(v);
}
