/*
**    Copyright 2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
#include "AsciiGraphics.h"
#include <QTextStream>
#include <QVector>

AsciiGraphics::AsciiGraphics()
{

}

QString AsciiGraphics::ascii_bar(int chars, double value)
{
    QString s = "|";
    value *= chars;
    for (int i = 0; i <= chars; i++) {
        double x = value - i;
        if (x >= 1) {
            s += "-";
        } else
            if (x < 0) {
            s += " ";
        } else {
            s += "*";
        }
    }
    return s;
}

void AsciiGraphics::drawWaveform(QTextStream &stream, const QVector<int> &w, const QString &title)
{
    QVector<double> vd;
    for (auto i : w)
        vd.push_back(i);
    drawWaveform(stream, vd, title);
}

void AsciiGraphics::drawWaveform(QTextStream &stream, const QVector<double> &w, const QString &title)
{
    double wmin = 1e100;
    double wmax = -1e100;
    for (auto s : w) {
        if (s < wmin)
            wmin = s;
        if (s > wmax)
            wmax = s;
    }
    for (int i=0; i < w.size(); i++) {
        stream << QString("      %1[%2]: %3 %4")
                      .arg(title)
                      .arg(i, 3)
                      .arg(w[i], 8)
                      .arg(AsciiGraphics::ascii_bar(40, 1.0 * (w[i] - wmin) / (wmax-wmin)))
            ;
        stream << endl;
    }
}
