//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef TEXTFORMATTER_H
#define TEXTFORMATTER_H

#include <ostream>
#include <QTextStream>
#include <QDebug>
#include <map>
#include <vector>

// std::ostream
template<class Key, class T>
std::ostream &operator<<(std::ostream &s, const std::map<Key, T> &v)
{
    s << "[";
    for (auto it = v.begin(); it!=v.end(); ++it) {
        if (it != v.begin())
            s << ",";
        s << "[" << (*it).first << "]:";
        s << (*it).second;
    }
    s << "]";
    return s;
}

template<class T>
std::ostream &operator<<(std::ostream &s, const std::vector<T> &v)
{
    s << "{";
    for (auto it = v.begin(); it!=v.end(); ++it) {
        if (it != v.begin())
            s << ", ";
        s << (*it);
    }
    s << "}";
    return s;
}

template<class T>
std::ostream &operator<<(std::ostream &s, const QVector<T> &v)
{
    s << v.toStdVector();
    return s;
}

// QTextStream
template<class Key, class T>
QTextStream &operator<<(QTextStream &s, const std::map<Key, T> &v)
{
    s << "[";
    for (auto it = v.begin(); it!=v.end(); ++it) {
        if (it != v.begin())
            s << ",";
        s << "[" << (*it).first << "]:";
        s << (*it).second;
    }
    s << "]";
    return s;
}

template<class T>
QTextStream &operator<<(QTextStream &s, const std::vector<T> &v)
{
    s << "[";
    for (auto it = v.begin(); it!=v.end(); ++it) {
        if (it != v.begin())
            s << ",";
        s << (*it);
    }
    s << "]";
    return s;
}

template<class T>
QTextStream &operator<<(QTextStream &s, const QVector<T> &v)
{
    s << v.toStdVector();
    return s;
}

/*
class HtmlStream: public QTextStream {
public:
    HtmlStream(QString * string, QIODevice::OpenMode openMode = QIODevice::ReadWrite)
        : QTextStream(string, openMode) { }
};

// HtmlStream
template<class Key, class T>
HtmlStream &operator<<(HtmlStream &s, const std::map<Key, T> &v)
{
    s << "<table>";
    for(typename std::map<Key, T>::const_iterator it = v.begin(); it!=v.end(); ++it) {
        s << "<tr><td>";
        s << (*it).first;
        s << "</td><td>";
        s << (*it).second;
        s << "</td</tr>";
    }
    s << "</table>";
    return s;
}

template<class T>
HtmlStream &operator<<(HtmlStream &s, const std::vector<T> &v)
{
    s << "<ol>";
    for(typename std::vector<T>::const_iterator it = v.begin(); it!=v.end(); ++it) {
        s << "<li>";
        s << (*it);
        s << "</li>";
    }
    s << "</ol>";
    return s;
}
*/

template<class T>
QDebug operator<<(QDebug s, const T &st)
{
    QString str;
    QTextStream stream(&str);
    stream << st;
    s.noquote() << str;
    return s;
}

#endif // TEXTFORMATTER_H
