#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QDateTime>
#include <QMainWindow>

#include "Config.h"
#include "mldiscover/DeviceDescription.h"
#include "mregdev/fw_version.h"

class QTimer;
class QLabel;
class EthFlashDev;
class DataBase;

class DiscoverDialog;
class DeviceDescription;
class QHostAddress;

namespace Ui {
class MainWindow;
}


class MainWindow : public QMainWindow
{
    Q_OBJECT
public:
    explicit MainWindow(QWidget *parent = nullptr);
    ~MainWindow();

private Q_SLOTS:
    void on_actionWrite_Golden_Image_triggered();
    void on_actionGet_record_logs_triggered();
    void on_actionVerify_after_write_toggled(bool verify);
    void on_actionSend_reboot_signal_triggered();
    void on_actionVerify_Golden_image_triggered();
    void on_actionVerify_regular_image_triggered();
    void on_actionErase_Golden_part_triggered();
    void on_actionErase_regular_part_triggered();
    void on_actionAbout_triggered();

    void on_pushButtonChange_clicked();
    void on_pushButtonFile_clicked();
    void on_pushButtonWrite_clicked();

    void deviceConnected(const DeviceIndex &index, QString fwStr, const fw_version_t &fwRev);
    void deviceDisconnected();
    void deviceUpdated(const DeviceDescription &dd);
    void deviceTempChanged(double temp);
    void deviceStatusUpdated(QString s);
    void progressFormatChanged(DeviceIndex index, QString format);
    void progressUpdated(DeviceIndex index, int p);
    void taskFinished(DeviceIndex index, uint id, bool isWrite, bool ok, bool isGoldenImage);
    void liveMagicFailed(DeviceIndex index);
    void deviceEnabled(bool st);


signals:
    void setDeviceEnabled(DeviceIndex, bool);
    void startDevErase(uint id, bool isGold);
    void startDevWrite(uint id, QByteArray file, int headerSize, bool isGold, bool verify);
    void startDevVerify(uint id, QByteArray file, int headerSize, bool isGold);
    void closeDevices();
    void disconnectFromHost(const DeviceIndex &index);

private:
    void addDevice(const DeviceDescription &dd);
    void disconnectFromAll();
    void readConfig();
    void saveConfig();
    void updateWriteButtonState() const;
    void write65k();

    bool readFile();
    void startWrite(bool isGoldenImage=false);
    void startVerify(bool isGoldenImage=false);
    void startErase(bool isGoldenImage=false);
    void startTask(int task, bool isGoldenImage);
    void initTable();
    void updateTableSize();
    DeviceIndex getIndexBySender(QObject *sender) const;
    DeviceIndex getIndexByRow(int row) const;
    int getRowByIndex(DeviceIndex index) const;
    int getRowBySender(QObject *sender) const;
    void setPushButtonEnabled(bool enabled) const;
    void setLabelStatus(QString msg, QString color) const;

    Ui::MainWindow *ui;
    Config config;
    QMap<DeviceIndex, EthFlashDev *> devices;
    QMap<DeviceIndex, QThread *> deviceThreads;
    QMap<DeviceIndex, bool> devOk;
    QMap<DeviceIndex, bool> devEn;
    QSet<DeviceIndex> devLiveMagicErr;
    QMap<DeviceIndex, uint> devTaskId; // if ok contains non-zero write-index
    bool connectedState;
    DiscoverDialog * const discoverDialog;
    double testValue;
    quint64 curDevSerial;
    QLabel * const tempLabel;
    QLabel * const fwLabel;
    QLabel * const lastStatusLabel;
    DataBase * dataBase;

    QByteArray fileData;
    int fileHeaderSize;
    int fileSize;
    QDateTime fileModified;
    int cmdPerformed;
    uint taskId;
};

#endif // MAINWINDOW_H
