/*
**    Copyright 2020 Ivan Filippov, Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "MainWindow.h"
#include "ui_MainWindow.h"

#include <iostream>

#include <QApplication>
#include <QCloseEvent>
#include <QDebug>
#include <QHostAddress>
#include <QMessageBox>
#include <QString>

#include "device-discover/DiscoverDialog.h"
#include "DeviceHistory.h"
#include "base/AboutBox.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow),
    discoverDialog(new DiscoverDialog(this))
{
    ui->setupUi(this);

    ui->verticalLayout_1->addWidget(discoverDialog);
    //        dialog.setDeviceIdFilter(0xc5);
    //        dialog.setSingleMode();
    discoverDialog->show_mac_adresses();
    discoverDialog->show();
    discoverDialog->setAttribute(Qt::WA_Moved, true);
    discoverDialog->setPrintInfoFull(false);

    connect(discoverDialog, &QDialog::accepted,
            this, &QMainWindow::close);
    connect(discoverDialog, &QDialog::rejected,
            this, &QMainWindow::close);

    connect(discoverDialog, &DiscoverDialog::deviceDiscovered,
                     this, &MainWindow::onDeviceDiscovered);
    QObject::connect(discoverDialog, &DiscoverDialog::deviceExpired,
                     this, &MainWindow::onDeviceExpired);

#ifndef QT_NO_DEBUG
    if (0) {
        QVector<DeviceDescription> list;
        DeviceDescription dd;
        //
        dd.device_id = 197;
        dd.serial_id = 74401055;
        dd.discoverTime = QDateTime::currentDateTime();
        dd.ttl = 3;
        list.push_back(dd);
        //
        dd.device_id = 196;
        dd.serial_id = 74401057;
        dd.discoverTime = QDateTime::currentDateTime();
        dd.ttl = 4;
        list.push_back(dd);
        //
        dd.device_id = 195;
        dd.serial_id = 74401056;
        dd.discoverTime = QDateTime::currentDateTime();
        dd.ttl = 5;
        list.push_back(dd);
        discoverDialog->setSelected(list);
    }
#endif
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    qDebug() << "Selected devices:";
    foreach (const DeviceDescription &dd, discoverDialog->getSelectedDevices()) {
        if (dd.enabled)
            qDebug() << QString("s/n:%1; type:%2; ip:%3")
                        .arg(dd.getSerialIdStr())
                        .arg(dd.getModelStr())
                        .arg(QHostAddress(dd.ip_addr).toString());
    }
    event->accept();
}

void MainWindow::on_actionExit_triggered()
{
    close();
}

void MainWindow::on_actionAbout_triggered()
{
    AboutBox::about();
}

void MainWindow::onDeviceDiscovered(const DeviceDescription &dd)
{
    qDebug() << dd.getIdent();
}

void MainWindow::onDeviceDiscoveredAll(const DeviceDescription &dd)
{
    qDebug() << dd.getIdent();
}

void MainWindow::onDeviceExpired(const DeviceIndex &i)
{
    qDebug() << i;
}
