#ifndef CLIENTMANAGER_H
#define CLIENTMANAGER_H

#include "pnp-server/PNPMessage.h"

#include <QObject>

template<class T> class RingBuffer;
class TcpClient;
class TcpStatistic;
class Analizer;
class EventMetaInfo;
class QHostAddress;
class QSemaphore;
class AnalyzerStat;
class RcRunId;

class ClientManager : public QObject
{
    Q_OBJECT
public:
    explicit ClientManager(int _index, int evNumIndex, bool readout, int preferedBufSize, QObject *parent = 0);
    virtual ~ClientManager();
    Analizer *getAnalizer() const { return analizer; }
    int getIndex() const { return index; }
    QString getHostAddr() const;
    int getHostPort() const;
    bool isReadoutEnabled() const { return readoutEnabled; }
    void setProgDescription(const ProgramDescription &descr);
    ProgramDescription getProgDescr() const { return progDescription; }
    QString getName() const { return QString("Client #%1 \"%2\"").arg(index).arg(progDescription.getName()); }

signals:
    void startReadOut();
    void stopReadOut();
    void started(int, bool, uint);
    void stopped(int,uint);
    void resetted(int,uint);
    void starting(uint);
    void stoping(uint);
    void resetting(uint);
    void destroyingClient();
    void sendStatus(int,bool,QString);
    void sendStatistic(int index, const TcpStatistic &statistic);
    void setTcpAddress(const QHostAddress &host, const int port);
    void setTcpAddress(const ProgramDescription &descr, const ProgramInterface &interface);
    void runError();
    void clientDescrChanged(const QString &descr) const;
    void statUpdated(const AnalyzerStat &) const;
    void inputBufferSizeUpdated(int index, quint64 size) const;

public slots:
    void startClient(uint stateIndex);
    void stopClient(uint stateIndex);
    void resetClient(const RcRunId &runId, uint stateIndex);
    void destroyClient(int i);
    void updateClientReadout(int i, bool readout);

private slots:
    void updateRunState(bool isAnalizer, bool isInRunMode, uint stateIndex);
    void onResetted(bool isAnalizerNumber, uint stateIndex);
    void onTcpError(QString err);

private:
    const int index;
    ProgramDescription progDescription;
    ProgramInterface tcpInterface;
    int evNumIndex;
    RingBuffer<char> * const dataRingBuff;
    RingBuffer<EventMetaInfo> * const evRingBuff;
    QThread * const tcpClientThread;
    QThread * const analizerThread;
    TcpClient * const tcpClient;
    Analizer * analizer;
    bool isAnalizerRunning;
    bool isTcpClientRunning;
    bool isAnalizerResetting;
    bool isTcpClientResetting;
    bool run;
    bool readoutEnabled;
    bool gotError;
    uint curStateIndex;
};

#endif // CLIENTMANAGER_H
