#include "EvBConfig.h"

EvBConfig::EvBConfig(QString program_type_init, QString program_index_init) :
    program_type(program_type_init),
    program_index(program_index_init),
    configuration_name("default"),
    runId(RcRunId(program_index_init, 0)),
    combinerOut(COMBINER_FILE),
    isRemoteEnabled(true),
    advancedCounters(false),
    combinerOutTcpNum(0),
    startEvNumber(0),
    evNumberStep(1),
    storageDir(""),
    timeDistrEn(false),
    monitorPort(0)
{
    QString default_config_name = "default";
    QString combinerOut = "file";
    isRemoteEnabled = true;
    advancedCounters = false;
    combinerOutTcpNum = 0;
    startEvNumber = 0;
    evNumberStep = 1;
    storageDir = "";
    timeDistrEn = false;
    monitorPort = 0;

    default_root.set_data("program_type", program_type);
    default_root.set_data("program_index", program_index);
    default_root.set_data("configuration_name", default_config_name);
    default_root.set_data("combinerOut", combinerOut);
    default_root.set_data("isRemoteEnabled", isRemoteEnabled);
    default_root.set_data("advancedCounters", advancedCounters);
    default_root.set_data("combinerOutTcpNum", combinerOutTcpNum);
    default_root.set_data("startEvNumber", startEvNumber);
    default_root.set_data("evNumberStep", evNumberStep);
    default_root.set_data("storageDir", storageDir);
    default_root.set_data("timeDistrEn", timeDistrEn);
    default_root.set_data("monitorPort", monitorPort);
}

EvBConfig::~EvBConfig()
{
}

void EvBConfig::read_config()
{
    root = BaseConfig::read_config(program_type, program_index, configuration_name);
    if(root.isEmpty()) return;

    program_index = root.get_data<QString>("program_index", program_index);
    runId.runNumber = root.get_data<int>("run_number", runId.runNumber);
    runId.runIndex = root.get_data<QString>("run_index", runId.runIndex);
    QString combinerOut_str = root.get_data<QString>("combinerOut");
    if(combinerOut_str == "file"){
        combinerOut = COMBINER_FILE;
    }
    if(combinerOut_str == "tcp"){
        combinerOut = COMBINER_TCP;
    }
    if(combinerOut_str == "idle"){
        combinerOut = COMBINER_IDLE;
    }
    isRemoteEnabled = root.get_data<bool>("isRemoteEnabled", isRemoteEnabled);
    advancedCounters = root.get_data<bool>("advancedCounters", advancedCounters);
    combinerOutTcpNum =  root.get_data<int>("combinerOutTcpNum", combinerOutTcpNum);
    evNumberStep = root.get_data<quint32>("evNumberStep", evNumberStep);
    startEvNumber = root.get_data<quint32>("startEvNumber", startEvNumber);
    storageDir = root.get_data<QString>("storageDir", storageDir);
    timeDistrEn = root.get_data<bool>("timeDistrEn", timeDistrEn);
    outputBufSize = root.get_data<quint64>("outputBufSize", outputBufSize);
    monitorPort = root.get_data<quint16>("monitorPort", monitorPort);

    int id=1;
    clients.clear();
    for(QString child_name : root.children().keys()){
        RootConfig child = root.child(child_name);
        ProgramDescription descr;
        descr.type = child.get_data<QString>("type");
        descr.index = child.get_data<QString>("index");
        bool readout = child.get_data<bool>("readout", true);
        if(!descr.isEmpty()){
            ClientConfig cc;
            cc.descr = descr;
            cc.enabled = readout;
            cc.id = id;
            quint64 inputBufSize = child.get_data<quint64>("inputBufSize", cc.inputBufSize);
            cc.inputBufSize = inputBufSize;
            clients[id] = cc;
            ++id;
        }
    }
}

void EvBConfig::write_config()
{
    root.remove_all_data();
    root.set_data("program_type", program_type);
    root.set_data("program_index", program_index);
    root.set_data("configuration_name", configuration_name);
    root.set_data("run_number", runId.runNumber);
    root.set_data("run_index", runId.runIndex);

    if(combinerOut == COMBINER_FILE){
        QString combinerOut_str = "file";
        root.set_data("combinerOut", combinerOut_str);
    }
    if(combinerOut == COMBINER_TCP){
        QString combinerOut_str = "tcp";
        root.set_data("combinerOut", combinerOut_str);
    }
    if(combinerOut == COMBINER_IDLE){
        QString combinerOut_str = "idle";
        root.set_data("combinerOut", combinerOut_str);
    }
    root.set_data("isRemoteEnabled", isRemoteEnabled);
    root.set_data("advancedCounters", advancedCounters);
    root.set_data("combinerOutTcpNum", combinerOutTcpNum);
    root.set_data("startEvNumber", startEvNumber);
    root.set_data("evNumberStep", evNumberStep);
    root.set_data("storageDir", storageDir);
    root.set_data("timeDistrEn", timeDistrEn);
    root.set_data("outputBufSize", outputBufSize);
    root.set_data("monitorPort", monitorPort);

    int i=1;
    for (const ClientConfig &cc: clients) {
        RootConfig child = RootConfig(QString("Client-%1").arg(i));
        child.set_data("type", cc.descr.type);
        child.set_data("index", cc.descr.index);
        child.set_data("readout", cc.enabled);
        child.set_data("inputBufSize", cc.inputBufSize);
        root.append_child(child);
        ++i;
    }
    BaseConfig::write_config(root);
}

void EvBConfig::reset_to_defaults()
{
    combinerOut = COMBINER_FILE;
    isRemoteEnabled = true;
    advancedCounters = false;
    combinerOutTcpNum = 0;
    startEvNumber = 0;
    evNumberStep = 1;
    storageDir = "";
    timeDistrEn = false;
    monitorPort = 0;
}

QString EvBConfig::getOutputStr() const
{
    switch (combinerOut) {
    case COMBINER_FILE: return "file";
    case COMBINER_TCP: return "tcp";
    case COMBINER_IDLE: return "idle";
    }
    return "unknown";
}

