#include "Config.h"

namespace {
const QString CONFIG_PREFIX = "device";
}

Config::Config(StartupOptions &start_opt):
    program_type(start_opt.program_type),
    program_index(start_opt.program_index),
    configuration_name(start_opt.configuration_name),
    show_log(true),
    hit_extension(0)
{
    update_root();
    default_root = root;
    default_root.set_data("configuration_name", QString("default"));
}

void Config::read_config()
{
    root = BaseConfig::read_config(program_type, program_index, configuration_name);
    if(root.isEmpty()){
        root.set_data("program_type", program_type);
        root.set_data("program_index", program_index);
        root.set_data("configuration_name", configuration_name);
        BaseConfig::write_config(root);
    }
    program_index = root.get_data<QString>("program_index");
    configuration_name = root.get_data<QString>("configuration_name");
    show_log = root.get_data<bool>("show_log");
    hit_extension = root.get_data<int>("hit_extension");

    device_configs.clear();
    for(QString child_name : root.children().keys()){
        if(child_name.split("_").at(0) == CONFIG_PREFIX){
            RootConfig root_child  = root.child(child_name);
            DeviceIndex device_index = DeviceIndex(root_child.get_data<quint64>("device_id"),
                                                   root_child.get_data<quint64>("device_serial"));

            DeviceConfig device_config;
            device_config.root_to_values(root.child(child_name));
            device_config.hit_extension = hit_extension;
            device_configs[device_index] = device_config;
        }
    }
}

void Config::write_config()
{
    update_root();
    BaseConfig::write_config(root);
}

void Config::update_root()
{
    root.remove_all_data();
    root.set_data("program_type", program_type);
    root.set_data("program_index", program_index);
    root.set_data("configuration_name", configuration_name);
    root.set_data("show_log", show_log);
    root.set_data("hit_extension", hit_extension);

    for(DeviceIndex device_index : device_configs.keys()){
        RootConfig root_child;
        root_child = device_configs[device_index].values_to_root();
        root_child.item_name = CONFIG_PREFIX + "_" + device_configs[device_index].device_index.getSerialStr();
        root.append_child(root_child);
    }
}


