#ifndef DEVICEWIDGET_H
#define DEVICEWIDGET_H

#include "DeviceConfig.h"

#include "QDoubleSpinBox"
#include "QSignalMapper"
#include "QWidget"

#include "MscDevice.h"

namespace Ui {
class DeviceWidget;
}

class QTableWidgetItem;
class QTableWidget;

class DeviceWidget : public QWidget
{
    Q_OBJECT
public:
    explicit DeviceWidget(DeviceConfig device_config, QWidget *parent = nullptr);
    ~DeviceWidget();
    void init_gui();
    DeviceIndex get_device_index() const { return device_config.device_index; }
    DeviceConfig get_device_config() const { return device_config;}

signals:
    void delete_hardware(DeviceIndex);
    void update_device_config(DeviceConfig);
    void update_status(DeviceStatus);
    void device_discovered(DeviceDescription) const;
    void destroyAll();
    void write_loot(QByteArray lut_data);
    void find_base_line();
    void soft_clear();

private slots:
    void on_pushButton_clicked();
    void on_pushButton_upload_clicked();
    void spin_box_channel_value_changed(double value);
    void on_doubleSpinBox_valueChanged(double thr);

    void update_status_device(DeviceStatus device_status);

private:
    void addLog(const QString &log);
    Ui::DeviceWidget *ui;

    QThread *device_thread;
    MscDevice *device;
    DeviceConfig device_config;
};

#endif // DEVICEWIDGET_H
