#include "QApplication"
#include "MainWindow.h"
#include "base/RunGuard.h"
#include "base/SyslogMessageSender.h"
#include "mongo/StartupOptions.h"
#include "mongo/BaseConfig.h"

const QString PROGRAM_TYPE = "MSC16VE-TM";

int main(int argc, char *argv[])
{
    QApplication a(argc, argv);
    a.setOrganizationDomain("local");
    a.setOrganizationName("AFI Electronics");
    a.setApplicationName(PROGRAM_TYPE);
    SyslogMessageSender::instance().install();

    #ifdef MONGO_DRIVERS
        mongocxx::instance insta{};
    #endif
    BaseConfig base_config;
    base_config.init(PROGRAM_TYPE);

    StartupOptions startup_options(PROGRAM_TYPE);
    startup_options.parse_options();

    QString program_index = startup_options.program_index;
    RunGuard runGuard(QString("%1_%2").arg(a.applicationName(), program_index));
    if(!runGuard.tryToRun()) {
        QString errStr = a.applicationName().append(" is already running; ");
        if(program_index.isEmpty())
            errStr += "in local run mode.";
        else
            errStr += QString("with run_index '%1'").arg(program_index);
        qFatal("%s", errStr.toStdString().c_str());
    }

    int res =1;
    MainWindow w(startup_options);
    QObject::connect(&SyslogMessageSender::instance(), SIGNAL(newMsg(QtMsgType,QString,QString)),
                     &w, SLOT(add_log(QtMsgType,QString,QString)));
    w.show();
    res = a.exec();

    SyslogMessageSender::instance().remove();
    return res;
}

