/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef FIRMWAREBUILDPARAMS_H
#define FIRMWAREBUILDPARAMS_H

#include <QtGlobal>

struct FirmwareBuildParams {
    FirmwareBuildParams()
        : ver(0), edge_rollover(false), two_level_trig(false), tih_log(false), multihist(false),
        nch(1), ntrig(1),
        spill_plot_ram_addr_bits(13), trig_hist_ram_addr_bits(13), level_cnt_bits(48), edge_cntr_bits(32),
        clk_hz(62500000) { }
    bool isValid() const {
        return true
               && ((ver == 0) || (ver == 0x81))
               && ((nch > 0) && (nch <= 16))
               && ((ntrig > 0) && (ntrig <= 16))
               && ((spill_plot_ram_addr_bits > 0) && (spill_plot_ram_addr_bits <= 24))
               && ((trig_hist_ram_addr_bits > 0) && (trig_hist_ram_addr_bits <= 24))
               && ((level_cnt_bits >= 16) && (level_cnt_bits <= 64))
               && ((edge_cntr_bits >= 16) && (edge_cntr_bits <= 64))
               && ((clk_hz > 0) && (clk_hz <= 1000000000))
            ;
    }
    quint8 ver;
    bool edge_rollover;
    bool two_level_trig;
    bool tih_log;
    bool multihist;
    quint8 nch;
    quint8 ntrig;
    quint8 spill_plot_ram_addr_bits;
    quint8 trig_hist_ram_addr_bits;
    quint8 level_cnt_bits;
    quint8 edge_cntr_bits;
    quint32 clk_hz;
};

#endif // FIRMWAREBUILDPARAMS_H
