
//    Copyright 2014-2015 Ivan Filippov
//    Copyright 2014-2015 Ilja Slepnev

//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include "RcConfig.h"

#include <QSettings>
#include <QDebug>
#include <QDesktopServices>
#include <QStringList>

RcConfig::RcConfig(const ConfigSelector &_cs)
    :
    BaseConfig (),
    cs(_cs),
    root(RootConfig("Root"))
{
    update_root();
    default_root = root;
    default_root.set_data("configuration_name", QString("default"));
}

void RcConfig::read_config()
{
    root.remove_all_data();
    root = BaseConfig::read_config(cs.program_type, cs.program_index, cs.configuration_name);

    if(root.isEmpty()){
        write_config();
        return;
    }

    clients.clear();
    channelDescr.clear();
    vmeTrigs.clear();

    cs.program_index = root.get_data<QString>("program_index", cs.program_index);
    rcTrigConfig.hostStr = root.get_data<QString>("host", rcTrigConfig.hostStr);
    quint64 serId = root.get_data<quint64>("rcSerial");
    if(serId) {
        rcTrigConfig.rcSerial = serId;
    }
    quint16 deviceId = root.get_data<quint16>("rcDeviceId");
    if(deviceId) {
        rcTrigConfig.rcDeviceId = deviceId;
    }
    rcTrigConfig.chNum = root.get_data<int>("chNum");
    rcTrigConfig.checkLvdsParity = root.get_data<bool>("checkLvdsParity");

    rcTrigConfig.checkBeforeProtection = root.get_data<bool>("checkBeforeProtection",
                                                              rcTrigConfig.checkBeforeProtection);
    rcTrigConfig.checkAfterProtection = root.get_data<bool>("checkAfterProtection",
                                                             rcTrigConfig.checkAfterProtection);

    rcTrigConfig.bapBlock = root.get_data<bool>("bapBlock");
    rcTrigConfig.trigWordMask = root.get_data<quint16>("trigWordMask");
    rcTrigConfig.lvdsMask = root.get_data<quint16>("lvdsMask");
    rcTrigConfig.trigRejectDelay = root.get_data<quint16>("trigRejectDelay");
    rcTrigConfig.beforeProtWidth = root.get_data<quint16>("beforeProtWidth");
    rcTrigConfig.afterProtWidth = root.get_data<quint16>("afterProtWidth");
    rcTrigConfig.plotSrc = root.get_data<quint16>("plotSrc");

    pc.runId.runIndex = root.get_data<QString>("run_index");
    pc.runId.runNumber = root.get_data<int>("runNumber");
    pc.localRunNumber = root.get_data<int>("localRunNumber");
    pc.allowRemoteControl = root.get_data<bool>("allowRemoteControl", pc.allowRemoteControl);
    pc.enableStatusDump = root.get_data<bool>("enableStatDump", pc.enableStatusDump);
    pc.statusDumpDirName = root.get_data<QString>("statDumpDirName", pc.statusDumpDirName);

    for(QString child_name : root.children().keys()){

        if(child_name == "Spill"){
            rcTrigConfig.spill.root_to_variables(root.child(child_name));
        }
        if(child_name == "Trigger"){
            rcTrigConfig.trig.root_to_variables(root.child(child_name));
        }

        if(child_name.split("_").at(0) == "Client"){
            RootConfig root_child = root.child(child_name);
            ClientConfig cc;
            cc.id = child_name.split("_").at(1).toInt();
            cc.program_type = root_child.get_data<QString>("program_type");
            cc.program_index = root_child.get_data<QString>("program_index");
            cc.readout = root_child.get_data<bool>("readout", true);
            cc.evNumCheck = root_child.get_data<bool>("evNumCheck", true);
            if(!cc.program_type.isEmpty() && !cc.program_index.isEmpty()){
                clients[cc.id] = cc;
            }
        }

        if(child_name.split("_").at(0) == "VmeTrig"){
            RootConfig root_child = root.child(child_name);
            quint64 serial = root_child.get_data<quint64>("serial");
            if(serial){
                VmeTrigConfig vc(serial);
                vc.readout = root_child.get_data<bool>("readout", true);
                vc.checkWr = root_child.get_data<bool>("checkWr", true);
                vmeTrigs[vc.serial] = vc;
            }
            else{
                 qDebug()<<"Can't parse serial number of Vme Trigger in config file; group="<<child_name;
            }

        }
        if(child_name == "ChannelDescriptions"){
            RootConfig root_child = root.child(child_name);
            for(QString ch : root_child.data().keys()){
                channelDescr[ch.toInt()] = root_child.get_data<QString>(ch);
            }
        }
    }
}

void RcConfig::write_config()
{
    update_root();
    BaseConfig::write_config(root);
}

void RcConfig::update_root()
{
    root.remove_all_data();

    // Config Selector
    root.set_data("program_type", cs.program_type);
    root.set_data("program_index", cs.program_index);
    root.set_data("configuration_name", cs.configuration_name);

    // Program Config
    root.set_data("run_index", pc.runId.runIndex);
    root.set_data("run_number", pc.runId.runNumber);
    root.set_data("localRunNumber", pc.localRunNumber);
    root.set_data("allowRemoteControl", pc.allowRemoteControl);
    root.set_data("enableStatDump", pc.enableStatusDump);
    root.set_data("statDumpDirName", pc.statusDumpDirName);

    //    rcTrigConfig
    root.set_data("host", rcTrigConfig.hostStr);
    root.set_data("hostTs", rcTrigConfig.hostTs);
    root.set_data("rcSerial", rcTrigConfig.rcSerial);
    root.set_data("rcDeviceId", rcTrigConfig.rcDeviceId);
    root.set_data("chNum", rcTrigConfig.chNum);
    root.set_data("checkLvdsParity", rcTrigConfig.checkLvdsParity);
    root.set_data("checkBeforeProtection", rcTrigConfig.checkBeforeProtection);
    root.set_data("checkAfterProtection",  rcTrigConfig.checkAfterProtection);
    root.set_data("bapBlock",  rcTrigConfig.bapBlock);
    root.set_data("trigRejectDelay", rcTrigConfig.trigRejectDelay);
    root.set_data("beforeProtWidth", rcTrigConfig.beforeProtWidth);
    root.set_data("afterProtWidth",  rcTrigConfig.afterProtWidth);
    root.set_data("lvdsMask", rcTrigConfig.lvdsMask);
    root.set_data("trigWordMask", rcTrigConfig.trigWordMask);
    root.set_data("plotSrc", rcTrigConfig.plotSrc);

    if(!channelDescr.isEmpty()){
        RootConfig root_child_channel_descr = RootConfig("ChannelDescriptions");
        bool not_empty = false;
        for(int ch : channelDescr.keys()){
            if(!channelDescr[ch].isEmpty()){
                root_child_channel_descr.set_data(QString::number(ch), channelDescr[ch]);
                not_empty = true;
            }
        }
        if(not_empty){
            root.append_child(root_child_channel_descr);
        }
    }

    int i=1;
    for (const auto &cc : clients.values()) {
        RootConfig root_child_client = RootConfig(QString("Client_%1").arg(i++));
        root_child_client.set_data("program_type", cc.program_type);
        root_child_client.set_data("program_index", cc.program_index);
        root_child_client.set_data("readout", cc.readout);
        root_child_client.set_data("evNumCheck", cc.evNumCheck);
        root_child_client.set_data("has_vme", cc.has_vme);
        root_child_client.set_data("protoId", static_cast<int>(cc.protoId));
        root.append_child(root_child_client);
    }

    i=1;
    for (const auto &vc : vmeTrigs.values()) {
        RootConfig root_child_vmetrig = RootConfig(QString("VmeTrig_%1").arg(i++));
        root_child_vmetrig.set_data("serial", QString("0x%1").arg(vc.serial,8,16,QChar('0')));
        root_child_vmetrig.set_data("readout", vc.readout);
        root_child_vmetrig.set_data("checkWr", vc.checkWr);
        root.append_child(root_child_vmetrig);
    }

    RootConfig root_child_trigger = rcTrigConfig.trig.variables_to_root();
    root.append_child(root_child_trigger);

    RootConfig root_child_spill = rcTrigConfig.spill.variables_to_root();
    root.append_child(root_child_spill);
}



