//
//    Copyright 2014-2015 Ivan Filippov
//    Copyright 2014-2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef CONFIG_H
#define CONFIG_H

#include <QDateTime>
#include <QMap>
#include <QString>
#include <QVector>

#include "daq-config/DaqConfig.h"
#include "mongo/BaseConfig.h"
#include "mongo/RootConfig.h"

#include "base/config/ConfigSelector.h"
#include "SpillConfig.h"
#include "TrigConfig.h"
#include "run-control-client-manager/Client.h"
#include "run-control-client-manager/ClientConfig.h"
#include "base/RcRunId.h"

struct VmeTrigConfig{
    VmeTrigConfig(quint64 ser=0):
        serial(ser),
        readout(true),
        checkWr(true)
    {}
    quint64 serial;
    bool readout;
    bool checkWr;
};

struct RcTrigConfig{
    RcTrigConfig():
        rcSerial(0),
        rcDeviceId(0),
        chNum(0),
        checkLvdsParity(false),
        checkBeforeProtection(false),
        checkAfterProtection(false),
        bapBlock(false),
        trigRejectDelay(0),
        beforeProtWidth(2000/16),
        afterProtWidth(2000/16),
        lvdsMask(0),
        trigWordMask(0),
        plotSrc(14)
    {}
    bool isSpillGate() const {
        return spill.source == SpillConfig::SPILL_GATE;
    }

    QString hostStr; // uses for quick start only
    QDateTime hostTs; // timestamp of hostStr update
    quint64 rcSerial;
    quint16 rcDeviceId;
    int chNum;
    bool checkLvdsParity;
    bool checkBeforeProtection;
    bool checkAfterProtection;
    bool bapBlock; // Logic Before/After protection (block or check)
    quint16 trigRejectDelay; // in 16ns steps
    quint16 beforeProtWidth; // in 16ns steps
    quint16 afterProtWidth; // in 16ns steps
    quint16 lvdsMask;
    quint16 trigWordMask;
    TrigConfig trig;
    SpillConfig spill;
    quint8 plotSrc;
};


struct RcProgramConfig
{
    bool runEnable = false;
    RcRunId runId;
    int localRunNumber = 0;
    bool allowRemoteControl = true;
    bool enableStatusDump = false;
    QString statusDumpDirName;
};

class QSettings;

class RcConfig  : public BaseConfig
{
public:
    RcConfig(const ConfigSelector &_cs);
    void read_config();
    void write_config();

    bool hasVme() const {
        for(auto cc : clients.values()) {
            if(cc.readout && cc.hasVme())
                return true;
        }
        return false;
    }

    ConfigSelector cs;
    RcProgramConfig pc;
    RcTrigConfig rcTrigConfig;
    QMap<int, ClientConfig> clients;
    QMap<int, QString> channelDescr;
    QMap<quint64, VmeTrigConfig> vmeTrigs;

    RootConfig root;
    RootConfig default_root;

private:
    void update_root();
};

#endif // CONFIG_H
