//
//    Copyright 2014-2015 Ivan Filippov
//    Copyright 2014-2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef RUNCONTROLREGS_H
#define RUNCONTROLREGS_H

#include <stdint.h>
#include "mregdev/mregdevice.h"

enum {
    REG_CSR_CTRL        = mlink::MregDeviceAdcm::ADCM16_CTRL_REG,

    // Run Control Core
    REG_RC_CTRL        = 0x80,
    REG_RC_CHANNEL_CNT = 0x81, // read-only
    REG_RC_TRIG_SRC    = 0x82,
    REG_RC_TRIG_STATE  = 0x83, // read-only
    REG_RC_XOFF_MASK   = 0x84,
    REG_RC_XOFF_STATE  = 0x85, // read-only
    REG_RC_TRIG_REJECT_DELAY = 0x86,
    REG_RC_XT_STATE    = 0x87, // read-only
    REG32_RC_TRIG_PERIODICAL = 0x88,
    REG32_RC_TRIG_RANDOM    = 0x8A,
    REG_RC_TRIG_WORD_MASK   = 0x8C,
    REG_RC_TRIG_BEFORE_AFTER_PROT_EN   = 0x8D, // 1st bit - before;  2nd bit - after
    REG_RC_TRIG_BEFORE_WIDTH   = 0x8E, // in 16 ns
    REG_RC_TRIG_AFTER_WIDTH   = 0x8F, // in 16 ns
    REG32_RC_SPILLPLOT_ADDR = 0x90, // read-only
    REG32_RC_SPILLPLOT_STEP = 0x92,
    REG32_RC_TRIGTIMEPLOT_STEP = 0x94, // deprecated since r.25460
    REG_RC_SPILLPLOT_SOURCE = 0x96,
    REG_RC_LVDS_TRIG_MASK   = 0x97,
    REG32_RC_TRIG_PDEAD    = 0x9A, // new since r.25367
    REG32_RC_TIH_S1    = 0x9C, // new since r.25460
    REG32_RC_TIH_S2    = 0x9E, // new since r.25460
    REG32_RC_BUILD_PARAM_0         = 0xA0, // new since r.25460
    REG32_RC_BUILD_PARAM_1         = 0xA2, // new since r.25460
    REG32_RC_BUILD_PARAM_2         = 0xA4, // new since r.25460
    REG32_RC_BUILD_PARAM_3         = 0xA6, // new since r.25460
    REG64_RC_GLOB_EV_NUM_LOAD      = 0xA8, // new since r.28813
    REG64_RC_GLOB_EV_NUM           = 0xAC, // new since r.28813

    // Spill Control Core
    REG32_SPILL_DUR_PHASE_1_MS     = 0xC0,
    REG32_SPILL_DUR_PHASE_2_MS     = 0xC2,
    REG32_SPILL_DUR_PHASE_3_MS     = 0xC4,
    REG32_SPILL_DUR_PHASE_4_MS     = 0xC6,
    REG32_SPILL_DUR_PHASE_5_MS     = 0xC8,
    REG32_SPILL_GATE_EXTEND_MS     = 0xCA,
    REG32_SPILL_COUNT              = 0xCC, // read-only
    REG32_SPILL_CURRENT_TIME_MS    = 0xCE, // read-only
    REG32_SPILL_CTRL               = 0xD0,
    REG32_SPILL_STATE              = 0xD2, // read-only
    REG32_SPILL_SOFTSPILL_PERIOD_MS = 0xD4,
    REG32_SPILL_DETECT_MIN_MS      = 0xD6,
    REG32_SPILL_DETECT_MAX_MS      = 0xD8,
    REG32_SPILL_TTL_LAST_PERIOD_MS = 0xDA, // read-only
    REG32_SPILL_WR_LAST_PERIOD_MS  = 0xDC, // read-only

    REG64_SPILL_NCU_TAI_TIMESTAMP  = 0xE0,

    // Run Control Counters  // read-only
    REG64_CNT_BASE   = 0x800, // see CNT_INDEX
    REG64_CNT_XOFF_HI_BASE  = 0xA00, // cnt[8]
    REG64_CNT_XOFF_LOW_BASE = 0xB00, // cnt[9]
    REG64_CNT_LVDS_BASE  = 0xC00, // ParityErr,L0Cnt,Befor,After
//    REG64_CNT_EVENT_NUMBER  = 0xD00, // changed since 23162
    REG64_CNT_XOFF_EVENTS_BASE = 0xD00,
};

enum CNT_INDEX {
    CNT_INDEX_TIME_TOTAL = 0,
    CNT_INDEX_EDGE_TOTAL = 1,
    CNT_INDEX_TIME_RUN   = 2,
    CNT_INDEX_EDGE_RUN   = 3,
    CNT_INDEX_TIME_SPILL = 4,
    CNT_INDEX_EDGE_SPILL = 5,
    CNT_INDEX_TIME_RUN_SPILL = 6,
    CNT_INDEX_EDGE_RUN_SPILL = 7
};

const uint16_t REG64_CNT_BLOCK_SIZE = 16; // 16 64-bit registers in each block
const uint16_t REG64_CNT_BLOCK_COUNT = 8;
const uint16_t REG64_CNT_TOTAL_NUM = REG64_CNT_BLOCK_SIZE * REG64_CNT_BLOCK_COUNT;

typedef union {
    uint16_t all;
    struct {
        uint16_t clear: 1;
        uint16_t reserved_13_1: 13;
        uint16_t softspill: 1;
        uint16_t run: 1;
    } bit;
} reg_spill_ctrl_t;

typedef union {
    uint16_t all;
    struct {
        uint16_t ttl_ok: 1;
        uint16_t wr_ok: 1;
        uint16_t reserved_2_3: 2;
        uint16_t sel: 4; // 2 bits before r.25461
        uint16_t phase: 5;
        uint16_t reserved_13_14_15: 3;
    } bit;
} reg_spill_status_t;

typedef union {
    uint16_t all[4];
    struct {
        uint16_t ver: 1;
        uint16_t wr_ok: 1;
        uint16_t reserved_2_3: 2;
        uint16_t sel: 4; // 2 bits before r.25461
        uint16_t phase: 5;
        uint16_t reserved_13_14_15: 3;
    } bit;
} reg_build_param_t;

#endif // RUNCONTROLREGS_H
