#ifndef SLOWCONTROLSERVER_H
#define SLOWCONTROLSERVER_H

#include <QObject>
#include <QScopedPointer>

class QTcpServer;
class QTcpSocket;
class TrigStatus;

const QString SC_STATE_ON = "ON";
//const QString SC_STATE_OFF = "OFF";
//const QString SC_STATE_CLOSE = "CLOSE";
//const QString SC_STATE_OPEN = "OPEN";
//const QString SC_STATE_INSERT = "INSERT";
//const QString SC_STATE_EXTRACT = "EXTRACT";
//const QString SC_STATE_MOVING = "MOVING";
const QString SC_STATE_STANDBY = "STANDBY";
const QString SC_STATE_FAULT = "FAULT";
const QString SC_STATE_INIT = "INIT";
const QString SC_STATE_RUNNING = "RUNNING";
//const QString SC_STATE_ALARM = "ALARM";
//const QString SC_STATE_DISABLE = "DISABLE";
//const QString SC_STATE_UNKNOWN = "UNKNOWN";
//const QString SC_STATE_WARNING = "WARNING";

class SpillPlotData;
class TriggerHistogramData;
class RcRunId;

class SlowControlServerPrivate;

class SlowControlServer : public QObject
{
    Q_OBJECT
public:
    explicit SlowControlServer(QObject *parent = nullptr);
    virtual ~SlowControlServer();

private slots:
    void updateConnections();
    void readCommandRequest();
    void onDisconnected();
    void responce() const;

public slots:
    void trigStatusUpdated(const TrigStatus &daqStatus);
    void spillPlotUpdated(const SpillPlotData &plot);
    void triggerHistogramUpdated(const TriggerHistogramData &trigData);
    void slowControlStateUpdated(QString _state, QString _status);
    void startListen(bool daqState);
    void onRunStart(const RcRunId &runId);

signals:
    void connectionStatusUpdated(quint16 tcpPort, bool listening, bool portInUse) const;

private:
    void init();
    void updateStatus() const;

    Q_DECLARE_PRIVATE(SlowControlServer)
    QScopedPointer<SlowControlServerPrivate> const d_ptr;
    QTcpServer * const qTcpServer;
    QTcpSocket *tcpSocket = nullptr;
    bool listening = false;
    quint16 tcpPort;
    bool portInUse = false;
};

#endif // SLOWCONTROLSERVER_H
