//
//    Copyright 2014-2015 Ivan Filippov
//    Copyright 2014-2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "TrigChannelConfig.h"
#include <QTableWidget>
#include <QHeaderView>
#include <QCheckBox>
#include <QLabel>
#include <QDebug>
#include <QPixmap>
#include <QProgressBar>
#include <QScrollBar>

#include "RcConfig.h"
#include "TrigChannelState.h"

namespace  {
enum TRIG_STAT_COLS {
    TRIG_CH_CONF_COL_CH_NUM,
    TRIG_CH_CONF_COL_DESCR,
    TRIG_CH_CONF_COL_CB_XOFF,
    TRIG_CH_CONF_COL_LED_XOFF,
    TRIG_CH_CONF_COL_TIME_XOFF,
    TRIG_CH_CONF_COL_TIMEOUT_XOFF,
    TRIG_CH_CONF_COL_TIMEOUT_READ,
    TRIG_CH_CONF_COL_TOTAL
};

}

TrigChannelConfig::TrigChannelConfig(QTableWidget *tableWidget) :
    QObject(tableWidget),
    table(tableWidget),
    chNum(0),
    xOffLed(0),
    timeoutLed(0),
    changesAllowed(true)
{
    setTableHeaders();
    updateTableSize();
    updateColVisibility();
}

void TrigChannelConfig::setChNumer(int newChNum)
{
    if (newChNum == chNum) return;
    if (newChNum < chNum) {
//        while(table->rowCount()>newChNum)
//            table->removeRow(0);
        table->setRowCount(newChNum);
        chNum = newChNum;
    }
    if (newChNum > chNum) {
        for(int ch=chNum; ch<newChNum; ++ch){
//            const int row = 0;
            const int row = ch;
            table->insertRow(row);
            QTableWidgetItem *item = new QTableWidgetItem(QString("%1").arg(ch+1, 2, 10, QChar(' ')));
            item->setFlags(item->flags() ^ Qt::ItemIsEditable);
            table->setItem(row, TRIG_CH_CONF_COL_CH_NUM, item);

            item = new QTableWidgetItem(config->channelDescr[ch]);
            table->setItem(row, TRIG_CH_CONF_COL_DESCR, item);
            connect(table, &QTableWidget::cellChanged,
                    this, &TrigChannelConfig::tableCellChanged);

            QCheckBox *cb = new QCheckBox(table);
            connect(cb, &QCheckBox::toggled,
                    this, &TrigChannelConfig::onXOffToggled);
            table->setCellWidget(row, TRIG_CH_CONF_COL_CB_XOFF, cb);

            QPixmap p = QPixmap(":/icons/red-led.png");
            p = p.scaledToHeight(16, Qt::SmoothTransformation);
            QLabel *label = new QLabel(table);
            label->setPixmap(p);
            label->setEnabled(false);
            table->setCellWidget(row, TRIG_CH_CONF_COL_LED_XOFF, label);

            if (0) {
                item = new QTableWidgetItem();
                item->setFlags(item->flags() ^ Qt::ItemIsEditable);
                table->setItem(row, TRIG_CH_CONF_COL_TIME_XOFF, item);
            }
            QProgressBar * const bar = new QProgressBar(table);
            bar->setVisible(false);
            table->setCellWidget(row, TRIG_CH_CONF_COL_TIME_XOFF, bar);

            item = new QTableWidgetItem();
            item->setFlags(item->flags() ^ Qt::ItemIsEditable);
            item->setTextColor(Qt::red);
            table->setItem(row, TRIG_CH_CONF_COL_TIMEOUT_XOFF, item);
            item = new QTableWidgetItem();
            item->setFlags(item->flags() ^ Qt::ItemIsEditable);
            table->setItem(row, TRIG_CH_CONF_COL_TIMEOUT_READ, item);
        }
        chNum = newChNum;
    }
    updateTableSize();
}

void TrigChannelConfig::updateConfig()
{
    setChNumer(config->rcTrigConfig.chNum);

    QWidget *w;
    QCheckBox *cb;
    for(int ch=0; ch<chNum; ++ch) {
        const int row = chToRow(ch);
        bool xOff = config->rcTrigConfig.trig.xOffMask&(1<<ch);
        w =table ->cellWidget(row, TRIG_CH_CONF_COL_CB_XOFF);
        Q_ASSERT(w);
        cb = reinterpret_cast<QCheckBox*>(w);
        Q_ASSERT(cb);
        cb->setChecked(xOff);
//        cb->setEnabled(trig);

        table ->item(row, TRIG_CH_CONF_COL_DESCR)->setText(config->channelDescr[ch]);
    }
    table->resizeColumnToContents(TRIG_CH_CONF_COL_DESCR);
    updateTableSize();
    xOffConf = config->rcTrigConfig.trig.xOffMask;
}

void TrigChannelConfig::init(RcConfig *conf)
{
    config = conf;
    bool block = changesAllowed;
    changesAllowed = false;
    updateConfig();
    changesAllowed = block;
}

void TrigChannelConfig::runUpdated()
{
    for(int row = 0; row < table->rowCount(); ++row) {
        table->cellWidget(row, TRIG_CH_CONF_COL_CB_XOFF)->setEnabled(!config->pc.runEnable);
    }
}

void TrigChannelConfig::onXOffToggled(bool checked)
{
    for(int ch=0;ch<chNum;++ch){
        if(sender() == table->cellWidget(chToRow(ch), TRIG_CH_CONF_COL_CB_XOFF)){
            if(checked)
                xOffConf |= 1<<ch;
            else
                xOffConf &= ~(quint16)(1<<ch);
            if(changesAllowed)
                emit configUpdated(xOffConf);
            return;
        }
    }
}
bool TrigChannelConfig::xOffEnabled(int ch) const
{
    return config->rcTrigConfig.trig.xOffMask&(1<<ch);
}

void TrigChannelConfig::updateChState(const TrigChannelState &chState)
{
//    qDebug() << QString("TrigChannel state: %1 %2 %3").arg(trig,chNum,2,QChar('0'))
//              .arg(xOff,chNum,2,QChar('0'))
//                .arg(timeout,chNum,2,QChar('0'));
    // maximum xoff time among enabled channels
    double maxXOffTime = chState.globalXOffTimePerEvent();
    for(int ch=0;ch<chNum;++ch) {
        if (xOffEnabled(ch))
            if (chState.xOffTimePerEvent(ch) > maxXOffTime)
                maxXOffTime = chState.xOffTimePerEvent(ch);
    }
    for(int ch=0;ch<chNum;++ch) {
        int row = chToRow(ch);
        quint16 mask = 1<<ch;
        bool xOffEnabled = xOffConf & mask;

        quint16 curXOffState = xOffEnabled && (chState.xOff & mask);

        if((xOffLed & mask) != curXOffState){
            QPixmap p(":/icons/red-led.png");
            p = p.scaledToHeight(16, Qt::SmoothTransformation);
            QLabel *label = reinterpret_cast<QLabel*>(table->cellWidget(row,TRIG_CH_CONF_COL_LED_XOFF));
            label->setPixmap(p);
            if(label)
                label->setEnabled(curXOffState);
            else
                qDebug()<<"no label";
        }
        QProgressBar * const bar = qobject_cast<QProgressBar *>(table->cellWidget(row, TRIG_CH_CONF_COL_TIME_XOFF));
        if(bar) {
            bool visible = true;
            // microseconds per event
            double maxValue = maxXOffTime * 1e9;
            if (chState.globalEventCount == 0 || maxValue <= 0 || !xOffEnabled)
                visible = false;
            double value = chState.xOffTimePerEvent(ch) * 1e9;

//            bar->setMaximum(visible ? maxValue : 1);
//            bar->setValue(value);
            bar->setMaximum(visible ? 100 : 1);
            bar->setValue(value/maxValue*100);
            bar->setVisible(visible);
            QString format = QString("%1 μs").arg(1e-3*value, 0, 'g', 3);
            bar->setFormat(format);
        }
//        ui->tableWidget->item(row, TRIG_CH_CONF_COL_TIME_XOFF)->setText(xOffTimeStr[ch]);
        QColor col;
        if(chState.timeout & mask)
            col = QColor(xOffEnabled ? Qt::yellow : Qt::red);
        else
            col = QColor(Qt::transparent);
        if(chState.timeoutXoff.contains(ch) && chState.timeoutXoff[ch])
            table->item(row, TRIG_CH_CONF_COL_TIMEOUT_XOFF)->setText(QString::number(chState.timeoutXoff[ch]));
        else
            table->item(row, TRIG_CH_CONF_COL_TIMEOUT_XOFF)->setText("");
        if(chState.timeoutRead.contains(ch) && chState.timeoutRead[ch])
            table->item(row, TRIG_CH_CONF_COL_TIMEOUT_READ)->setText(QString::number(chState.timeoutRead[ch]));
        else
            table->item(row, TRIG_CH_CONF_COL_TIMEOUT_READ)->setText("");

        table->item(row, TRIG_CH_CONF_COL_CH_NUM)->setData(Qt::BackgroundColorRole, col );
        table->item(row, TRIG_CH_CONF_COL_DESCR)->setData(Qt::BackgroundColorRole, col );
    }
    xOffLed = chState.xOff;
    timeoutLed = chState.timeout;
    updateColVisibility();
//    updateTableSize();
}

void TrigChannelConfig::tableCellChanged(int row, int col)
{
    if(col!=TRIG_CH_CONF_COL_DESCR)
        return;
    QString descr = table->item(row, col)->text();
    int ch = rowToCh(row);
    if(config->channelDescr[ch] == descr)
        return;
    config->channelDescr[ch] = descr;
    table->resizeColumnToContents(TRIG_CH_CONF_COL_DESCR);
    updateTableSize();
    if(changesAllowed){
        config->write_config();
    }


}

void TrigChannelConfig::setTableHeaders()
{
#if QT_VERSION >= 0x050000
    table->horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
    table->verticalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
#else
    table->horizontalHeader()->setResizeMode(QHeaderView::ResizeToContents);
    table->verticalHeader()->setResizeMode(QHeaderView::ResizeToContents);
#endif

    table->setColumnCount(TRIG_CH_CONF_COL_TOTAL);
    table->setHorizontalHeaderItem(TRIG_CH_CONF_COL_CH_NUM, new QTableWidgetItem("Ch#"));
    table->setHorizontalHeaderItem(TRIG_CH_CONF_COL_DESCR, new QTableWidgetItem("Descr"));
    table->setHorizontalHeaderItem(TRIG_CH_CONF_COL_CB_XOFF, new QTableWidgetItem("XOff"));
    table->setHorizontalHeaderItem(TRIG_CH_CONF_COL_LED_XOFF, new QTableWidgetItem(""));
    table->setHorizontalHeaderItem(TRIG_CH_CONF_COL_TIME_XOFF, new QTableWidgetItem("XOff time"));
    table->setHorizontalHeaderItem(TRIG_CH_CONF_COL_TIMEOUT_XOFF, new QTableWidgetItem("XOff timeout"));
    table->setHorizontalHeaderItem(TRIG_CH_CONF_COL_TIMEOUT_READ, new QTableWidgetItem("Read timeout"));
//    table->hideColumn(TRIG_CH_CONF_COL_CH_NUM);
    table->verticalHeader()->hide();
}

void TrigChannelConfig::addRow(int row)
{
    QTableWidget *t = table;
    t->setRowCount(row);
}

void TrigChannelConfig::updateTableSize()
{
//    int height=table->horizontalHeader()->height();
//    for(int row=0;row<table->rowCount();++row){
//        height += table->rowHeight(row);
//    }
//    height += 5;
//    tableWidget->setMinimumHeight(height);

    int width = 5 + table->verticalHeader()->width()
            + table->verticalScrollBar()->width();
    for(int col=0;col<table->columnCount();++col){
        width += table->columnWidth(col);
    }
    table->setMinimumWidth(width);
}

void TrigChannelConfig::updateColVisibility()
{
    const int rows = table->rowCount();
    bool hideHi=true, hideLow=true;
    bool ok;
    quint64 val;
    for(int row=0; row<rows; ++row) {
        if(hideHi){
            val = table->item(row, TRIG_CH_CONF_COL_TIMEOUT_XOFF)->text().toLongLong(&ok);
            if(val && ok)
                hideHi = false;
        }
        if(hideLow){
            val = table->item(row, TRIG_CH_CONF_COL_TIMEOUT_READ)->text().toLongLong(&ok);
            if(val && ok)
                hideLow = false;
        }
        if(!hideHi && !hideLow) break;
    }
    bool doResize = false;
    if(table->isColumnHidden(TRIG_CH_CONF_COL_TIMEOUT_XOFF) != hideHi) {
        table->setColumnHidden(TRIG_CH_CONF_COL_TIMEOUT_XOFF, hideHi);
        doResize = true;
    }
    if(table->isColumnHidden(TRIG_CH_CONF_COL_TIMEOUT_READ) != hideLow) {
        table->setColumnHidden(TRIG_CH_CONF_COL_TIMEOUT_READ, hideLow);
        doResize = true;
    }
    if(doResize)
        updateTableSize();
}
