//
//    Copyright 2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "TrigConfig.h"

#include <QDebug>
#include <QSettings>

TrigConfig::TrigConfig()
    :
      xOffMask(0),
      trigOpen(true),
      trigSrc(1),
      trigTimerFreq(1000.0),
      trigRandomFreq(1000.0),
      trigRandomDeadTimeUs(10.0)
{
}

bool TrigConfig::operator==(const TrigConfig &rhs) const
{
    return true
            && xOffMask == rhs.xOffMask
            && trigOpen == rhs.trigOpen
            && trigSrc == rhs.trigSrc
            && trigTimerFreq == rhs.trigTimerFreq
            && trigRandomFreq == rhs.trigRandomFreq
            && trigRandomDeadTimeUs == rhs.trigRandomDeadTimeUs
            ;
}

QDebug operator <<(QDebug stream, const TrigConfig &trig)
{
    return stream
            << QString("XOFF    mask: %1").arg(trig.xOffMask, 8, 16, QChar('0')) << endl
            << QString("Trigger source: %1").arg(trig.trigSrc) << endl
            << QString("Trigger open: %1").arg(trig.trigOpen) << endl
            << QString("Triger Timer freq: %1").arg(trig.trigTimerFreq) << endl
            << QString("Triger Random freq: %1").arg(trig.trigRandomFreq) << endl
            << QString("Triger Random dead time: %1").arg(trig.trigRandomDeadTimeUs);
}

bool TrigConfig::root_to_variables(RootConfig root)
{
    xOffMask = root.get_data<quint16>("xOffMask", xOffMask);
    trigOpen = root.get_data<bool>("trigOpen", trigOpen);
    trigSrc = root.get_data<quint16>("trigSrc", trigSrc);
    trigTimerFreq = root.get_data<double>("trigTimerFreq", trigTimerFreq);
    trigRandomFreq = root.get_data<double>("trigRandomFreq", trigRandomFreq);
    trigRandomDeadTimeUs = root.get_data<double>("trigRandomDeadTimeUs", trigRandomDeadTimeUs);
    return true;
}

RootConfig TrigConfig::variables_to_root()
{
    RootConfig root_to_return = RootConfig("Trigger");
    root_to_return.set_data("trigSrc", trigSrc);
    root_to_return.set_data("trigTimerFreq", trigTimerFreq);
    root_to_return.set_data("trigRandomFreq", trigRandomFreq);
    root_to_return.set_data("trigRandomDeadTimeUs", trigRandomDeadTimeUs);
    root_to_return.set_data("xOffMask", xOffMask);
    root_to_return.set_data("trigOpen", trigOpen);
    return root_to_return;
}

