#ifndef TRIGDEVICE_H
#define TRIGDEVICE_H

#include "FirmwareBuildParams.h"
#include "RcConfig.h"
#include "RunControlRegs.h"
#include "SpillPlotData.h"
#include "TrigStatisticsWriter.h"
#include "TrigStatus.h"
#include "TriggerHistogramData.h"
#include "mregdev/qtmregdevice.h"

#include <QVector>
#include <QMap>
#include <QMetaType>

class SpillConfig;
class TrigConfig;
namespace mlink {class DeviceAttr;}


class TrigDevice : public QtMregDevice
{
    Q_OBJECT
public:
    explicit TrigDevice(RcTrigConfig conf, quint32 ip=0, QObject *parent = nullptr);
    quint8 getTrigSrcCount() const;
    FirmwareBuildParams readFirmwareBuildParams();
    int spillPlotSize() const { return 1 << fbp.spill_plot_ram_addr_bits;}
    int triggerHistogramSize() const {return 1 << fbp.trig_hist_ram_addr_bits;}
    mlink::RegOpVector setControlStateCmd(bool trigOpen, bool softSpill);
    mlink::RegOpVector setTrigTimerFreqCmd(double freq);
    mlink::RegOpVector setTrigRandomFreqCmd(double freq);
    mlink::RegOpVector setTrigRandomDeadTimeCmd(double microseconds);
    mlink::RegOpVector setSpillCmd(const SpillConfig &spill);
    mlink::RegOpVector setTrigCmd(const TrigConfig &trig);
    static double getClockFreqHz() { return 62500000; }
    static double getTrigTimerMinFreqHz() { return getClockFreqHz() / (quint32)(-1); }
    static double getTrigTimerMaxFreqHz() { return getClockFreqHz() / 2; }
    virtual bool checkLiveMagic();
    bool isSupportParityCheck() const;
    bool isL0Module() const;
    bool isSupportedXoffCounters() const;
    void setStatusPollTimeMs(int ms);

signals:
    void updateStatus(const TrigStatus &status);
    void spillPlotUpdated(const SpillPlotData &);
    void triggerHistogramUpdated(const TriggerHistogramData &);
    void chNumberUpdated(int chNum);
    void trigSrcCntUpdated(quint8 trigScrCnt);
    void setupWritten(int setupSeqNum);
    void updateTabState(bool isOk, int tabIndex=0);
    void trigEvNumResponse(int checkIndex, quint64 evNum);
    void devEvNumResponse(int checkIndex, quint64 devSer, quint64 evNum);
    void setStatusDumpDirName(const QString &);
    void setStatusDumpEnable(bool);

public slots:
    void writeSetup(RcTrigConfig cfg, int setupSeqNum=0);
    void resetAll();
    void onRunStart();
    void clearStats(bool onStartRun);
    void evNumRequest(int checkIndex, bool local);

protected slots:
    void trigStatusPoll();
    virtual void on_deviceConnected();
    virtual void on_deviceDisconnected();

protected:
    quint16 globEvNumRegister() const;

private:
    void evNumReset();
    SpillPlotData readSpillPlotRam();
    TriggerHistogramData readTriggerTimeHistogramRam();

    TrigStatisticsWriter tsw;
    RcTrigConfig lastConfig;
    QTimer * const trigStatusTimer;
    FirmwareBuildParams fbp;
    TrigStatus status;
    reg_spill_ctrl_t regCtrl;
    int statusPollTimeMs;
};

#endif // TRIGDEVICE_H
