/*
**    Copyright 2020 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "TrigHistView.h"
#include "ui_TrigHistView.h"

#include "RcConfig.h"
#include "TrigDevice.h"
#include "rc_gui_utils.h"

TrigHistView::TrigHistView(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::TrigHistView)
{
    ui->setupUi(this);

    connect(this, &TrigHistView::spillPlotUpdated,
            ui->spillPlot, &SpillPlot::updateSpillPlot);
    connect(this, &TrigHistView::triggerHistogramUpdated,
            ui->triggerHistogram, &TriggerHistogram::updateHistogram);

    connect(this, &TrigHistView::trigSrcCntUpdated,
            this, &TrigHistView::updatePlotTypeList);

    initHistWidget();
}

TrigHistView::~TrigHistView()
{
    delete ui;
}

void TrigHistView::init(RcConfig *conf)
{
    config = conf;
    restoreGuiFromConfig();
}

void TrigHistView::initHistWidget()
{
    int i = 0;
    ui->comboBoxSpillStep->insertItem(i++, "100 ms", 100e-3);
    ui->comboBoxSpillStep->insertItem(i++, "10 ms", 10e-3);
    ui->comboBoxSpillStep->insertItem(i++, "1 ms", 1e-3);
    ui->comboBoxSpillStep->insertItem(i++, "350 us", 350e-6);
    ui->comboBoxSpillStep->insertItem(i++, "100 us", 100e-6);
//    connect(ui->comboBoxSpillStep, SIGNAL(activated(int)),
//            this, SLOT(on_comboBoxSpillStep_activated(int)));

    i = 0;
    ui->comboBoxHistStep->insertItem(i++, "100 ns", 100e-9);
    ui->comboBoxHistStep->insertItem(i++, "1 us", 1e-6);
    ui->comboBoxHistStep->insertItem(i++, "10 us", 10e-6);
    ui->comboBoxHistStep->insertItem(i++, "100 us", 100e-6);
    ui->comboBoxHistStep->insertItem(i++, "1 ms", 1e-3);
}

void TrigHistView::restoreGuiFromConfig()
{
    {
        double value = config->rcTrigConfig.spill.spillPlotStep / TrigDevice::getClockFreqHz();
        int index = findComboBoxNearestItem(*ui->comboBoxSpillStep, value);
        WidgetSignalBlocker(ui->comboBoxSpillStep);
        ui->comboBoxSpillStep->setCurrentIndex(index);
    }
    {
        double value = config->rcTrigConfig.spill.trigTimeStep / TrigDevice::getClockFreqHz();
        int index = findComboBoxNearestItem(*ui->comboBoxHistStep, value);
        WidgetSignalBlocker(ui->comboBoxHistStep);
        ui->comboBoxHistStep->setCurrentIndex(index);
    }
    updatePlotTypeList(0);
}

void TrigHistView::on_comboBoxSpillStep_activated(int index)
{
//    config->rcTrigConfig.spill.spillPlotStep = index;
    bool ok = false;
    double value = ui->comboBoxSpillStep->itemData(index).toDouble(&ok);
    if (!ok)
        return;
    config->rcTrigConfig.spill.spillPlotStep = value * TrigDevice::getClockFreqHz();
    emit rcConfigUpdated();
}

void TrigHistView::on_comboBoxHistStep_activated(int index)
{
//    config->rcTrigConfig.spill.trigTimeStep = index;
    bool ok = false;
    double value = ui->comboBoxHistStep->itemData(index).toDouble(&ok);
    config->rcTrigConfig.spill.trigTimeStep = value * TrigDevice::getClockFreqHz();
    if (!ok)
        return;
    emit rcConfigUpdated();
}

void TrigHistView::updatePlotTypeList(quint8 trigSrcCnt)
{
    QComboBox * const cb = ui->comboBoxPlotType;
    if(cb->count() == (trigSrcCnt+2)) return;
    cb->blockSignals(true);
    cb->clear();
    cb->setEnabled(trigSrcCnt);
    switch (trigSrcCnt) {
    case 0:
        break;
    case 1:
        cb->insertItem(0, "External TTL", 0);
        break;
    default:
        for(int i=0; i<trigSrcCnt; ++i)
            cb->insertItem(i, QString("External TTL %1").arg(i+1), i);
        break;
    }
    cb->insertItem(14, "Available", 14);
    cb->insertItem(15, "Accepted", 15);
    cb->blockSignals(false);

    if(trigSrcCnt){
        if(config->rcTrigConfig.plotSrc>=14){
            cb->setCurrentIndex(config->rcTrigConfig.plotSrc-14+trigSrcCnt);
        } else
            cb->setCurrentIndex(qMin(config->rcTrigConfig.plotSrc, trigSrcCnt));
    }
}

void TrigHistView::on_comboBoxPlotType_currentIndexChanged(int)
{
    bool ok;
    quint8 srcIndex = ui->comboBoxPlotType->currentData().toUInt(&ok);
    if(ok) {
        config->rcTrigConfig.plotSrc = srcIndex;
        emit rcConfigUpdated();
    }
}

