/*
**    Copyright 2020 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef TRIGHISTVIEW_H
#define TRIGHISTVIEW_H

#include <QWidget>

namespace Ui {
class TrigHistView;
}

class RcConfig;
class SpillPlotData;
class TriggerHistogramData;

class TrigHistView : public QWidget
{
    Q_OBJECT

public:
    explicit TrigHistView(QWidget *parent = nullptr);
    ~TrigHistView();
    void init(RcConfig *conf);

public slots:
    void updatePlotTypeList(quint8 trigSrcCnt);

signals:
    void spillPlotUpdated(const SpillPlotData &);
    void triggerHistogramUpdated(const TriggerHistogramData &);
    void rcConfigUpdated();
    void trigSrcCntUpdated(quint8 trigScrCnt);

private slots:
    void on_comboBoxSpillStep_activated(int index);
    void on_comboBoxHistStep_activated(int index);
    void on_comboBoxPlotType_currentIndexChanged(int);

private:
    void restoreGuiFromConfig();
    void initHistWidget();

    Ui::TrigHistView *ui;
    RcConfig * config = nullptr;
};

#endif // TRIGHISTVIEW_H
