//
//    Copyright 2014-2015 Ivan Filippov
//    Copyright 2014-2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef TRIGMAINFORM_H
#define TRIGMAINFORM_H

#include <QWidget>

#include "RcConfig.h"

class TrigStatus;
class SpillPlotData;
class TriggerHistogramData;
//class Config;
class TrigChannelConfig;
class TrigChannelState;

namespace Ui {
class TrigMainForm;
}

class TrigMainForm : public QWidget
{
    Q_OBJECT

public:
    explicit TrigMainForm(QWidget *parent = nullptr);
    ~TrigMainForm();
    void init(RcConfig *conf);
    void updateExtTrigGroup();
    void openTrig();
    void closeTrig();
    void clearStats();
    static QString timeStr(quint64 t);
    static double timeFromClocks(quint64 t);
    void updateRemoteBlocking(bool remoteConnected);

private slots:
    void displaySpillSource(const TrigStatus &status);
    void on_radioButtonTrigOpen_toggled(bool open);
    void configUpdated(quint16 xOff);
    void on_pushButtonClearStat_clicked();
    void on_pushButtonResetAll_clicked();
    // Trig Input
    void on_doubleSpinBoxTrigTimerFreq_valueChanged(double arg1);
    void on_doubleSpinBoxTrigRandomFreq_valueChanged(double arg1);
    void on_doubleSpinBoxTrigRandomDeadTime_valueChanged(double arg1);
    void on_checkBoxTrigIn_toggled(bool checked);
    void on_checkBoxTrigIn2_toggled(bool checked);
    void on_checkBoxTrigIn3_toggled(bool checked);
    void on_checkBoxTrigIn4_toggled(bool checked);
    void on_checkBoxTrigLvds_toggled(bool checked);
    void on_checkBoxTrigPeriod_toggled(bool checked);
    void on_checkBoxTrigRandom_toggled(bool checked);
    // Spill
    void on_spinBoxSpillDurPhase1_valueChanged(int arg1);
    void on_spinBoxSpillDurPhase2_valueChanged(int arg1);
    void on_spinBoxSpillDurPhase3_valueChanged(int arg1);
    void on_spinBoxSpillDurPhase4_valueChanged(int arg1);
    void on_spinBoxSpillDurPhase5_valueChanged(int arg1);
    // Soft Spill
    void on_groupBoxSoftSpill_toggled(bool arg1);
    void on_spinBoxPeriodTimer_valueChanged(int arg1);

//    void on_checkBoxSpillSourceAuto_toggled(bool);
    void on_radioButtonSpillSourceAuto_toggled(bool checked);
    void on_radioButtonSpillSourceTtl_toggled(bool checked);
    void on_radioButtonSpillSourceWr_toggled(bool checked);
    void on_radioButtonSpillSourceTimer_toggled(bool checked);
    void on_radioButtonSpillSourceGate_toggled(bool checked);
    void on_spinBoxLvdsMask_valueChanged(int val);
    void on_spinBoxTrigWord_valueChanged(int val);
    void on_spinBoxTrigRejectDelay_valueChanged(int val);
    void on_checkBoxBeforeProtect_toggled(bool checked);
    void on_checkBoxAfterProtect_toggled(bool checked);
    void on_radioButtonBapBlock_toggled(bool checked);
    void on_spinBoxBeforeWidth_valueChanged(int val);
    void on_spinBoxAfterWidth_valueChanged(int val);

    void setSpillMode(const SpillConfig::spill_sel_t &sel);

    void writeConfig();

public slots:
    void trigStatusUpdated(const TrigStatus &status);
    void setChNumber(int chNum);
    void checkLvdsParityToggled(bool checked);
    void spillPlotStepUpdated();
    void setControlEnable(bool enabled);

signals:
//    void updateLeds(quint16 curState, quint16 trig, quint16 xOff, QVector<QString> chTime);
    void updateChState(const TrigChannelState &);
    void trigSetupUpdated(RcTrigConfig cfg, int setupSeqNum);
    void runStart();
    void clearTrigStats(bool onStartRun);
    void resetAll();

    // proxy signals
    void trigSrcCntUpdated(quint8 trigScrCnt);
    void spillPlotUpdated(const SpillPlotData &);
    void triggerHistogramUpdated(const TriggerHistogramData &);
    void rcConfigUpdated();

private:
    void sendTrigStatusMetrics(const TrigStatus &prevTrigStatus, const TrigStatus &status);
    void setTrigSrcMask(quint16 mask, bool checked);
    void updateTableSize();
    void restoreGuiFromConfig();

    Ui::TrigMainForm *ui;
    RcConfig * config;
    int cfgSeqNumber;
    TrigChannelConfig * trigChannelConfig;
    TrigStatus * const prevTrigStatus;
    bool controlEnabled;
};

#endif // TRIGMAINFORM_H
