/*
**    Copyright 2020 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "TrigStatisticsWriter.h"

#include <QDateTime>
#include <QDebug>
#include <QSettings>

void TrigStatisticsWriter::writeFile()
{
    if (!enable)
        return;
    // write file
    QString fileName = QString("rc_%1_%2.txt")
            .arg(QDateTime::currentDateTime().toString("yyyyMMdd_hhmmss"))
            .arg(status.spillCount);

    QSettings settings(dirName + "/" + fileName, QSettings::IniFormat);
    settings.beginGroup("main");
    settings.setValue("ident", status.ident);
    settings.setValue("spillCount", status.spillCount);
    settings.setValue("SpillTimeMs", status.spillCurrentTimeMs);
    settings.endGroup();
    settings.beginGroup("spill_plot");
    if (!spd.data.isEmpty()) {
        settings.setValue("stepSec", spd.stepSec);
        settings.setValue("startOffset", spd.startOffset);
        settings.beginGroup("0");
        QStringList sl;
        Q_FOREACH(quint32 val, spd.data)
            sl += QString("%1").arg(val);
        settings.setValue("data", sl.join(","));
        settings.endGroup();
    }
    settings.endGroup();
    settings.beginGroup("histograms");
    Q_FOREACH(const TriggerHistogramChannelData &thcd, thd.histograms) {
        if (thcd.isEmpty())
            continue;
        settings.beginGroup(QString("%1").arg(thcd.channel));
        QStringList sl;
        Q_FOREACH(quint32 val, thcd.data)
            sl += QString("%1").arg(val);
        settings.setValue("data", sl.join(","));
        settings.setValue("overflowCount", QString("%1").arg(thcd.overflowCount));
        settings.endGroup();
    }
    settings.endGroup();
    settings.sync();
    if (settings.status() != QSettings::NoError) {
        qWarning() << QString("Error writing status file '%1': %2").arg(settings.fileName()).arg(settings.status());
    }
}
void TrigStatisticsWriter::setStatusDumpEnable(bool val)
{
    enable = val;
}

void TrigStatisticsWriter::setStatusDumpDirName(const QString &str)
{
    dirName = str;
}

void TrigStatisticsWriter::writeStatisticsDump(const TrigStatus &newStatus, const SpillPlotData &newSpd, const TriggerHistogramData &newThd)
{
    bool endOfReadout = false;
    // graphs are cleared on beginning of phase 2
    if (status.spillState.reg.bit.phase == 4) {
        if (newStatus.spillState.reg.bit.phase < 2 || newStatus.spillState.reg.bit.phase == 5) {
            // flush existing graphs
            spd = newSpd;
            thd = newThd;
            endOfReadout = true;
        }
        if (newStatus.spillState.reg.bit.phase == 4) {
            if (status.spillCount != newStatus.spillCount) {
                endOfReadout = true; // spill incremented
            }
        } else {
            endOfReadout = true;
        }
    }
    if (endOfReadout)
        writeFile();
    status = newStatus;
    spd = newSpd;
    thd = newThd;
}
