//
//    Copyright 2014-2015 Ivan Filippov
//    Copyright 2014-2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//


#ifndef VMETRIGDEVICE_H
#define VMETRIGDEVICE_H

#include "mregdev/qtmregdevice.h"
#include "run-control-client-manager/RunStates.h"

class RcRunId;

class VmeTrigStatus
{
public:
    VmeTrigStatus() :
        liveMagikOk(false),
        connected(false)
    { clear(); }
    bool isWrTimeValid() const { return deviceStatus & 0x1; }
    void clear() {
        temp  = 0;
        evNum = 0;
        wrSyncFail  = 0;
        wrLinkError = 0;
        deviceStatus= 0;
        busyTimer= 0;
        idleTimer= 0;
    }
    QString ident;
    bool liveMagikOk;
    bool connected;
    double temp;
    quint64 evNum;
    quint32 wrSyncFail;
    quint32 wrLinkError;
    quint16 deviceStatus;
    quint64 busyTimer;
    quint64 idleTimer;
};

class VmeTrigDevice : public QtMregDevice
{
    Q_OBJECT
public:
    explicit VmeTrigDevice(quint64 serId, QObject *parent = nullptr);

signals:
    void updateStatus(const VmeTrigStatus &status);
    void evNumResponse(int checkIndex, quint64 serial, quint64 evNum) const;
    void runStateUpdated(quint64 serial, RUN_STATES state, const RcRunId &runId);

public slots:
    void deleteTriger(quint64 serial); // serial==0 - for all
    virtual void statusPoll();
    void resetEvNum(const RcRunId &runId);
    void vmeReadoutUpdated(quint64 serial, bool readout);
    void evNumRequest(int checkIndex);

private:
    VmeTrigStatus status;
    const quint64 serial;
    bool readout;
};

#endif // VMETRIGDEVICE_H
