//
//    Copyright 2014-2015 Ivan Filippov
//    Copyright 2014-2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "MainWindow.h"
#include "base/SyslogMessageSender.h"
#include "metric-sender/MetricSender.h"
#include "base/RunGuard.h"
#include "mongo/StartupOptions.h"
#include "mongo/BaseConfig.h"

#include <iostream>
#include <QCoreApplication>
#include <QApplication>
#include <QString>
#include <QDebug>
#include <QSplashScreen>
#include <QPixmap>
#include <QStyle>
#include <QStyleFactory>
#include <QLocale>

const QString PROGRAM_TYPE = "RunControl";

int main(int argc, char *argv[])
{
    QCoreApplication::setOrganizationName("AFI Electronics");
    QCoreApplication::setOrganizationDomain("local");
    QCoreApplication::setApplicationName("Run Control");
    SyslogMessageSender::instance().install();

    int ret = 1;
    try{
        QApplication a(argc, argv);
        #ifdef MONGO_DRIVERS
            mongocxx::instance insta{};
        #endif
        BaseConfig base_config;
        base_config.init(PROGRAM_TYPE);
        StartupOptions startup_options(PROGRAM_TYPE);
        startup_options.parse_options();

        RunGuard runGuard(QString("%1_%2").arg(a.applicationName(), startup_options.program_index));
        if(!runGuard.tryToRun()) {
            QString errStr = a.applicationName().append(" is already running; ");
            if(startup_options.program_index.isEmpty())
                errStr += "in local run mode.";
            else
                errStr += QString("with cfgIndex '%1'").arg(startup_options.program_index);
            qFatal("%s", errStr.toStdString().c_str());
        }

        if(QStyleFactory::keys().contains("Fusion")) {
            a.setStyle(QStyleFactory::create("Fusion"));
        } else
            if(QStyleFactory::keys().contains("Plastique")) {
                a.setStyle(QStyleFactory::create("Plastique"));
            } else
                if(QStyleFactory::keys().contains("Windows")) {
                    a.setStyle(QStyleFactory::create("Windows"));
                }

        QSplashScreen splash;
        splash.show();
        splash.showMessage("Starting Run Control");
        MetricSender::instance().install();

        if(startup_options.list == false){
            MainWindow w(startup_options);

            w.show();

            splash.hide();
            ret = a.exec();
        }

    } catch (std::exception &e) {
        std::cerr << "Exception: " << e.what () << std::endl;
    }
    SyslogMessageSender::instance().remove();

    return ret;
}
