#ifndef TCPCLIENT_H
#define TCPCLIENT_H

#include <QHostAddress>
#include <QTimer>
#include <QElapsedTimer>

#include <mldiscover/DeviceIndex.h>

class QTcpSocket;
class QSemaphore;

class TcpClient : public QObject
{
    Q_OBJECT
public:
    TcpClient(QSemaphore *analyzerSem, QObject *parent=nullptr);
    ~TcpClient();

public slots:
    void setAddress(QHostAddress, quint16 port);

private slots:
    void requestNewData();
    void readData();
    void displayError(QAbstractSocket::SocketError socketError);
    void connected();
    void disconnected();
    void sendStatus();

signals:
    void dataAccuired(QByteArray);
    void connectedStateChanged(bool state);
    void addLog(QString);
    void statusUpdated(bool connOk, double dataRate) const;

private:
    void startReadOut();
    void stopReadOut();

    QSemaphore * const analyzerSem; // free space in analyzer queue
    bool enabled;
    QTcpSocket * const tcpSocket;
    QHostAddress hostAddr;
    quint16 hostPort;
    QTimer reconnectTimer;
    QTimer delayReadTimer;
    QTimer statusTimer;
    int connectingAttempts;
    QElapsedTimer statusPeriod;
    qint64 dataRecieved;
    qint64 lastDataRecieved;
};

#endif // TCPCLIENT_H
