#include "AdjustDialog.h"
#include "ui_AdjustDialog.h"

#include <QComboBox>
#include "QDebug"
#include <math.h>

AdjustDialog::AdjustDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::AdjustDialog),
    block_write(false)
{
    ui->setupUi(this);
    qRegisterMetaType<Adjust_setup>();
    ui->checkBox_rc_compression->setChecked(true);
    for(int i=0; i<=0b111; ++i)
        ui->comboBox_readout_fifo->addItem(QString::number(std::pow(2, i+1)), i);

    ui->comboBox_lr->addItems(QStringList()
                               << "100 ps"
                               << "200 ps"
                               << "400 ps"
                               << "800 ps"
                               << "1.6 ns"
                               << "3.12 ns"
                               << "6.25 ns"
                               << "12.5 ns");

    ui->checkBox_pairing_mode->setChecked(false);

    ui->comboBox_dll_mode->addItems(QStringList()
                                      << "40 MHz"
                                      << "80 MHZ"
                                      << "320 Mhz");

    ui->comboBox_dll_clock->addItems(QStringList()
                                      << "clock_40"
                                      << "pll_clock_40"
                                      << "pll_clock_160"
                                      << "pll_clock_320"
                                      << "aux_clock");


    ui->comboBox_core_clock_source->addItems(QStringList()
                                             << "clock_40"
                                             << "pll_clock_80"
                                             << "pll_clock_160"
                                             << "aux_clock");
}

AdjustDialog::~AdjustDialog()
{
    delete ui;
}

void AdjustDialog::setConfig(Adjust_setup s)
{
    adjust_set = s;
    block_write = true;
    ui->comboBox_readout_fifo->setCurrentIndex(s.readout_fifo_size);
    ui->checkBox_rc_compression->setChecked(s.mode_rc_compression);
    ui->comboBox_lr->setCurrentIndex(s.leading_resolution);
    ui->checkBox_pairing_mode->setChecked(s.enable_pair);
    ui->comboBox_dll_mode->setCurrentIndex(s.dll_mode);
    ui->comboBox_dll_clock->setCurrentIndex(s.dll_clock);
    ui->comboBox_core_clock_source->setCurrentIndex(s.core_clock_source);
    block_write = false;
}

void AdjustDialog::on_comboBox_readout_fifo_currentIndexChanged(int index)
{
    bool ok;
    int v = ui->comboBox_readout_fifo->itemData(index).toInt(&ok);
    Q_ASSERT(ok && index == v);
    adjust_set.readout_fifo_size = static_cast<quint8>(v);
    write_setup();
}

void AdjustDialog::on_checkBox_rc_compression_clicked(bool checked)
{
    adjust_set.mode_rc_compression = checked;
    write_setup();
}

void AdjustDialog::on_comboBox_lr_currentIndexChanged(int index)
{
    adjust_set.leading_resolution = static_cast<quint16>(index);
    write_setup();
}

void AdjustDialog::on_checkBox_pairing_mode_clicked(bool checked)
{
    adjust_set.enable_pair = checked;
    write_setup();
}

void AdjustDialog::on_comboBox_dll_mode_currentIndexChanged(int index)
{
    adjust_set.dll_mode = static_cast<quint16>(index);
    write_setup();
}

void AdjustDialog::on_comboBox_dll_clock_currentIndexChanged(int index)
{
    adjust_set.dll_clock = static_cast<quint16>(index);
    write_setup();
}

void AdjustDialog::on_comboBox_core_clock_source_currentIndexChanged(int index)
{
    adjust_set.core_clock_source = static_cast<quint16>(index);
    write_setup();
}

void AdjustDialog::on_pushButton_OK_clicked()
{
    hide();
}

void AdjustDialog::write_setup()
{
    if(block_write) return;
    emit adjust_setup_changed(adjust_set);
}

void AdjustDialog::activate_dialog()
{
    emit adjust_setup_changed(adjust_set);
}
