//
//    Copyright 2014 AFI Electronics
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef TDCREGS_H
#define TDCREGS_H

#include "mregdev/mregdevice.h"

// Control registers
enum {
    REG_DEVICE_CTRL = mlink::MregDeviceAdcm::ADCM16_CTRL_REG,//0x40, //64
    REG_TEMPERATURE = mlink::MregDeviceAdcm::ADCM16_TEMPERATURE_REG, //0x4A, // 74
    REG_FW_REV = mlink::MregDeviceAdcm::ADCM16_FW_REV_REG, //0x4A, // 74
    REG_HPTDC_CHIP_ERROR_FLAGS = 0x56, // RO
    REG_CLOCK_STATUS = 0x56, // RO
    REG_SYS_CLK_SOURCE = 0x57, // RO
};

// Trigger control & statistic
enum {
    REG_TRIG_CSR = 0x100,
    REG_TRIG_SRC = 0x101,
    REG32_TRIG_TIMER_PERIOD = 0x102, // 32bit RW
    REG64_TRIG_EVENT_NUM_LOAD = 0x104, // 64bit RW (since 28451)
    REG32_TRIG_EVENT_NUM_LOAD = 0x104, // 32bit RW
    REG_TRIG_QUEUE = 0x106, // RO (moved to 10C on 28451)
    REG64_TRIG_CUR_EV_NUM = 0x108, // 64bit RO (since 28451)
    REG_TRIG_BUSY_STATUS = 0x109, // RO (moved to 10D on 28451)
    REG_TRIG_QUEUE_NEW = 0x10C, // 64bit RO (since 28451)
    REG_TRIG_BUSY_STATUS_NEW = 0x10D, // RO (since 28451)
//    REG64_TRIG_CNT_ALL = 0x110, // 64bit RO (deprecated on 28451)
    REG64_TRIG_CNT_CANDID = 0x114, // 64bit RO (moved to 0x700 since 28451)
    REG64_TRIG_CNT_ACCEPTED = 0x118, // 64bit RO (moved to 0x700 since 28451)
};

enum {
    REG_TRIG_CSR_BIT_CNT_RESET = 1,
//    REG_TRIG_CSR_BIT_RUN_CNT_RESET = 2,
    REG_TRIG_CSR_BIT_CNT_LOCK = 4,
};

enum {
    REG_TRIG_SRC_BIT_TIMER = 1,
    REG_TRIG_SRC_BIT_LEMO = 2,
    REG_TRIG_SRC_BIT_TTC = 4,
};

// TDC Core Registers
enum {
    REG32_TDC_CH_EN_1 = 0x200,
    REG32_TDC_CH_EN_2 = 0x202,
    REG32_TDC_GEN_CTRL = 0x220,
    REG32_TDC_READ_CTRL = 0x222,
    REG32_TDC_MEASURE_CTRL = 0x224,
    REG32_TDC_TRIG_WIN_SETUP = 0x226,
    REG32_TDC_GENERAL_STATUS = 0x22C
//    REG32_TDC_RC_ADJUST_SETUP = 0x228 //unused
};

// HPTDCs Status Registers
enum {
    REG_HPTDC1_STATUS_1 = 0x280,
    REG_HPTDC1_STATUS_2 = 0x281,
    REG_HPTDC1_STATUS_3 = 0x282,
    REG_HPTDC1_STATUS_4 = 0x283,

    REG_HPTDC2_STATUS_1 = 0x284,
    REG_HPTDC2_STATUS_2 = 0x285,
    REG_HPTDC2_STATUS_3 = 0x286,
    REG_HPTDC2_STATUS_4 = 0x287
};

enum {
    REG32_TDC_GEN_CTRL_BIT_STANDBY = 1
};

// WR Time Emulator Registers
enum {
    REG_TAI_0 = 0x400, // tai sec 15:0
    REG_TAI_1 = 0x401, // tai sec 31:16
    REG_TAI_2 = 0x402, // tai sec 39:32
    REG_TAI_UPDATE = 0x403, // any write for update
};

// Run Statistic Counters
enum {
    REG_RUN_STAT_BASE                     = 0x700,
    REG64_RUN_STAT_RUN_TIME               = 0x0,
    REG64_RUN_STAT_LEMO_Input_Pulse_Count = 0x4,
    REG64_RUN_STAT_TTC_Input_Pulse_Count  = 0x8,
    REG64_RUN_STAT_VXS_Input_Pulse_Count  = 0xC,
    REG64_RUN_STAT_RESERV                 = 0x10,
    REG64_RUN_STAT_TRIG_CANDID            = 0x14,
    REG64_RUN_STAT_ACCEPT                 = 0x18,
    REG64_RUN_STAT_BLOCKED                = 0x1C,

    REG64_RUN_STAT_BUSY_TOTAL             = 0x40,
    REG64_RUN_STAT_BUSY_TDC               = 0x44,
    REG64_RUN_STAT_BUSY_READOUT           = 0x48,

    REG_RUN_STAT_CNT                      = REG64_RUN_STAT_BUSY_READOUT/4+1
};
//Statistics Readout CSR
enum {
    REG_STAT_BASE = 0x300,
    REG_STAT_BASE_NEW = 0x900,
    REG_STAT_CTRL = 0x00,
    REG_STAT_PERIOD = 0x02,
    REG_STAT_BLOCK_SIZE = 0x04,
    REG_STAT_RAM_ADDR = 0x05,
    REG_STAT_RAM_DATA = 0x06
};

//Error Counters
enum {
    REG64_HWIP_LINK_DOWN = 0x1000,
    REG64_REGIO_ERR = 0x1004,
    REG64_SFP_ERR = 0x1008,
    REG64_HWIP_ERR = 0x100C,
    REG64_TX_ERR = 0x1010,
    REG64_RX_ERR = 0x1014,
};

//  == Board Level Registers ==
// MCU SPI Slave
enum {
    REG_BMC_DEV_ID        = 0x4000,
    REG_BMC_ONBOARD_PLL_STATUS = 0x4001,
    REG_BMC_PLL_UNLOCK_CNT= 0x4002, // [12:0] - unlock counter
    REG_BMC_TEMP_1        = 0x4003, // pll temp
    REG_BMC_TEMP_2        = 0x4004, // hptdc_group_a temp
    REG_BMC_TEMP_3        = 0x4005, // hptdc_group_b temp
    REG_BMC_TEMP_4        = 0x4006, // hptdc_group_c temp
    REG_BMC_VERSION       = 0x4008,
    REG_BMC_REVERSION     = 0x4009,
    REG_BMC_SYSTEM_STATUS = 0x400A, // [1:0] - 0: unknown, 1: normal, 2: warning, 3: critical
    REG_BMC_POWER_STATUS  = 0x400B,
    REG_BMC_PLL_STATUS    = 0x400C,

    REG_BMC_VOL_TDC_A    = 0x4010, // mV
    REG_BMC_CUR_TDC_A    = 0x4011, // mA
    REG_BMC_VOL_TDC_B    = 0x4012,
    REG_BMC_CUR_TDC_B    = 0x4013,
    REG_BMC_VOL_TDC_C    = 0x4014,
    REG_BMC_CUR_TDC_C    = 0x4015,

    REG_BMC_VOL_BP_5     = 0x4016, // Backplane +5V supply voltage, mV
    REG_BMC_CUR_BP_5     = 0x4017, // Backplane +5V supply current, mA
    REG_BMC_VOL_BP_33    = 0x4018, // Backplane +3.3V supply voltage, mV
    REG_BMC_CUR_BP_33    = 0x4019, // Backplane +3.3V supply current, mA

    REG_BMC_VOL_TDC_D    = 0x401A,
    REG_BMC_CUR_TDC_D    = 0x401B,
};

// On-Board Clock CSR
enum {
    REG_ON_BOARD_CLK_BASE   = 0x4300,
    REG_ON_BOARD_CLK_CSR    = REG_ON_BOARD_CLK_BASE + 0x0,
    REG_ON_BOARD_CLK_STATUS = REG_ON_BOARD_CLK_BASE + 0x1,
    REG_ON_BOARD_CLK_SOURCE = REG_ON_BOARD_CLK_BASE + 0x2,
};
enum {
    REG_DEVICE_CTRL_BIT_RUN = 0x8000
};

#endif // TDCREGS_H
