#ifndef TDCSTATUS_H
#define TDCSTATUS_H

#include <QColor>
#include <QMetaType>
#include <QMap>

#include "base/bmc/BmcStatus.h"
#include "mldiscover/DeviceDescription.h"

#include "TdcStatistic.h"

struct HptdcStatus {
    quint16 regValues[4]; // 4 16-bit status per one HPTDC

    bool isOk() const;
    bool isDllLock() const;
    bool hasError() const;
    QString getSummary() const;
};

struct TdcStatus
{
    TdcStatus() :
        deviceId(0),
        serialId(0),
        mstreamMultiAck(false),
        mstreamHwBufSize(8)
    {
        clear();
    }
    void clear() {
        processed = false;
        bmc_status.clear();
        connected = false;
        runFinished=true;
        temp.clear();
        status=0;
        pllStatus=0;
        hptdcStatus.clear();
        hptdcChipErrorFlags=0;
        clockStatus=0;
        sysClockSource=0;
        busyStatus=0;
        trigQueue=0;
        curEvNum=0;
        statistic.clear();
    }
    DeviceIndex getIndex() const { return DeviceIndex(deviceId, serialId); }
    bool isVxsClockOk() const { return clockStatus & 0x4; }
    QString getClockStatus() const;
    bool isMStreamRestartNeeded(const TdcStatus& prevSt) const
    {
        return mstreamMultiAck != prevSt.mstreamMultiAck ||
                mstreamHwBufSize != prevSt.mstreamHwBufSize;
    }

    bool processed;
    quint16 deviceId;
    quint64 serialId;
    QString fwStr; // Reg 0x56/0x57, FW_VER/FW_REV, 16-bit, firmware version/revision number, RO
    BmcStatus bmc_status;
    bool connected;
    bool runFinished;
    QMap<int, double> temp;
    quint16 status; // 0x2 TM Time Valid
//    quint64 wrTime; // ?
    quint16 pllStatus;
    bool mstreamMultiAck;
    quint8 mstreamHwBufSize;
    QMap<int, HptdcStatus> hptdcStatus; // index by HPTDC id
    quint16 hptdcChipErrorFlags;
    quint16 clockStatus;
    quint16 sysClockSource;
    quint16 busyStatus;
    quint16 trigQueue;
    quint64 curEvNum;

    TdcStatistic statistic;
};
Q_DECLARE_METATYPE(TdcStatus)

#endif // TDCSTATUS_H
