#include "MainWindow.h"
#include "mongo/BaseConfig.h"
#include "mongo/StartupOptions.h"
#include "base/RunGuard.h"
#include "base/SyslogMessageSender.h"
#include "metric-sender/MetricSender.h"

#include <QApplication>
#include <QStyleFactory>
#include <QCommandLineParser>

const QString PROGRAM_TYPE = "TDC2-CTRL";

int main(int argc, char *argv[])
{
    QApplication a(argc, argv);
    a.setOrganizationName("AFI Electronics");
    a.setOrganizationDomain("local");
    a.setApplicationName(PROGRAM_TYPE);
    SyslogMessageSender::instance().install();

    #ifdef MONGO_DRIVERS
        mongocxx::instance insta{};
    #endif
    BaseConfig base_config;
    base_config.init(PROGRAM_TYPE);
    StartupOptions startup_options(PROGRAM_TYPE);
    startup_options.parse_options();

    RunGuard runGuard(QString("%1_%2").arg(a.applicationName(), startup_options.program_index));
    if(!runGuard.tryToRun()) {
        QString errStr = a.applicationName().append(" is already running; ");
        if(startup_options.program_index.isEmpty())
            errStr += "in local run mode.";
        else
            errStr += QString("with run_index '%1'").arg(startup_options.program_index);
        qFatal("%s", errStr.toStdString().c_str());
    }

    if(QStyleFactory::keys().contains("Fusion")) {
        a.setStyle(QStyleFactory::create("Fusion"));
    } else
        if(QStyleFactory::keys().contains("Plastique")) {
            a.setStyle(QStyleFactory::create("Plastique"));
        } else
            if(QStyleFactory::keys().contains("Windows")) {
                a.setStyle(QStyleFactory::create("Windows"));
            }

    int res =1;
    MetricSender::instance().install();

    if((startup_options.list == false) && (startup_options.version == false)){
        MainWindow w(startup_options);
        QObject::connect(&SyslogMessageSender::instance(), &SyslogMessageSender::newMsg,
                         &w, &MainWindow::addLogFromSender);
        w.show();
        res = a.exec();
    }
    return res;
}
