#ifndef DEVICESETUP_H
#define DEVICESETUP_H
#include "mldiscover/DeviceIndex.h"
#include "DeviceCalibration.h"
#include "mongo/RootConfig.h"

#include <QMetaType>

class QSettings;

typedef union {
    quint16 all;
    struct {
        unsigned int timer_en: 1;
        unsigned int ttl_en: 1;
        unsigned int ttc_en: 1;
        unsigned int selftrig_en: 1;
        unsigned int unused2: 12;
    } bit;
    bool root_to_variables(RootConfig root)
    {
        bit.timer_en =  root.get_data<uint>("timer_en", bit.timer_en);
        bit.ttl_en = root.get_data<uint>("ttl_en", bit.ttl_en);
        bit.ttc_en = root.get_data<uint>("ttc_en", bit.ttc_en);
        bit.selftrig_en = root.get_data<uint>("vxs_en", bit.selftrig_en);
        return true;
    }
    RootConfig variables_to_root()
    {
        RootConfig root_to_return;
        root_to_return.set_data("timer_en", bit.timer_en);
        root_to_return.set_data("ttl_en", bit.ttl_en);
        root_to_return.set_data("ttc_en", bit.ttc_en);
        root_to_return.set_data("vxs_en", bit.selftrig_en);
        return root_to_return;
    }
} reg_trig_ctrl;

class DeviceSetup
{
public:
    DeviceSetup();
    bool root_to_variables(RootConfig root, QMap<int, int> &thrOffset);
    RootConfig variables_to_root(DeviceCalibration cal);
    bool loadSettings(QSettings &s);
    void saveSettings(QSettings &s, DeviceCalibration deviceCalibration) const;
    void setThreshold(int thr);
    int getNCh() const { return nch; }
    void setNCh(int n);
    bool isGain4() const;

    bool enabled;
    bool run;

    int adcPreWin;
    int readCellNumber;
    int latency; // in 24ns steps
    int matchWin; // in 24ns steps
    bool hptdcStandby;  // not used
    bool invertInput; // not used
    bool invert_thr_trig; // not used
    bool invert_zs_thr; // not used

    bool zsEn;
    quint32 trigPeriod; // in 8ns step
    quint16 selfTrigDelay; // in 8ns step
    int maxSampleCnt; // doesn't store

    reg_trig_ctrl trigsetup;
    DeviceIndex index;
    QMap<int, bool> chTdcEn;
    QMap<int, bool> chAdcEn;
    QMap<int, bool> chTrigEn; // self-trig
    QMap<int, int> chThr; // in mV
    QMap<int, bool> chInv;
    QMap<int, bool> chGain;

private:
    int nch;
};
Q_DECLARE_METATYPE(DeviceSetup)

#endif // DEVICESETUP_H
