#ifndef HISTMNGRWIDGET_H
#define HISTMNGRWIDGET_H

#include <QMap>
#include <QWidget>
#include <QElapsedTimer>

#include "HistMngrCfg.h"
#include "HistogramItem.h"
#include "mldiscover/DeviceIndex.h"
#include "histogramm/Histogramm.h"

namespace Ui {
class HistMngrWidget;
}

class DeviceIndex;
class DeviceData;
class QwtPlotCurve;
class QTime;
class QTimer;
class QSemaphore;

class TqdcEvent;
class CompleteEvent;

struct InlStat
{
    QElapsedTimer time;
    QMap<int, double> count;
};

class HistMngrWidget : public QWidget
{
    Q_OBJECT

public:
    explicit HistMngrWidget(QWidget *parent = 0);
    ~HistMngrWidget();
    void init(QSemaphore *_histSem, bool isInlEnabled = false);
    static QVector<double> makeDNLCorrectionTable(const QVector<double> code_density);
    void updateInlCorr(const DeviceIndex &index, const QMap<int, QVector<double> > &corrMap);

signals:
    void inlCorrUpdated(const DeviceIndex &index, const QMap<int, QVector<double> > &corrMap);

public slots:
    void dataUpdated(const DeviceData &data, bool);
    void matchWinUpdated(int matchWin);
    void requestInlCorr(const DeviceIndex &index);

private slots:
    void refreshCurrentPlot(bool force=false);
    void refreshCurrentPlotBypass();
    void on_comboBoxGroup_currentIndexChanged(int);
    void on_checkBoxHistGroupEn_toggled(bool checked);
    void on_spinBoxCh_valueChanged(int);
    void on_groupBoxStopCh_toggled(bool val);
    void on_spinBoxStopCh_valueChanged(int ch);
    void on_radioButtonMin_toggled(bool checked);
    void on_radioButtonMax_toggled(bool checked);
    void on_pushButtonClear_clicked();
    void on_pushButtonClearAll_clicked();
    void on_checkBoxEnable_toggled(bool checked);
    void on_groupBoxFit_toggled(bool checked);

private:
    void dataProcessingTdc(const CompleteEvent &ev, int &tdcHits, double &tdcPrevTime);
    void dataProcessingAdc(const CompleteEvent &ev, int &adcHits, double &adcPrevTime);
    void fillDoublePulse(const TqdcEvent &stopEv,
                         const std::map<int, QVector<TqdcEvent> > &tqdcEvents);
    void updateFit(const Histogramm &curHist);
    int getCurCh() const;
    HistCfg getCurHist(); // return current histCfg
    HistCfg onCurHistChanged(); // update GUI for current hist & return histCfg

    Ui::HistMngrWidget *ui;
    QElapsedTimer lastRefreshTime;
    QTimer * const refreshTimer;
    QMap<int, HistogramItem *> histQwtItems;
    QMap<int, QwtPlotCurve *> inlCurvs;
    QMap<int, Histogramm> histTRoot;
    QSemaphore * histSem;
    HistMngrCfg cfg;
    DeviceIndex inlDeviceIndex;
    QMap<int, QVector<double> > curDevCorrMap;
    InlStat inlStat;
};

#endif // HISTMNGRWIDGET_H
