#ifndef TQDCDEVICE_H
#define TQDCDEVICE_H

#include "mregdev/qtmregdevice.h"
#include "DeviceSetup.h"
#include "DeviceCalibration.h"
#include <QMetaType>

class QtMregDevice;

struct TqdcStatistic
{
    void clear() {
        evNumAll=0;
        evNumCandidateRun=0;
        evNumAccepted=0;
        timeValidLoss=0;
        rxCodeError=0;
        pllUnlockCnt=0;
    }

    quint64 evNumAll;
    quint64 evNumCandidateRun;
    quint64 evNumAccepted;
    quint32 timeValidLoss;
    quint32 rxCodeError;
    quint32 pllUnlockCnt;
};

struct HptdcStatus {
    quint16 regValues[4];

    bool isOk() const;
    bool isDllLock() const;
    bool hasError() const;
    QString getSummary() const;
};

struct TqdcStatus
{
    TqdcStatus() :
        mstreamMultiAck(false),
        mstreamHwBufSize(8)
    {
        clear();
    }
    void clear() {
        statistic.clear();
        status=0;
        adcStatus=0;
        adcSelfTestOk = false;
        connected = false;
        runFinished=true;
    }

    DeviceIndex index;
    QString fwStr; // Reg 0x56/0x57, FW_VER/FW_REV, 16-bit, firmware version/revision number, RO
    quint16 mcuVer;
    quint16 mcuRev;
    bool connected;
    bool runFinished;
    QMap<int, double> temp;
    quint16 status; // 0x2 TM Time Valid
    quint64 wrTime;
    quint16 adcStatus;
    bool adcSelfTestOk;
    quint16 pllStatus;
    bool mstreamMultiAck;
    quint8 mstreamHwBufSize;
    QMap<int, double> hitRate;
    HptdcStatus hptdcStatus[2];
    quint16 busyStatus;

    TqdcStatistic statistic;
};
Q_DECLARE_METATYPE(TqdcStatus)



class TQDCDevice : public QtMregDevice
{
    Q_OBJECT
public:
    typedef enum {TRIG_NEGATIVE = 0, TRIG_POSITIVE = 1} TrigPolarity;
    explicit TQDCDevice(quint16 devId, quint64 devSerial);
    void readStatistics();
    void clearStatistics();
    virtual bool checkLiveMagic();
    static quint16 dacValue(int mV);
    static quint16 dacValue(int mV, int dacShift);
    static double dac_mV(quint16 dacData);

signals:
    void statusUpdated(const TqdcStatus &);
    void deviceInfoUpdated(DeviceIndex index, int chNum, int maxSampleCnt);
    void calibrateFinished(const DeviceIndex &index,
                           const QMap<int, double> &adcVals, const QMap<int, double> &rms,
                           const QMap<int, double> &adcVals4, const QMap<int, double> &rms4,
                           const QMap<int, int> &dacOffset);

public slots:
    void writeSetup();
    void setSetup(const DeviceSetup &s, const DeviceCalibration &cal);
    void writeSetupForce();
    void softReset();
    void blockDeviceRun(bool _isBlock);
    void startCalibration();

protected slots:
    virtual void statusPoll();
    virtual void on_deviceConnected();
    virtual void on_deviceDisconnected();

private:
    mlink::RegOpVector WriteTrigSetup(reg_trig_ctrl trigsetup);
    mlink::RegOpVector setChThreshold(QMap<int, int> chThr, QMap<int, int> thrOffset, QMap<int, bool> chInv);
    mlink::RegOpVector setChThreshold(QMap<int, int> chThr);
    mlink::RegOpVector setAdcLatency(int latency);
    void StartRun();
    void StopRun();
    bool isDesStatusOk(const quint16 &adc_status);
    int getDeserializeNum() const { return 4; }
    int getNumberOfChannels() const;
    void detectChannelMaxSampleCount();
    void setupAdc();
    void resetDac();
    void writeTime();
    void writeClockDataDelay();
    void ResetAdc();
//    QString adcStatusStr(int status);
    void readoutHptdcSetup();
    bool waitEndOfRun();
    void resetEvNumAndStats();
    void calibrateBaseline(QMap<int, double> &mean, QMap<int, double> &rms);
    QMap<int, int> calibrateDac();

protected:
    void sendLog(const QString &);
    DeviceSetup setup;
    DeviceSetup setupCache;
    DeviceCalibration calibration;
    DeviceCalibration calibrationCache;
    bool setupCacheValid;
    TqdcStatus status;
    quint32 maxSampleCount;
    QMap<int, quint32> hitCnt;
    quint64 hitCntTime;
    bool isBlock;
};

#endif // TQDCDEVICE_H
