#include "TqdcFileReader.h"
#include <QDebug>
#include <QDir>
#include <QTime>
#include <QTimer>
#include <QSemaphore>

namespace {
const qint64 PACK_SIZE_LIMIT = 20e6;
}

TqdcFileReader::TqdcFileReader(QSemaphore *sem):
    freeRawDataSem(sem),
    readFileTimer(new QTimer(this))
{
    connect(readFileTimer, &QTimer::timeout, this, &TqdcFileReader::readFile);
    readFileTimer->setSingleShot(false);
    readFileTimer->setInterval(1000);
    readFileTimer->start();
}


void TqdcFileReader::readFile()
{
    if(!dataFile.exists() || !dataFile.isReadable())
        return;
    while(!dataFile.atEnd()){
        int sizeToRead = qMin(dataFile.size()-dataFile.pos(), PACK_SIZE_LIMIT);
        if(sizeToRead==0){
            break;
        }

        if(!freeRawDataSem->tryAcquire()){
            qInfo() << QString("[%1] file read delayed (analyzer is slow)")
                     .arg(QTime::currentTime().toString("hh:mm:ss:zzz"));
            break;
        }

        emit dataAccuired(dataFile.read(sizeToRead));
    }
}


void TqdcFileReader::setFileName(QString fileName)
{
    if(dataFile.isOpen())
        dataFile.close();
    dataFile.setFileName(fileName);
    dataFile.open(QIODevice::ReadOnly);
}
