#include "SnmpCtrl.h"

#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <cstring>
#include <iostream>
#include <net-snmp/session_api.h>

#include <QDebug>

#include "ttvxs_snmp_types.h"

namespace {

bool get_int_from_responce(struct snmp_pdu *response, oid *anOID, size_t anOID_len, long *val_int) {
    struct variable_list *vars;
    for(vars = response->variables; vars; vars = vars->next_variable) {
        if(vars->type == ASN_INTEGER && memcmp(vars->name_loc, anOID,anOID_len*sizeof(u_long))==0) {
            *val_int = *vars->val.integer;
            return true;
        }
    }

    return false;
}
bool get_str_from_responce(struct snmp_pdu *response, oid *anOID, size_t anOID_len, QString *str) {
    struct variable_list *vars;
    for(vars = response->variables; vars; vars = vars->next_variable) {
        if(vars->type == ASN_OCTET_STR && memcmp(vars->name_loc, anOID,anOID_len*sizeof(u_long))==0) {
            *str = QString::fromUtf8((char *)vars->val.string, vars->val_len);
            return true;
        }
    }

    return false;
}

void print_responce(struct snmp_pdu *response, QVector<QPair<QString, QVariant> > *response_vars = nullptr) {
    struct variable_list *vars;
    for(vars = response->variables; vars; vars = vars->next_variable) {
//        qInfo() << *vars->name_loc;
        char buf[200];

        int bul_len = 200;
        bul_len = snprint_objid(buf, bul_len,vars->name, vars->name_length);


        QString prefix = QString::fromUtf8((char*)buf, bul_len);
        QVariant var;

//        std::cout<<"std::flush";
//        for(int i=0; i< vars->name_length; ++i) {
//            qInfo() << vars->name[i];
//        }
//        qInfo() << QString::fromUtf8((char *)vars->name, vars->name_length);
        switch (vars->type) {
//        case ASN_BOOLEAN:
//            qInfo() << *vars->val.string;
//            break;
        case ASN_INTEGER:
            var = QVariant::fromValue(*vars->val.integer);
            qInfo() << prefix <<"I:"<< var.toInt();
            break;
        case ASN_BIT_STR:
            qInfo() << prefix <<"B:"<< *vars->val.bitstring;
            break;
        case ASN_OCTET_STR:
            var = QString::fromUtf8((char *)vars->val.string, vars->val_len);
            qInfo() << prefix <<"O:"<< var.toString();
            break;
//        case ASN_NULL:
//            qInfo() << *vars->val.string;
//            break;
//        case ASN_OBJECT_ID:
//            qInfo() << *vars->val.string;
//            break;
//        case ASN_SEQUENCE:
//            qInfo() << *vars->val.string;
//            break;
//        case ASN_SET:
//            qInfo() << *vars->val.string;
//            break;
        case ASN_LONG_LEN:
            qInfo() << prefix << "LONG_LEN:" << vars->val_len;
            break;
        default:
            Q_ASSERT(false);
        }
        if(response_vars)
            response_vars->push_back(QPair<QString, QVariant>(prefix,var));
    }
}
}

SnmpCtrl::SnmpCtrl():
    session(new snmp_session())
{
    init_mib();
}

int SnmpCtrl::add_mib_dir(QString dir)
{
    // ~/.snmp/mibs - adds automaticatly
    int mib_files = add_mibdir(dir.toStdString().c_str());
    return mib_files;
}

void SnmpCtrl::setHost(QString s)
{
    if(isConnected())
        disconnect();

    host = s;
    host_str = s.toStdString();
    sprintf(host_buff, "%s", host_str.c_str());
}

void SnmpCtrl::setCommunity(QString s)
{
    if(isConnected())
        disconnect();

    community = s;
    community_str = s.toStdString();
    sprintf(community_buff, "%s", community_str.c_str());
}

bool SnmpCtrl::get_str(const char *oid_str, QString *str)
{
    if(!isConnected())
        connect();

    oid anOID[MAX_OID_LEN];
    size_t anOID_len = MAX_OID_LEN;
    int status_read_objid = read_objid(oid_str, anOID, &anOID_len);
    if(status_read_objid == 0)
        return false;

    struct snmp_pdu *pdu, *response;
    pdu = snmp_pdu_create(SNMP_MSG_GET);
    snmp_add_null_var(pdu, anOID, anOID_len);

    int status_synch = snmp_synch_response(ss, pdu, &response);

    bool ok = status_synch == STAT_SUCCESS && response->errstat == SNMP_ERR_NOERROR;
    if (ok) {
        u_char *buf = nullptr;
        size_t len;
        ok = get_str_from_responce(response, anOID, anOID_len, str);
    } else {
        printf("s_errno: %s\ns_snmp_errno: %s", strerror(ss->s_errno), snmp_api_errstring(ss->s_snmp_errno));
    }
    snmp_free_pdu(response);

    return ok;
}

bool SnmpCtrl::get_int(const char *oid_str, long *val)
{
    if(!isConnected())
        connect();

    oid anOID[MAX_OID_LEN];
    size_t anOID_len = MAX_OID_LEN;
    int status_read_objid = read_objid(oid_str, anOID, &anOID_len);
    if(status_read_objid == 0)
        return false;

    struct snmp_pdu *pdu, *response;
    pdu = snmp_pdu_create(SNMP_MSG_GET);
    snmp_add_null_var(pdu, anOID, anOID_len);

    int status_synch = snmp_synch_response(ss, pdu, &response);

    bool ok = status_synch == STAT_SUCCESS && response->errstat == SNMP_ERR_NOERROR;
    if (ok) {
        ok = get_int_from_responce(response, anOID, anOID_len, val);
    } else {
        printf("s_errno: %s\ns_snmp_errno: %s", strerror(ss->s_errno), snmp_api_errstring(ss->s_snmp_errno));
    }
    snmp_free_pdu(response);

    return ok;
}

bool SnmpCtrl::debug_print(const char *oid_str)
{
    qInfo() << "debug_print:" << oid_str;
    if(!isConnected())
        connect();

    oid anOID[MAX_OID_LEN];
    size_t anOID_len = MAX_OID_LEN;
    int status_read_objid = read_objid(oid_str, anOID, &anOID_len);
    if(status_read_objid == 0)
        return false;

    struct snmp_pdu *pdu, *response;
    pdu = snmp_pdu_create(SNMP_MSG_GETNEXT);
    snmp_add_null_var(pdu, anOID, anOID_len);

    int status_synch = snmp_synch_response(ss, pdu, &response);

    bool ok = status_synch == STAT_SUCCESS && response->errstat == SNMP_ERR_NOERROR;
    if (ok) {
        print_responce(response);
    } else {
        printf("s_errno: %s\ns_snmp_errno: %s", strerror(ss->s_errno), snmp_api_errstring(ss->s_snmp_errno));
    }
    snmp_free_pdu(response);

    return ok;
}

bool SnmpCtrl::print_test2(QStringList id_list, QVector<QPair<QString, QVariant> > *responce_vars)
{
    int id = which_module("AFI-BMC-MIB");
    id = which_module("WIENER-CRATE-MIB");
    tree *t = find_node2("systemStatus", "AFI-BMC-MIB");
    {
        oid anOID[MAX_OID_LEN];
        size_t anOID_len = MAX_OID_LEN;

        int status_read_objid = read_objid(sensorsVoltageNom_8, anOID, &anOID_len);
        if(status_read_objid == 0)
            return false;

        tree *tree_head = get_tree_head();
        tree *tree_i = get_tree(anOID, anOID_len, tree_head);
        qInfo() << tree_i->subid;
    }
//    find_tree_node
    qInfo() << id;
    qInfo() << "print_test2(QStringList id_list)";
    if(!isConnected())
        connect();

    struct snmp_pdu *pdu, *response;
    pdu = snmp_pdu_create(SNMP_MSG_GETNEXT);
    for(const QString &id : id_list) {
        oid anOID[MAX_OID_LEN];
        size_t anOID_len = MAX_OID_LEN;
        int status_read_objid = read_objid(id.toStdString().c_str(), anOID, &anOID_len);
        if(status_read_objid == 0)
            return false;

        snmp_add_null_var(pdu, anOID, anOID_len);
    }

    int status_synch = snmp_synch_response(ss, pdu, &response);

    bool ok = status_synch == STAT_SUCCESS && response->errstat == SNMP_ERR_NOERROR;
    if (ok) {
        print_responce(response, responce_vars);
    } else {
        printf("s_errno: %s\ns_snmp_errno: %s", strerror(ss->s_errno), snmp_api_errstring(ss->s_snmp_errno));
    }
    snmp_free_pdu(response);

    return ok;
}

bool SnmpCtrl::print_test_dec()
{
    tree *t = find_node2("sensorsVoltageNom", "AFI-BMC-MIB");
    {
        oid anOID[MAX_OID_LEN];
        size_t anOID_len = MAX_OID_LEN;

        int status_read_objid = read_objid(sensorsVoltageNom_8, anOID, &anOID_len);
        if(status_read_objid == 0)
            return false;

        tree *tree_head = get_tree_head();
        tree *tree_i = get_tree(anOID, anOID_len, tree_head);
        qInfo() << tree_i->subid;
    }
    return true;
}


bool SnmpCtrl::set_vars(QMap<QString, QVariant> vars)
{
    if(!isConnected())
        connect();


    struct snmp_pdu *pdu, *response;
    pdu = snmp_pdu_create(SNMP_MSG_SET);

    for(auto key : vars.keys()) {
        oid anOID[MAX_OID_LEN];
        size_t anOID_len = MAX_OID_LEN;

        int status_read_objid = read_objid(key.toStdString().c_str(), anOID, &anOID_len);
        if(status_read_objid == 0)
            return false;

        long val = vars[key].toInt();
        snmp_varlist_add_variable(&pdu->variables, anOID, anOID_len,
                                  ASN_INTEGER, &val, sizeof(val));
    }

    int status_synch = snmp_synch_response(ss, pdu, &response);

    bool ok = status_synch == STAT_SUCCESS && response->errstat == SNMP_ERR_NOERROR;
    snmp_free_pdu(response);

    return ok;
}

bool SnmpCtrl::connect()
{
    init_snmp(("crate_"+host_str).c_str());
    snmp_sess_init(session);

    session->peername = host_buff;

    session->version = SNMP_VERSION_2c;

    session->community = reinterpret_cast<u_char*>(community_buff);
    session->community_len = std::strlen(community_buff);

    ss = snmp_open(session);
    if (!ss) {
        std::cerr << "Couldn't connect to " << host_str << std::endl;
        printf("s_errno: %s\ns_snmp_errno: %s\n", strerror(session->s_errno), snmp_api_errstring(session->s_snmp_errno));
    }
    return isConnected();
}

void SnmpCtrl::disconnect()
{
    if(snmp_close(ss) != 1) {
        printf("Error on SNMP close.\n");
    }
    ss = nullptr;
}

void SnmpCtrl::check_mib()
{
    char name[200];
    sprintf(name, "AFI-BMC-MIB::sensorsVoltageNom.8");

//        init_mib();
//        int val = add_mibdir("/home/daq/.snmp/mibs");
    oid descr_oid[MAX_OID_LEN];
    size_t descr_oid_len = MAX_OID_LEN;

    int val2 = read_objid(name, descr_oid, &descr_oid_len);
    printf("check_mib: %d\n", val2);
    Q_ASSERT(val2 == 1 && descr_oid_len==15);
    printf("check_mib: END\n");
}
