#ifndef TTVXSCONFIG_H
#define TTVXSCONFIG_H

#include <QObject>
#include <QDialog>

#include "StatPreset.h"

#include "mldiscover/DeviceIndex.h"

typedef union {
    quint16 all=0;
    struct {
        unsigned int ttl: 1;
        unsigned int sfp: 1;
        unsigned int unused2: 12;
        unsigned int random_pulser: 1;
        unsigned int fixed_pulser: 1;
    } bit;
} reg_trig_src;

typedef union {
    quint16 all=0;
    struct {
        unsigned int clear: 1;
        unsigned int reserved: 11;
        unsigned int en_aux_cnt: 1;
        unsigned int reset_aux_cnt: 1;
        unsigned int sendTimeReset: 1;
        unsigned int run: 1;
    } bit;
} reg_rc_ctrl;

typedef union {
    quint16 all=0;
    struct {
        unsigned int softClear: 1;
        unsigned int cntLock: 1;
        unsigned int reserved: 13;
        unsigned int run_csr: 1;
    } bit;
} reg_base_ctrl;

class TtvxsConfig
{
public:
    TtvxsConfig(DeviceIndex i=DeviceIndex());
    void unify_config(const TtvxsConfig &cfg) {
        run = cfg.run;
        trig_open = cfg.trig_open;
        trigSrc = cfg.trigSrc;
        trigTimerFreq = cfg.trigTimerFreq;
        trigRandomFreq = cfg.trigRandomFreq;
        trigRandomDeadTimeUs = cfg.trigRandomDeadTimeUs;

        aux_cnt_for_trig = cfg.aux_cnt_for_trig;

        stat_readout = cfg.stat_readout;
        stat_period = cfg.stat_period;
        stat_preset = cfg.stat_preset;
    }

    DeviceIndex device_index;
    bool enabled;
    bool run;
    bool trig_open;
    reg_trig_src trigSrc;
    quint32 vxsChMask;
    int trigTimerFreq;  // Hz
    int trigRandomFreq; // Hz
    double trigRandomDeadTimeUs; // us

    bool aux_cnt_for_trig;

    bool stat_readout;
    quint16 stat_period;
    StatPreset stat_preset;

    int slot;
    QString custom_name;
};
Q_DECLARE_METATYPE(TtvxsConfig)


#endif // TTVXSCONFIG_H
