#ifndef TTVXSREGS_H
#define TTVXSREGS_H

#include "mregdev/mregdevice.h"

enum {
    // Control registers
    REG_DEVICE_CTRL = mlink::MregDeviceAdcm::ADCM16_CTRL_REG,
};
enum {
    // Run Control Core Registers
    REG_RC_BASE = 0x200,
    REG_RC_CTRL                     = REG_RC_BASE + 0x00,
    REG_RC_TRIG_SRC                 = REG_RC_BASE + 0x02,
    REG_RC_TRIG_STATE               = REG_RC_BASE + 0x03,
    REG32_RC_BUSY_MASK              = REG_RC_BASE + 0x04,
    REG32_BUSY_INPUT_STATE          = REG_RC_BASE + 0x06, // RO
    REG32_BUSY_TIMEOUT_STATE        = REG_RC_BASE + 0x08, // RO
    REG32_FIXED_FREQ_PULSER_PERIOD  = REG_RC_BASE + 0x0A, // R/W
    REG32_RANDOM_PULSER_PROBABILITY = REG_RC_BASE + 0x0C, // R/W
    REG32_RANDOM_PULSER_DEAD_TIME   = REG_RC_BASE + 0x0E, // R/W
    REG_TRIG_QUEUE                  = REG_RC_BASE + 0x10, // RO
    REG64_EVENT_NUMBER_LOAD         = REG_RC_BASE + 0x14, // R/W
    REG64_CUR_EVENT_NUMBER          = REG_RC_BASE + 0x18, // RO
    REG128_BUILD_PARAMS             = REG_RC_BASE + 0x20, //RO
};

enum {
    // Statistics Readout CSR Registers
    REG_STAT_BASE = 0x900,
    REG_STAT_CTRL       = REG_STAT_BASE + 0x00,
    REG_STAT_PERIOD     = REG_STAT_BASE + 0x02,
    REG_STAT_BLOCK_SIZE = REG_STAT_BASE + 0x04,
    REG_STAT_RAM_ADDR   = REG_STAT_BASE + 0x05,
    REG_STAT_RAM_DATA   = REG_STAT_BASE + 0x06
};

enum {
    // Error Counters
    REG_ERROR_BASE = 0x1000,
    REG64_HWIP_Link_Down = REG_ERROR_BASE + 0x00,
    REG64_REG_IO_ERROR   = REG_ERROR_BASE + 0x04,
    REG64_SFP_ERROR_ETH  = REG_ERROR_BASE + 0x08,
    REG64_HWIP_ERROR     = REG_ERROR_BASE + 0x0C,
    REG64_TX_ERROR_ETH   = REG_ERROR_BASE + 0x10,
    REG64_RX_ERROR_ETH   = REG_ERROR_BASE + 0x14
};

enum {
    // Error Counters
    REG_AUX_CNT_BASE = 0x1200,
//    REG64_HWIP_Link_Down = REG_ERROR_BASE + 0x00,
//    REG64_REG_IO_ERROR   = REG_ERROR_BASE + 0x04,
//    REG64_SFP_ERROR_ETH  = REG_ERROR_BASE + 0x08,
//    REG64_HWIP_ERROR     = REG_ERROR_BASE + 0x0C,
//    REG64_TX_ERROR_ETH   = REG_ERROR_BASE + 0x10,
//    REG64_RX_ERROR_ETH   = REG_ERROR_BASE + 0x14
};

// MCU SPI Slave
enum {
    REG_BMC_BASE               = 0x4000,
    REG_BMC_DEV_ID             = REG_BMC_BASE + 0x00,
    REG_BMC_ONBOARD_PLL_STATUS = REG_BMC_BASE + 0x01,
//    REG_BMC_PLL_UNLOCK_CNT     = REG_BMC_BASE + 0x02, // [12:0] - unlock counter
    REG_BMC_TEMP_1             = REG_BMC_BASE + 0x03, // pll temp
//    REG_BMC_TEMP_2             = REG_BMC_BASE + 0x04,
//    REG_BMC_TEMP_3             = REG_BMC_BASE + 0x05,
//    REG_BMC_TEMP_4             = REG_BMC_BASE + 0x06,
    REG_BMC_VERSION            = REG_BMC_BASE + 0x08,
    REG_BMC_REVERSION          = REG_BMC_BASE + 0x09,
    REG_BMC_SYSTEM_STATUS      = REG_BMC_BASE + 0x0A, // [1:0] - 0: unknown, 1: normal, 2: warning, 3: critical
    REG_BMC_POWER_STATUS       = REG_BMC_BASE + 0x0B,
    REG_BMC_PLL_STATUS         = REG_BMC_BASE + 0x0C,

    REG_BMC_VOL_BP_5           = REG_BMC_BASE + 0x10, // Backplane +5V supply voltage, mV
    REG_BMC_CUR_BP_5           = REG_BMC_BASE + 0x11, // Backplane +5V supply current, mA
    REG_BMC_VOL_BP_33          = REG_BMC_BASE + 0x12, // Backplane +3.3V supply voltage, mV
    REG_BMC_CUR_BP_33          = REG_BMC_BASE + 0x13, // Backplane +3.3V supply current, mA
};

#endif // TTVXSREGS_H

