/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef TTVXSSTATUS_H
#define TTVXSSTATUS_H

#include <QMetaType>

#include "mldiscover/DeviceIndex.h"
#include "TtvxsAuxCounters.h"
#include "base/bmc/BmcStatus.h"
#include "mregdev/WrEpStatus.h"

class HwipErrorCounters
{
public:
    HwipErrorCounters() {
        clear();
    }
    void clear() {
        hwipLinkDown=0;
        regIoError=0;
        sfpErrorEth=0;
        hwipError=0;
        txErrorEth=0;
        rxErrorEth=0;
    }

    quint64 hwipLinkDown;
    quint64 regIoError;
    quint64 sfpErrorEth;
    quint64 hwipError;
    quint64 txErrorEth;
    quint64 rxErrorEth;
};

class TtvxsStatus
{
public:
    TtvxsStatus() :
        isConnected(false),
        liveMagikOk(false),
        trigState(0),
        timeoutState(0),
        curEvNum(0),
        inputState(0),
        mstreamMultiAck(false),
        mstreamHwBufSize(1)
    {
        memset(buildParams, 0, 4*sizeof (quint32));
        hwipErrors.clear();
        temp = 0;
    }

    DeviceIndex devIndex;
    bool isConnected;
    bool liveMagikOk;
    quint16 trigState;
    quint32 buildParams[4];
    quint32 timeoutState;
    quint64 curEvNum;
    quint16 trigQueue;
    quint32 inputState;
    HwipErrorCounters hwipErrors;
    QString fwStr;
    bool mstreamMultiAck;
    quint8 mstreamHwBufSize;
    double temp;

    TtvxsAuxCounters aux_cnt;
    BmcStatus bmc;
    WrEpStatus wrEpStatus;
};
Q_DECLARE_METATYPE(TtvxsStatus)

#endif // TTVXSSTATUS_H
