#include <QApplication>

#include "MainWindow.h"

#include "daq-config/DaqConfig.h"
#include "base/SyslogMessageSender.h"
#include "metric-sender/MetricSender.h"
#include "mongo/BaseConfig.h"
#include "mongo/StartupOptions.h"
#include "base/RunGuard.h"

int main(int argc, char *argv[])
{
    const QString PROGRAM_TYPE = DaqConfig::getTypeName(DaqConfigTtvxs);
    QApplication a(argc, argv);
    a.setApplicationName(PROGRAM_TYPE);
    a.setOrganizationName("AFI Electronics");

    SyslogMessageSender::instance().install();
    #ifdef MONGO_DRIVERS
        mongocxx::instance insta{};
    #endif
    BaseConfig base_config;
    base_config.init(PROGRAM_TYPE);
    StartupOptions startup_options(PROGRAM_TYPE);
    startup_options.parse_options();

    RunGuard runGuard(QString("%1_%2").arg(a.applicationName(), startup_options.program_index));
    if(!runGuard.tryToRun()) {
        QString errStr = a.applicationName().append(" is already running; ");
        if(startup_options.program_index.isEmpty())
            errStr += "in local run mode.";
        else
            errStr += QString("with run_index '%1'").arg(startup_options.program_index);
        qFatal("%s", errStr.toStdString().c_str());
    }

    int res =1;
    MetricSender::instance().install();
    if(startup_options.list == false){
        MainWindow w(startup_options);
        QObject::connect(&SyslogMessageSender::instance(), SIGNAL(newMsg(QtMsgType, QString, QString)),
                         &w, SLOT(addLog(QtMsgType, QString, QString)));
        w.show();
        res = a.exec();
    }

    return res;
}
