/*
**    Copyright 2020 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef RCRUNID_H
#define RCRUNID_H

#include <QString>
#include <QMetaType>

struct RcRunId
{
    RcRunId() = default;
    RcRunId(const QString &index, int number) :
        runIndex(index),
        runNumber(number)
    { }

    void set(const RcRunId &r) { *this = r; }
    void reset() { *this = RcRunId(); }
    void resetRunNumber() { runNumber = 0; }

    bool operator==(const RcRunId &r) const {
        return
                runIndex == r.runIndex &&
                runNumber == r.runNumber;
    }
    bool operator!=(const RcRunId &r) const {
        return ! (*this==r);
    }
    explicit operator QString() const {
        return QString("%1 %2").arg(runIndex).arg(runNumber);
    }

    QString runIndex;
    int runNumber = 0;
};

Q_DECLARE_METATYPE(RcRunId)

#endif // RCRUNID_H
