#ifndef DEVICEDESCRIPTION_H
#define DEVICEDESCRIPTION_H

#include <QDateTime>
#include <QString>

#include <QElapsedTimer>

#include "DeviceIndex.h"

class DeviceDescription
{
public:
    DeviceDescription(DeviceIndex devIndex);
    DeviceDescription(quint16 devId=0, quint64 devSerial=0, quint32 ip=0);

    void clear();
    DeviceIndex getIndex() const { return DeviceIndex(device_id, serial_id); }
    bool isNull() const { return getIndex().isNull(); }
    bool isOutOfDate() const;
    QString getIdent() const;
    QString getSerialIdStr() const;
    QString getModelStr() const;
    QString getModelVerStr() const; // "BOARD-1.2"
    QString getFirmWareStr() const;
    int getHwMajVer() const;
    static int getHwMajVer(const QString &str);
    QPair<int,int> getHwVer() const;
    static QPair<int,int> getHwVer(const QString &str);
    bool deviceInfoAreSame(const DeviceDescription &dd) const;
    bool operator ==(const DeviceDescription &dd) const;

    quint64 haddr = 0; // mac-address: 1811275143411
    quint32 ip_addr = 0; // 173897917 = 0x0A5D78BD = 10.93.120.189
    quint32 ttl = 0; // Time-to-live: 8
    // LLDP-MED
    QString portID; // id for the port from which this LLDPDU was transmitted: "2"
//    QString hw;
    QString hw_str; // 5 Hardware Revision, formatted: "1.0"
    QString fw_str; // 6 Firmware Revision, formatted: "2.0.19676"
    QString sn_str; // 8 Serial Number: "0000-0569-59F2"
    QString mn_str; // 9 Manufacturer Name: "JINR AFI"
    QString modelname_str; // 10 Model Name: "        U40VE-RC"
    // Local - 1
    quint16 device_id = 0; // (hex) cc
    quint64 serial_id = 0; // (hex) 56959f2
    quint16 chassis_slot = 0;
    // Local - 2
    quint64 master_mac = 0;
    quint32 master_ip = 0;
    quint16 master_udp_port = 0;
    quint64 mstream_mac = 0;
    quint32 mstream_ip = 0;
    quint16 mstream_udp_port = 0;
    bool mstream_locked = false;
    bool master_locked = false;

    bool enabled = true;
    QDateTime discoverTime;
    QElapsedTimer discoverTimer;
};

Q_DECLARE_METATYPE(DeviceDescription)

#endif // DEVICEDESCRIPTION_H
