#ifndef DEVICEINDEX_H
#define DEVICEINDEX_H
#include <QDebug>
#include <QPair>
#include <QMetaType>

class DeviceIndex : public QPair<quint16, quint64>
{
public:
    DeviceIndex() : DeviceIndex(0,0)
    {}

    DeviceIndex(const DeviceIndex &index):
        QPair<quint16, quint64>(index)
    {}

    DeviceIndex(quint16 _id, quint64 _serial)
    {
        first = _id;
        second = _serial;
    }
    DeviceIndex &operator=(const DeviceIndex&) = default;
    bool operator <(const DeviceIndex &index){
        if(first < index.first)
            return true;
        if(first > index.first)
            return false;
        if(second < index.second)
            return true;
        if(second > index.second)
            return false;
        return false;
    }

    void clear() { first=0; second=0; }
    bool isNull() const { return first==0 || second==0; }

    inline quint16 getDevId() const { return first; }
    inline quint64 getSerial() const { return second; }
    QString getIdent() const {
        return QString("%1 %2")
                .arg(QString("%1").arg(getDevId(), 2, 16, QChar('0')).toUpper())
                .arg(getSerialStr());
    }
    QString getSerialStr() const {
        QString serial = QString("%1-%2").arg((second>>16) & 0xFFFFu, 4, 16, QChar('0'))
                .arg(second&0xFFFFu, 4, 16, QChar('0'));
        if(second>>32) {
            serial.prepend(QString("%1-").arg((second>>32) & 0xFFFFu, 4, 16, QChar('0')));
        }
        if(second>>48) {
            serial.prepend(QString("%1-").arg((second>>48) & 0xFFFFu, 4, 16, QChar('0')));
        }
        return serial.toUpper();
    }
};

//QDebug operator<<(QDebug dbg, const DeviceIndex &index)
//{
//    dbg.nospace() << hex << "(0x" << index.getDevId() << ", " << index.getSerial() << ")" << dec;
//    return dbg.space();
//}


Q_DECLARE_METATYPE(DeviceIndex)

#endif // DEVICEINDEX_H
