#include "GeneratorConfig.h"

#include <QDebug>

GeneratorConfig::GeneratorConfig(QString program_type_,
                                 QString program_index_,
                                 QString configuration_name_,
                                 bool run_)
{
    program_type = program_type_;
    program_index = program_index_;
    configuration_name = configuration_name_;
    run = run_;
    update_root();
    default_root = root;
    default_root.set_data("configuration_name", QString("default"));
}

GeneratorConfig::~GeneratorConfig()
{
}

bool GeneratorConfig::read_config()
{
    root = BaseConfig::read_config(program_type, program_index, configuration_name);
    if(root.isEmpty()){
        return 0;
    }
    program_index = root.get_data<QString>("program_index");
    configuration_name = root.get_data<QString>("configuration_name");
    spill_count = root.get_data<int>("spill_count");
    spill_size = root.get_data<int>("spill_size");
    interspill_size = root.get_data<int>("interspill_size");
    spill_events = root.get_data<int>("spill_events");
    event_length = root.get_data<int>("event_length");
    return 1;
}

bool GeneratorConfig::write_config()
{
    update_root();
    return BaseConfig::write_config(root);
}

void GeneratorConfig::update_root()
{
    root.remove_all_data();
    root.set_data("program_type", program_type);
    root.set_data("program_index", program_index);
    root.set_data("configuration_name", configuration_name);
    root.set_data("run_number", run_number);
    root.set_data("run_index", run_index);
    root.set_data("spill_count", spill_count);
    root.set_data("spill_size", spill_size);
    root.set_data("interspill_size", interspill_size);
    root.set_data("spill_events", spill_events);
    root.set_data("length_min", event_length);
}

void GeneratorConfig::debug_print()
{
    qInfo() << "=== program_type: " << program_type;
    qInfo() << "=== program_index: " << program_index;
    qInfo() << "=== configuration_name: " << configuration_name;
    qInfo() << "=== spill_count: " << spill_count;
    qInfo() << "=== spill_size: " << spill_size;
    qInfo() << "=== interspill_size: " << interspill_size;
    qInfo() << "=== spill_events: " << spill_events;
    qInfo() << "=== length: " << event_length;
}


