#ifndef FILEREADERDIALOG_H
#define FILEREADERDIALOG_H

#include <QDialog>

class FileReader;
class QSemaphore;

namespace Ui {
class FileReaderDialog;
}

struct FileReaderConfig
{
    FileReaderConfig() :
        playbackInterval(500),
        readBy(1)
    {}
    QString lastFile;
    int playbackInterval;
    int readBy;
};


class FileReaderDialog : public QDialog
{
    Q_OBJECT

public:
    explicit FileReaderDialog(FileReaderConfig *conf, QSemaphore *freeRawDataSem, QWidget *parent = nullptr);
    ~FileReaderDialog();
    void configUpdated();
    QString getCurrentFile() { return currentFile; }
    void setPackSizeLimit(qint64 limit);

signals:
    void fileNameUpdated(QString fileName);
    void changeRecord(int recordNum, int count=1);
    void dataAccuired(QByteArray);
    void closeAll();
    void closeFile();
    void writeSettings();
    void packSizeLimitUpdated(qint64 limit);

private slots:
    void on_pushButtonSetFile_clicked();
    void on_spinBoxCurEvent_valueChanged(int evNum);
    void on_pushButtonBackward_toggled(bool checked);
    void on_pushButtonStop_toggled(bool checked);
    void on_pushButtonForward_toggled(bool checked);
    void on_spinBoxInterval_valueChanged(int intervalMs);
    void on_comboBoxReadBy_currentIndexChanged(int);

    void playback();
    void fileMapUpdated(qint64 fileSize, QString lastModify, int recordsSize, double decodedPart);
    void eventRead(quint32 ev_first, quint32 ev_last, qint64 evSize);

public slots:
    void onCloseFile();

private:
    void saveSettings();
    void updateProgressBar(int recordNum) const;

    Ui::FileReaderDialog *ui;
    FileReaderConfig * const config;
    QSemaphore * const freeRawDataSem;
    FileReader * const reader;
    QThread * const fileReaderThread;
    QString currentFile;
    QTimer * const playbackTimer;
    bool playbackForward;
    bool block_write_settings;
};

#endif // FILEREADERDIALOG_H
