#include "HistMngrCfg.h"

#include <QSettings>

#include "DeviceHelper.h"

int HistId::tdcCh=-1;
int HistId::tdcTaiTime=-1;
int HistId::tdcTime=-1;
int HistId::tdcWidth=-1;
int HistId::tdcDoublePulseTime=-1;
int HistId::tdcHitEv=-1;
int HistId::inl=-1;
int HistId::inlCheck=-1;
int HistId::errors=-1;
const int HistId::nch = 72;

void HistMngrCfg::init(bool isInlEnabled)
{
    int maxGroupId = 0;

    HistId::tdcTaiTime = maxGroupId;
    histCfg.push_back(HistCfg(true, maxGroupId++, "TDC TAI Time, ns", 0, 0, 0));

    HistId::tdcTime = maxGroupId;
    histCfg.push_back(HistCfg(true, maxGroupId++, "TDC Hit Time, ns", 51200, 0, 51200));

    HistId::tdcWidth = maxGroupId;
    histCfg.push_back(HistCfg(true, maxGroupId++, "TDC Hit Width, ns", 51200, 0, 51200));

    HistId::tdcDoublePulseTime = maxGroupId;
    histCfg.push_back(HistCfg(true, maxGroupId++, "TDC Double-Pulse Time, ns", 2*51200*1024/24, -51200, 51200));

    // remove from base hists
    HistId::tdcCh = maxGroupId;
    histCfg.push_back(HistCfg(true, maxGroupId++, "TDC Hits by channel", HistId::nch, 0, HistId::nch));

    HistId::tdcHitEv = maxGroupId;
    histCfg.push_back(HistCfg(true, maxGroupId++, "TDC Hits per event", 100, 0, 100));

    HistId::errors= maxGroupId;
    histCfg.push_back(HistCfg(true, maxGroupId++, "TDC Errors bits", 15, 1, 100));

    if(isInlEnabled) {
        HistId::inl = maxGroupId;
        histCfg.push_back(HistCfg(true, maxGroupId++, "INL", 1024, 0, 1024, false, false));

        const qint32 range = 1024;
        HistId::inlCheck = maxGroupId;
        histCfg.push_back(HistCfg(true, maxGroupId++, "INL Check", 2*range*40, range*(-1), range, true, false));
    }
}

void HistMngrCfg::load()
{
    QSettings s;
    s.beginGroup("Histogram");
    curHistTitle = s.value("curHistType").toString();
    useStopCh = s.value("useStopCh").toBool();
    stopCh = static_cast<quint16>(s.value("stopCh", stopCh).toUInt());
    quint16 stopDevId = static_cast<quint16>(s.value("stopDevId", 0).toUInt());
    quint64 stopDevSerial = s.value("stopDevSerial", 0).toULongLong();
    if(stopDevId && stopDevSerial)
        stopDevIndex = DeviceIndex(stopDevId, stopDevSerial);
    inlHistCodeDens = s.value("inlHistCodeDens").toBool();
    globalEnabled = s.value("enabled").toBool();
    taiTsWindow = s.value("taiTsWindow", taiTsWindow).toInt();
    mwMin = s.value("mwMin", mwMin).toInt();
    mwMax = s.value("mwMax", mwMax).toInt();
    log_y_axis = s.value("log_y_axis", log_y_axis).toBool();
    if(HistId::tdcTime != -1) {
        matchWinUpdated(mwMin, mwMax);
    }

    QStringList subGroups = s.childGroups();
    for(int i=0; i<histCfg.size(); ++i) {
        HistCfg &hCfg = histCfg[i];
        if(!subGroups.contains(hCfg.title))
            continue;
        s.beginGroup(hCfg.title);
        hCfg.histGroupEn = s.value("en", hCfg.histGroupEn).toBool();
        hCfg.showFit = s.value("showFit", hCfg.showFit).toBool();

        hCfg.devCh = s.value("devCh", -1).toInt();
        if(hCfg.devCh != -1) {
            bool ok1,ok2;
            hCfg.devIndex = DeviceIndex(static_cast<quint16>(s.value("devId", 0).toString().toUInt(&ok1)),
                                        s.value("devSer", 0).toString().toULongLong(&ok2));
            if(!ok1 || !ok2)
                hCfg.devIndex = DeviceIndex();
        }

        s.endGroup();
    }
    s.endGroup();


    int offsetSize = s.beginReadArray("ChOffset");
    bool ok;
    for(int i=0; i<offsetSize; ++i) {
        s.setArrayIndex(i);
        quint16 id = static_cast<quint16>(s.value("id").toUInt(&ok));
        if(!ok) {
            qCritical() << QString("Bad configuration for chOffset #%1").arg(i);
            continue;
        }
        quint32 ser = s.value("serial").toUInt(&ok);
        if(!ok) {
            qCritical() << QString("Bad configuration for chOffset #%1").arg(i);
            continue;
        }
        DeviceIndex index(id, ser);
        if(index.isNull()) {
            qCritical() << QString("Bad configuration for chOffset #%1").arg(i);
            continue;
        }
        int offset = s.value("offset").toInt(&ok);
        if(!ok) {
            qCritical() << QString("Bad configuration for chOffset #%1").arg(i);
            continue;
        }
        devChOffset[index] = offset;
    }
    s.endArray();
}

void HistMngrCfg::save()
{
    QSettings s;
    s.beginGroup("Histogram");
    s.setValue("curHistType", curHistTitle);
    s.setValue("useStopCh", useStopCh);
    s.setValue("stopCh", stopCh);
    s.setValue("stopDevId", stopDevIndex.getDevId());
    s.setValue("stopDevSerial", stopDevIndex.getSerial());
    s.setValue("inlHistCodeDens", inlHistCodeDens);
    s.setValue("enabled", globalEnabled);
    s.setValue("taiTsWindow", taiTsWindow);
    s.setValue("mwMin", mwMin);
    s.setValue("mwMax", mwMax);
    s.setValue("log_y_axis", log_y_axis);

    for(int i=0; i<histCfg.size(); ++i) {
        auto &hCfg = histCfg[i];
        s.beginGroup(hCfg.title);
        s.setValue("en", hCfg.histGroupEn);
        s.setValue("showFit", hCfg.showFit);
        if(hCfg.devCh == -1) {
            s.remove("devId");
            s.remove("devSer");
            s.remove("devCh");
        } else {
            s.setValue("devId", QString("0x%1").arg(hCfg.devIndex.getDevId(), 0, 16));
            s.setValue("devSer", QString("0x%1").arg(hCfg.devIndex.getSerial(), 0, 16));
            s.setValue("devCh", hCfg.devCh);
        }
        s.endGroup();
    }
    s.endGroup();

    s.remove("ChOffset");
    s.beginWriteArray("ChOffset");
    int chOffsetIndex = 0;
    for (const DeviceIndex &index: devChOffset.keys()) {
        s.setArrayIndex(chOffsetIndex);
        s.setValue("ident", QString("0x%1_0x%2")
                   .arg(index.getDevId(), 2, 16, QChar('0'))
                   .arg(index.getSerial(), 8, 16, QChar('0')));
        s.setValue("id", index.getDevId());
        s.setValue("serial", index.getSerial());
        s.setValue("offset", devChOffset[index]);
        chOffsetIndex++;
    }
    s.endArray();
}

void HistMngrCfg::matchWinUpdated(int min, int max)
{
    mwMin = min;
    mwMax = max;
    histCfg[HistId::tdcTime].min = mwMin;
    histCfg[HistId::tdcTime].max = mwMax;
    histCfg[HistId::tdcTime].nBins = (mwMax-mwMin)*1024/24;
}

QString HistMngrCfg::getGroupToolTip(int groupId, bool with_suffix) const
{
    QString toolTip;
    if(groupId == -1)
        return QString();
    else if(HistId::tdcTaiTime)
        toolTip = "Histogram fills with trigger TAI";
    else if(HistId::tdcTime)
        toolTip = "Histogram fills with hit time from HPTDC part of event";
    else if(HistId::tdcWidth)
        toolTip = "Histogram fills with hit width from HPTDC part of event";
    else if(HistId::tdcCh)
        toolTip = "Hist. fills with ch. number of HPTDC hit";
    else if(HistId::tdcDoublePulseTime)
        toolTip = "Hist fills with hit time differance relative to each hit from 'StopCh'";
    else if(HistId::tdcHitEv)
        toolTip = "Hist. fills with number of HPTDC hits from all channels per event";
    else if(HistId::inl)
        toolTip = "Code density distribution or INL correction";
    else if(HistId::inlCheck)
        toolTip = "Hit time differance relative to all hits in 'StopCh'";
    else if(HistId::errors)
        toolTip = "Histogram fills with error bits from HPTDC event";

    if(with_suffix && toolTipSuffix.contains(groupId))
        toolTip.append(toolTipSuffix.value(groupId));
    return toolTip;
}

bool HistMngrCfg::doTdcProcessing() const
{
    return false ||
            histCfg.value(HistId::tdcTaiTime).histGroupEn ||
            histCfg.value(HistId::tdcHitEv).histGroupEn ||
            histCfg.value(HistId::tdcCh).histGroupEn ||
            histCfg.value(HistId::tdcTime).histGroupEn ||
            histCfg.value(HistId::tdcWidth).histGroupEn ||
            histCfg.value(HistId::inl).histGroupEn ||
            histCfg.value(HistId::tdcDoublePulseTime).histGroupEn ||
            histCfg.value(HistId::errors).histGroupEn;
}

QList<DeviceIndex> HistMngrCfg::getDevOrder() const
{
    QList<DeviceIndex> list;
    int ch=0;
    while(true) {
        DeviceIndex i = devChOffset.key(ch);
        if(i.isNull()) break;
        list << i;
        ch += getDeviceChNum(i);
    }
    Q_ASSERT(list.size() == devChOffset.size());
    return list;
}

int HistMngrCfg::getMaxChNum() const
{
    int maxChNum = 0;
    auto it = devChOffset.begin();
    while(it!=devChOffset.end()) {
        maxChNum = std::max(maxChNum, it.value() + getDeviceChNum(it.key()));
        ++it;
    }
    return maxChNum;
}

bool HistCfg::hasStopCh() const
{
    return (HistId::tdcDoublePulseTime!=-1 && groupId==HistId::tdcDoublePulseTime)
            ||(HistId::inlCheck !=-1 && groupId==HistId::inlCheck);
}
