#ifndef HISTWORKER_H
#define HISTWORKER_H

#include <QObject>

#include "mldiscover/DeviceIndex.h"
#include "histogramm/Histogramm.h"
#include "HistMngrCfg.h"

#include "Config.h"


#include "DeviceCalibration.h"
#include "DeviceData.h"


class QTime;
class QTimer;
class QSemaphore;
class QwtPlotCurve;

//class CompleteEvent;
//class DecodedData;
//class DeviceIndex;
//class TdcChHit;
//class TdcEvent;
//class TAI;
struct TestDp {
    TestDp() { clear(); }
    void clear() {
        dt=0;
        cnt=0;
        temp.clear();
        startTai.clear();
        started=false;
    }
    bool expired(TAI tai) {
        double dt = (tai.sec-startTai.sec)*1e9;
        dt += tai.nSec-startTai.nSec;
        return dt > 1e9;
    }

    double dt;
    int cnt;
    QMap<DeviceIndex, QMap<int, QPair<double, int> > > temp; // per [devIndex][tempId]={sum_temp, sum_cnt}
    TAI startTai;
    bool started;
};


struct HistContainer {
    QMap<int, Histogramm> baseHist;
    QMap<DeviceIndex, QMap<int, Histogramm> > inlHist;
    QMap<DeviceIndex, QMap<int, Histogramm> > inlCheckHist;
};
Q_DECLARE_METATYPE(HistContainer)

typedef QMap<DeviceIndex, int> HistDevType;
Q_DECLARE_METATYPE(HistDevType)

class HistWorker : public QObject
{
    Q_OBJECT
public:
    explicit HistWorker(QSemaphore *_histSem);
    ~HistWorker();
    void init(const HistMngrCfg &cfg);

signals:
    void devListUpdated(const HistDevType &devChOffset, const HistContainer &);
    void inlResultsUpdated(const InlResults &, bool allRes=false) const;
    void hist_updated(const HistContainer &, bool force) const;
    void resetHistMngr() const;

private slots :
    void clearAll();
    void handle_data();
    void refreshCurrentPlot(bool force=false);

public slots:
    void reset();
    void clearHist(int id, DeviceIndex=DeviceIndex(), int=0);
    void dataUpdated(const DecodedData &data);
    void hist_cfg_updated(const HistMngrCfg &newCfg);
    void inlConfigUpdated(InlResults inl);
    Histogramm &recreateTaiHist(int winSec);
    void tdcTtvxMapUpdated(TdcTtvxsMapType ttMap);
    void requestInlResults(bool allRes=false) const;

private:
    void addDevice(const DeviceIndex &index);
    void dataProcessingSingleEv(CompleteEvent &ev, int &tdcHitsPerEv);
    void dataProcessingTdc(TdcEvent &ev, int &tdcHitsPerEv, TdcEvent &tdcStopEvent, QVector<TdcChHit> &stopHits);
    void fillDoublePulse(Histogramm *histDp,
                         TdcEvent &tdcStopEvent, QVector<TdcChHit> &stopHits,
                         TdcEvent &ev, QVector<TdcChHit> &tdcEvents);
    QVector<double> getInl(const TdcEvent &ev, const TdcChHit &tdcEv) const;
    QVector<double> getInl(const DeviceIndex &index, uint ch) const;
    bool hasHistById(int id) const;
    Histogramm *getHistById(int id);
    Histogramm *getHistByIdIndexCh(int id, const DeviceIndex &index, uint ch);
    QMap<DeviceIndex, QMap<uint, uint> > getDevChCntMap();
    void fillDevChHist(QMap<DeviceIndex, QMap<uint, uint> > map);
    void trashAllData();
    void flushTestDp();

    HistContainer histContainer;
    QSemaphore * histSem;
    HistMngrCfg cfg;

    QList<DeviceIndex> knownDevices;
    quint32 taiReperSec;
    quint32 taiReperNs;
    InlResults curDevCorrMap; // INL from config
    InlResults inlResults; // collecting data for new INL
    QMap<DeviceIndex, QMap<uint, quint16> > devErrors;
    bool data_processing;
    bool destroy_flg;
    QList<DecodedData> data_list;
    TdcTtvxsMapType tdcTtvxsMap;
    const CompleteEvent *curEv;
    TestDp testDp;
};

#endif // HISTWORKER_H
