#ifndef HIST_H
#define HIST_H

#include <QString>
#include <QVector>

struct DistributionParams {
    DistributionParams() : cnt(0), mean(0), rms(0), min(0), max(0) {}
    uint cnt;
    double mean;
    double rms;
    double min;
    double max;
};

struct Histogramm
{
    Histogramm();
    Histogramm(QString _title, int _nBins, double _min, double _max);
    bool isEmpty();
    void reset();
    void resize(int _nBins, double _min, double _max);
    void fill(double v, int w=1);
    int findBin(double v) const;
    uint getUnderFlow() const { return under_flow; }
    uint getOverFlow() const { return over_flow; }
    uint getEntries() const;
    double getBinLower(int bin) const;
    double getBinCenter(int bin) const;
    double getBinUpper(int bin) const;
    DistributionParams getDistribution() const;

    QString title;
    int nBins; // exclude under/over-flow
    double min;
    double max;
    double binStep;

    QVector<double> bins;
    uint under_flow;
    uint over_flow;
    uint nEntries;
    double min_filled_val;
    double max_filled_val;
};

#endif // HIST_H
