#include "MStreamOutput.h"
#include "ui_MStreamOutput.h"

#include <QDebug>
#include <QPlainTextEdit>

namespace {
const QStringList ERROR_TAGS = {"fragDroped", "fragMissed", "checkFailed"};
const QString STAT_DUMPER_PREFIX = "[stat]";
const QString STAT_RECIEVER_PREFIX = "[RecvStat]";
}

MStreamOutput::MStreamOutput(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::MStreamOutput),
    ms_tag_dublicate(false)
{
    ui->setupUi(this);
    qRegisterMetaType<MStreamStatus>();

    QTableWidget * const table = ui->tableWidgetMStream;
    table->setColumnCount(1);
    table->setHorizontalHeaderItem(0, new QTableWidgetItem("no dev"));
    table->horizontalHeaderItem(0)->setData(Qt::UserRole, 0);
}

MStreamOutput::~MStreamOutput()
{
    delete ui;
}

void MStreamOutput::add_device(QString serial)
{
    QTableWidget * const table = ui->tableWidgetMStream;
    int col = -1;
    if(table->columnCount()==1 && table->horizontalHeaderItem(0)->data(Qt::UserRole).toULongLong()==0) {
        col = 0;
    }
    if(col==-1) {
        col = table->columnCount();
        table->setColumnCount(col+1);
        for(int r=0; r<table->rowCount(); ++r)
            table->setItem(r, col, new QTableWidgetItem());
    }
    table->setHorizontalHeaderItem(col, new QTableWidgetItem(serial));
    quint64 ser = 444;
    table->horizontalHeaderItem(col)->setData(Qt::UserRole, ser);

    for(int i=0; i<ui->tabWidgetLog->count(); ++i) {
        if(ui->tabWidgetLog->tabText(i) == serial) {
            qCritical() << "addMStreamLogTab: tab is already created";
            return;
        }
    }

    QPlainTextEdit *tEdit = new QPlainTextEdit(ui->tabWidgetLog);
    tEdit->setReadOnly(true);
    tEdit->setMaximumBlockCount(200);
    ui->tabWidgetLog->addTab(tEdit, serial);
    error_status[serial][STAT_DUMPER_PREFIX] = false;
    error_status[serial][STAT_RECIEVER_PREFIX] = false;
}

void MStreamOutput::remove_device(QString serial)
{
    QTableWidget *table = ui->tableWidgetMStream;
    const int mstreamCol = get_column_by_serial(serial);
    if(mstreamCol != -1) {
        if(table->columnCount()==1) {
            Q_ASSERT(mstreamCol == 0);
            table->horizontalHeaderItem(0)->setText("no Dev");
            table->horizontalHeaderItem(0)->setData(Qt::UserRole, 0);
            for(int r=table->rowCount()-1; r>=0; --r)
                table->item(r, 0)->setText("-");
        } else {
            table->removeColumn(mstreamCol);
        }
    }

    for(int i=0; i<ui->tabWidgetLog->count(); ++i) {
        if(ui->tabWidgetLog->tabText(i) == serial) {
            ui->tabWidgetLog->removeTab(i);
            return;
        }
    }
}

void MStreamOutput::mstream_log_output(QString serial, QString str)
{
    for(int i=0; i< ui->tabWidgetLog->count(); ++i){
        if(ui->tabWidgetLog->tabText(i) != serial) continue;

        QPlainTextEdit *tEdit = qobject_cast<QPlainTextEdit*>(ui->tabWidgetLog->widget(i));
        if(tEdit)
            tEdit->appendPlainText(str);
        return;
    }
}

void MStreamOutput::mstream_table_output(QString serial, QString out_str_full)
{
    const int col = get_column_by_serial(serial);
    auto &dev_stat = status[serial];

    for (const QString &out_str: out_str_full.split('\n')) {
//    QString tStr =
//    "[stat]packages=381;packLastSec=100;retransmitCount=0;evCount=381;evCountLastSec=100;fragAll=381;fragDroped=0;fragMissed=0;recvSize_B=365760;recvSizeLastSec_kBs=96;upTimeSec=6;tcpOccupancy=0.00"
//    "[RecvStat] noSpaceLoop=0;processDidntHelp=0;loopOk=999;checkFailed=0;noData=832 Loop count:999; LoopTime:1675ms;Time/Loop:1676 us;"
//    "pckSize: 0:832 1:167";

        QRegularExpression re("(?<tag>[^\\[\\]\\=]+?)"
                              "="
                              "(?<val>[+-]?[\\d.]+);?");
        re.setPatternOptions(QRegularExpression::MultilineOption);

        const QString badFormatTag = "badFormatPackages";
        const QString speedTag = "recvSizeLastSec_kBs";
        QString prefix;
        if(out_str.startsWith(STAT_DUMPER_PREFIX) && !out_str.contains(STAT_RECIEVER_PREFIX)) {
            prefix = STAT_DUMPER_PREFIX;
        } else if(out_str.startsWith(STAT_RECIEVER_PREFIX) && !out_str.contains(STAT_DUMPER_PREFIX)) {
            prefix = STAT_RECIEVER_PREFIX;
        }

        if(!prefix.isEmpty()) {
            bool has_error = false;
            auto &tags = (prefix == STAT_DUMPER_PREFIX) ? dumper_tags : reciever_tags;
            for (const QString &k: tags.keys()) {
                tags[k] = "";
            }

            QRegularExpressionMatchIterator i = re.globalMatch(out_str, prefix.size());
            while(i.hasNext()) {
                QRegularExpressionMatch match = i.next();
                QString tagStr = match.captured("tag");
                QString valStr = match.captured("val");
                tags[tagStr] = valStr;
            }

            if(prefix==STAT_DUMPER_PREFIX && tags.contains(speedTag)) {
                if(tags.contains(badFormatTag)) {
                    dev_stat.bad_format = tags[badFormatTag];
                } else {
                    dev_stat.bad_format.clear();
                }
                dev_stat.recv_speed = tags[speedTag];
            }
            emit mstream_stat_updated(serial, dev_stat);

            if(col != -1) {
                if(ui->tableWidgetMStream->updatesEnabled())
                    ui->tableWidgetMStream->setUpdatesEnabled(false);
                for (const QString &tag: tags.keys()) {
                    int tagRow = get_mstream_row(tag);
                    const auto &tagValStr = tags[tag];
                    auto cellItem = ui->tableWidgetMStream->item(tagRow, col);
                    cellItem->setText(tagValStr);
                    if(ERROR_TAGS.contains(tag)) {
                        bool ok;
                        bool err = tagValStr.isEmpty() && tagValStr.toInt(&ok)!=0 && ok;
                        QColor tagCol = err ? Qt::red : Qt::transparent;
                        if(cellItem->backgroundColor() != tagCol)
                            cellItem->setBackgroundColor(tagCol);
                        has_error |= err;
                    }
                }
            }
            if(error_status.value(serial).value(prefix, false) != has_error) {
                error_status[serial][prefix] = has_error;
                update_error_status(serial);
            }
        }
        check_for_tag_dublicate(out_str, out_str_full);
    }

    if(!ui->tableWidgetMStream->updatesEnabled()) {
        ui->tableWidgetMStream->resizeColumnsToContents();
        ui->tableWidgetMStream->setUpdatesEnabled(true);
    }
}


int MStreamOutput::get_mstream_row(const QString &tag)
{
    int row = mstream_tags.indexOf(tag);
    if(row < 0) {
        const QString pckSizePrefix = "pckSize";
        if(tag.contains(pckSizePrefix)) {
            int mySize = QString(tag).remove(pckSizePrefix).toInt();
            row = mstream_tags.size();
            for(int i=0; i<mstream_tags.size(); ++i) {
                const QString &curTag = mstream_tags.value(i);
                if(!curTag.contains(pckSizePrefix)) {
                    if(row != mstream_tags.size()) {
                        ++row;
                        break;
                    }
                    continue;
                }
                row = i;
                int curSize = QString(curTag).remove(pckSizePrefix).toInt();
                if(curSize>mySize)
                    break;
            }
        } else {
            row = mstream_tags.size();
        }
        mstream_tags.insert(row, tag);
        QTableWidget *t = ui->tableWidgetMStream;
        t->insertRow(row);
        t->setVerticalHeaderItem(row, new QTableWidgetItem(tag));
        for(int col=0; col<t->columnCount(); ++col)
            t->setItem(row, col, new QTableWidgetItem("-"));
    }
    return row;
}

void MStreamOutput::check_for_tag_dublicate(QString line, QString full_text)
{
    if(!ms_tag_dublicate) {
        for(auto tag : dumper_tags.keys()) {
            for(auto tag2 : reciever_tags.keys()) {
                if(tag == tag2) {
                    qInfo() << "================" << tag;
                    qInfo() << "======out_str==========";
                    qInfo() << line;
                    qInfo() << "======out_str_full==========";
                    qInfo() << full_text;
                    qInfo() << "======dumpersTag==========";
                    qInfo() << dumper_tags;
                    qInfo() << "======recieverTag==========";
                    qInfo() << reciever_tags;
                    qInfo() << "================";
                    ms_tag_dublicate = true;
                }
            }
        }
    }
}

void MStreamOutput::update_error_status(QString serial)
{
    bool has_err = false;
    for(bool err : error_status[serial].values())
        has_err |= err;


    QTableWidget *table = ui->tableWidgetMStream;
    const int mstreamCol = get_column_by_serial(serial);
    QColor color = has_err ? Qt::red : Qt::black;
    if(mstreamCol != -1) {
        auto header_item = table->horizontalHeaderItem(mstreamCol);
        if(header_item->textColor() != color)
            header_item->setTextColor(color);
    }

    for(int i=0; i<ui->tabWidgetLog->count(); ++i) {
        if(ui->tabWidgetLog->tabText(i) == serial) {
            if(ui->tabWidgetLog->tabBar()->tabTextColor(i) != color)
                ui->tabWidgetLog->tabBar()->setTabTextColor(i, color);
            break;
        }
    }
}


int MStreamOutput::get_column_by_serial(QString serial)
{
    QTableWidget *table = ui->tableWidgetMStream;
    for(int col=table->columnCount()-1; col>=0; --col) {
        if(table->horizontalHeaderItem(col)->text() == serial){
            return col;
        }
    }
    return -1;
}



void MStreamOutput::on_checkBox_table_toggled(bool checked)
{
    ui->tableWidgetMStream->setVisible(checked);
}

void MStreamOutput::on_checkBox_log_toggled(bool checked)
{
    ui->tabWidgetLog->setVisible(checked);
}
