//
//    Copyright 2014-2015 Ivan Filippov
//    Copyright 2014-2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef CLIENTMANAGERWIDGET_H
#define CLIENTMANAGERWIDGET_H

#include "RunStates.h"
#include "mongo/RootConfig.h"
#include "Client.h"
#include "ClientConfig.h"
#include "remote-control-server/types.h"
#include "RcClientData.h"
#include "RcClientManager.h"

#include <QWidget>
#include <QVector>
#include <QMap>
#include <QDateTime>

class QHostAddress;
class Client;
class Config;
class ProgramDescription;
class TrigStatus;
class QElapsedTimer;

namespace Ui {
class ClientManagerWidget;
}

class ClientManagerWidget : public QWidget
{
    Q_OBJECT
public:
    explicit ClientManagerWidget(QWidget *parent = nullptr);
    virtual ~ClientManagerWidget();
    void init(QMap<int, ClientConfig> client_config_);
    void set_run_enable(bool run_enable_);

    // temporary proxies
    RcRunId get_run_id() const { return m->ri.get_run_id(); }
//    QString get_run_index() const { return m->ri.get_run_index(); }
//    int get_run_number() const { return m->ri.get_run_number(); }
    RUN_STATES getRunState() const { return m->ri.getRunState(); }

signals:
    void deleteClient(int index); // 0 - for all
    void startListen(int index); // 0 - for all
    void stopListen(int index); // 0 - for all
    void sendRunState(RUN_STATES state, const RcRunId &id);
    void sendStartRun(const RcRunId &id);
    void sendStopRun(const RcRunId &id);
    void runError(const RcRunId &id, QString errStr);
    void runWarn(const RcRunId &id, QString warnStr) const;
    void setClientAddress(int clientId, const QHostAddress &host, quint16 port);
    void setClientEmptyAddress(int clientId);
    void updateTabState(bool isOk);
    void clientMapUpdated(QMap<int,QString> map) const; // for evNumCheck
    void evNumRequest(int checkIndex) const;
    void evNumResponse(int checkIndex, int clientId, EvNumSet evNum) const;
    void backPressure(const RcRunId &id, bool backPressure);
    void config_transfer(int id, RootConfig root);
    void client_config_changed(QMap<int, ClientConfig> client_config);
    void programDiscovered(const ProgramDescription &newDescr);
    void programClosed(const ProgramDescription &newDescr);
    void vmeFlagUpdated() const;

public slots:
    void onRestartAll(const RcRunId &runId);
    void onStopAll();
    void setControlEnable(bool enabled);

private slots:
    void on_pushButtonAdd_clicked();
    void on_pushButtonRemove_clicked();
    void on_tableWidgetClients_itemSelectionChanged();

    void clientRunError(const RcRunId &runId, QString errStr);
    void clientRunWarn(const RcRunId &runId, QString warnStr);
    void clientBackPressure(const RcRunId &runId, bool bp);
    void clientStatusUpdated(int id, QString str);
    void clientDescrUpdated(int id, QString str);
    void clientRunStateUpdated(int id, RUN_STATES state, const RcRunId &runId);
    void clientReadoutToggled(bool readout);
    void gotProgramDescription(const ProgramDescription &newDescr);
    void gotClosedProgram(const ProgramDescription &descr);
    void clientReady4Run(int id, bool ready);
    void clientVmeFlag(int id, bool hasVme);
    void clientProtocolChanged(int id, CLIENT_PROTO_ID protoId);
    void updateButtonsState();
    void checkBackupStatus();

private:
    void setupTable();
    int addClient(ClientConfig cc);
    int getClientRow(QObject *sender) const;
    int getClientRow(int clientId) const;
    inline int getClientId(int row) const;
    int getClientId(QObject *sender) const;

    QString getClientDescr(int clientId) const;
    QVector<ProgramDescription> getFreeProgs() const;
    bool isFreeProgs(const ProgramDescription &descr) const;
    int getSelectedRow() const;
    bool getClientsStates() const;
    void highlightClient(int id);
    void sendClientMap() const;
    void updateWarnString() const;
    static bool isSupportEvNumCheck(QString type);
    void updateVisibilityColumn(int col = -1);
    void removeAllClients();
    void removeClient(int clientId, int rowToRemove = -1, QElapsedTimer *timer=nullptr);
    void sendUpdatedClientConfig();
    bool isBackPressureOn() const;

    Ui::ClientManagerWidget *ui;
    RcClientManager * const m;

    QMap<int, ClientInfo> clientsInfo;
    QVector<ProgramDescription> progDescrs;
    bool controlEnabled;
    QTimer *const statusBackupTimer;
    QMap<int, StatusBackup> statusBackup; // by client id
    quint32 curSpillNum;
    QMap<int, bool> backPressureMap;
    bool run_enable;
};

#endif // CLIENTMANAGERWIDGET_H
