//
//    Copyright 2016 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "MetricSender.h"

#include <QUdpSocket>
#include <QElapsedTimer>

#include "singleton.h"
#include "MetricSenderHelper.h"
#include "StatsAccumulator.h"

MetricSender::MetricSender(QObject *parent) :
    QObject(parent),
    helper(new MetricSenderHelper(this)),
    numSent(0),
    numDropped(0),
    acc(new StatsAccumulator(10)),
    socket(new QUdpSocket(this))
{
    connect(this, &MetricSender::internalSendSignal,
            this, &MetricSender::internalSendSlot, Qt::QueuedConnection);
}

MetricSender::~MetricSender()
{
    socket->abort();
    if (helper->isEnabled())
        qInfo() << QString("Metrics sent %1, dropped %2")
               .arg(numSent)
               .arg(numDropped);
    else
        qInfo() << QString("Metrics disabled, dropped %1")
               .arg(numDropped);
    delete acc;
}

MetricSender & MetricSender::instance()
{
    MetricSender &instance = Singleton<MetricSender>::instance();
    instance.setObjectName("MetricSender");
    return instance;
}

void MetricSender::setNamespace(const QString &s)
{
    instance().helper->setNamespace(s);
}

void MetricSender::send(const QString &name, const QVariant &value)
{
//    QElapsedTimer t;
//    t.start();
    emit instance().internalSendSignal(name, value);
//    if (rand() % 10 == 0) qInfo() << t.nsecsElapsed();
}

void MetricSender::internalSendSlot(const QString &name, const QVariant &value)
{
    if (!helper->isEnabled()) {
        numDropped++;
        return;
    }
    const QString &line = helper->formatMetricLine(name, value);
    qint64 numWritten = socket->writeDatagram(line.toUtf8(), helper->hostAddr(), helper->hostPort());
    if (numWritten == -1) {
//        qWarning() << socket->errorString();
        numDropped++;
    } else {
        numSent++;
    }
}
