//
//    Copyright 2016 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

// Usage
// define environment variable METRIC_HOST=hostname:port
// Create persistent instance of MetricSender after QApplication initialization (main.cpp)
// MetricSender::instance().install();
// To send a metric, use MetricSender::send(metric, value);

#ifndef METRICSENDER_H
#define METRICSENDER_H

#include <QObject>
#include <QHostAddress>
class StatsAccumulator;

class MetricSenderHelper;
class QUdpSocket;

class MetricSender : public QObject
{
    Q_OBJECT
public:
    explicit MetricSender(QObject *parent = nullptr);
    virtual ~MetricSender();
    static MetricSender & instance();
    void install() {} // keep it here
    static void setNamespace(const QString &s);
    static void send(const QString &name, const QVariant &value);

signals:
    void internalSendSignal(const QString &name, const QVariant &value);

protected slots:
    void internalSendSlot(const QString &name, const QVariant &value);

protected:
    MetricSenderHelper * const helper;
    quint64 numSent;
    quint64 numDropped;
    StatsAccumulator * const acc;

private:
    QUdpSocket * const socket;
};

#endif // METRICSENDER_H
