//
//    Copyright 2016 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "MetricSenderAsync.h"

#include "singleton.h"
#include "MetricSenderHelper.h"
#include "TcpMetricSender.h"
#include "base/ThreadController.h"

MetricSenderAsync::MetricSenderAsync(QObject *parent) :
    QObject(parent),
    numSent(0),
    numDropped(0),
    tcpMetricSender(new TcpMetricSender(nullptr)) // parent should be NULL
{
    connect(this, &MetricSenderAsync::privateSendMetric,
            tcpMetricSender, &TcpMetricSender::sendMetric, Qt::QueuedConnection);
    connect(this, &MetricSenderAsync::privateSetNamespace,
            tcpMetricSender, &TcpMetricSender::setNamespace, Qt::QueuedConnection);
    tcpMetricSender->setObjectName("MetricSender");
    new ThreadController(tcpMetricSender, this);
}

MetricSenderAsync::~MetricSenderAsync()
{
    qInfo() << QString("MetricSenderAsync stats: sent %1, dropped %2 messages").arg(numSent).arg(numDropped);
}

MetricSenderAsync & MetricSenderAsync::instance()
{
    MetricSenderAsync &instance = Singleton<MetricSenderAsync>::instance();
    instance.setObjectName("MetricSenderAsync");
    return instance;
}

void MetricSenderAsync::setNamespace(const QString &s)
{
    emit MetricSenderAsync::instance().privateSetNamespace(s);
}

void MetricSenderAsync::send(const QString &name, const QVariant &value)
{
    MetricSenderAsync &metricSenderAsync = instance();
    if (metricSenderAsync.tcpMetricSender->sem.tryAcquire()) {
        emit metricSenderAsync.privateSendMetric(name, value);
        metricSenderAsync.numSent++;
    } else {
        // silently drop message
        metricSenderAsync.numDropped++;
    }
}
