/*
**    Copyright 2019 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "MlinkDeviceInterfaceDump.h"
#include "MlinkDeviceInterfaceBase.h"

namespace mlink {

int MlinkDeviceInterfaceDump::ml_open(const char *devname)
{
    int ret = MlinkDeviceInterfaceBase::ml_open(devname);
    ml_rx_dump_file = fopen("rx.txt", "a");
    fprintf(ml_rx_dump_file, "0 0 50bc\n");
    ml_tx_dump_file = fopen("tx.txt", "a");
    fprintf(ml_tx_dump_file, "0 0 50bc\n");
    return ret;
}

int MlinkDeviceInterfaceDump::ml_close()
{
    int ret = MlinkDeviceInterfaceBase::ml_close();
    if (ml_rx_dump_file)
        fclose(ml_rx_dump_file);
    ml_rx_dump_file = nullptr;
    if (ml_tx_dump_file)
        fclose(ml_tx_dump_file);
    ml_tx_dump_file = nullptr;
    return ret;
}

int MlinkDeviceInterfaceDump::ml_dma_read (uint32_t buf[], size_t nwords, size_t * words_received, int timeout)
{
    int ret = MlinkDeviceInterfaceBase::ml_dma_read(buf, nwords, words_received, timeout);
    if (ret == 0 && buf && words_received) {
        fprintf(ml_rx_dump_file, "0 1 f7f7\n");
        for(size_t i=0; i<*words_received; i++) {
            fprintf(ml_rx_dump_file, "1 0 %02x%02x\n", ((unsigned char *)buf)[4*i+3], ((unsigned char *)buf)[4*i+2]);
            fprintf(ml_rx_dump_file, "1 0 %02x%02x\n", ((unsigned char *)buf)[4*i+1], ((unsigned char *)buf)[4*i]);
        }
        fprintf(ml_rx_dump_file, "0 0 50bc\n");
        fflush(ml_rx_dump_file);
    }
    return ret;
}

int MlinkDeviceInterfaceDump::ml_dma_write (const uint32_t buf[], size_t nwords, int timeout)
{
    int ret = MlinkDeviceInterfaceBase::ml_dma_write (buf, nwords, timeout);
    if (buf) {
        fprintf(ml_tx_dump_file, "0 1 f7f7\n");
        for(size_t i=0; i<nwords; i++) {
            fprintf(ml_tx_dump_file, "1 0 %02x%02x\n", ((unsigned char *)buf)[4*i+3], ((unsigned char *)buf)[4*i+2]);
            fprintf(ml_tx_dump_file, "1 0 %02x%02x\n", ((unsigned char *)buf)[4*i+1], ((unsigned char *)buf)[4*i]);
        }
        fprintf(ml_tx_dump_file, "0 0 50bc\n");
        fflush(ml_tx_dump_file);
    }
    return ret;
}

}
