#ifndef _MLINK_H
#define _MLINK_H

#define ML_DEVICE_NAME "mlink"

/*
 * Using M-Link Specification version 1.0
 * http://afi.jinr.ru/pub/doc/M-Link/M-Link%20Data%20Interface%20Manual.html
 *
 * M-Link data layer operates with 4-byte words only.
 * Physical layer may be 16 bits wide or any other.
 */

enum { ML_RX_BUFFER_SIZE = 65536 }; // 64 Kbytes
enum { ML_RX_BUFFER_WORDS = ML_RX_BUFFER_SIZE/4 }; // words


typedef struct {
    unsigned char *buf; // virtual address
    unsigned long size; // size
    int timeout; // timeout in ms, -1 for infinite, 0 for immediate
} ml_dmaop_t;

typedef struct {
    const unsigned char *buf; // virtual address
    unsigned long size; // size
    int timeout; // timeout in ms, -1 for infinite, 0 for immediate
} ml_dmaop_ro_t;

typedef struct {
    unsigned int timeout_ms;
    unsigned int mask;
} ml_poll_ev_op_t;

// Events, bit fields. May be OR-ed.
enum {
    ML_EV_BIT_SPILL_ON  = 0x0001,
    ML_EV_BIT_SPILL_OFF = 0x0002,
    ML_EV_BIT_DATA = 0x0004,
    ML_EV_BIT_RX_PKT = 0x0008,
};

#pragma pack(push, 1)
typedef union {
    unsigned int all;
    struct {
        unsigned int link_down: 1;      // no connection
        unsigned int data_error: 1;     // link data error
        unsigned int pci_error: 1;      // pci bus signaled error (unused)
        unsigned int internal_error: 1; // any internal error
        unsigned int loss_of_signal: 1; // LOS#
        unsigned int laser_fault: 1;    // O_FAULT
        unsigned int lcf_timeout: 1;    // lcf_timeout
        unsigned int reserved: 1;
    } bit;
} mlink_status_t;

typedef union {
    unsigned int all;
    struct {
        unsigned int int_loop_test_unavailable : 1;
        unsigned int serdes_loop_test : 1;
        unsigned int tx_cost : 10;
        unsigned int rx_cost : 10;
        unsigned int unused : 10;
    } bit;
} mlink_caps_t;
#pragma pack(pop)

typedef struct {
    unsigned int port;
    unsigned int offset;
    unsigned int data;
    unsigned int unused;
} ml_port_op_t;

typedef struct {
    unsigned int offset;
    unsigned int data;
} ml_flasher_op_t;

typedef struct {
    unsigned int fw_ver_maj;
    unsigned int fw_ver_min;
    unsigned int fw_rev;
    unsigned int reserved;
} firmware_version_t;

// 64 bytes
typedef struct {
    firmware_version_t fw_version;
    unsigned long long board_serial;
    unsigned long long reserved_0;
    unsigned long long reserved_1;
    unsigned long long reserved_2;
    unsigned long long reserved_3;
    unsigned long long reserved_4;
} board_info_t;

#endif /* _MLINK_H */
