#ifndef MLINK_REGIOMLINK_H
#define MLINK_REGIOMLINK_H

/*
 * Register-based I/O
 */

#include <cstddef>
#include <string>
#include <vector>
#include <list>
#include <regio/regiobase.h>

namespace mlink {

class MlinkDeviceInterfaceBase;

    class RegIOMLink: public RegIOBase {
    public:
        RegIOMLink(const std::string &name = std::string()); //<! Construct M-Link Register I/O object with specified device name
         ~RegIOMLink() override;
        static std::string defaultDevName();
        void reset() override;
        //  virtual void setLocalJtagMode(bool state = true);
        //  virtual RegIoPacket io(const RegIoPacket &dataToSend);
        void rawSend(const RegIoPacket &packet) override; //< \deprecated
        //  virtual void sendPackets(const PacketList &);
        static unsigned long getMaxDataSize();
        bool isConnected() final; // requires ioctl()
        int open(); // returns -errno
        int close(); // returns -errno
        bool isOpen() const;

    protected:
        double getDeviceThroughput() const override;
        RawPacketList getRxPackets(double timeout) override; //<! get packets from hardware
        int nextTxPacketSequenceNumber() { return txPacketSequenceNumber++; }

        MlinkDeviceInterfaceBase * const dev;
        std::vector<quint32> rawRxBuf;
        int txPacketSequenceNumber;
    };
} // namespace mlink

#endif /* MLINK_REGIOMLINK_H */
