//
//    Copyright 2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef WREPSTATUS_H
#define WREPSTATUS_H

#include <QtGlobal>

class WrEpStatus
{
public:
    bool timeNull() const { return 0 == (wrTimecode >> 2);}
    bool timeValid() const {return ((wrTimecode & 0x3) == 0x2) && !timeNull(); }

    quint64 mSecsSinceEpoch() const {
        quint64 ms_sec = (quint64)wrTAISeconds() * 1000;
        quint64 ms_ns = (quint64)wrTAINanoseconds() / 1000000;
        return  ms_sec + ms_ns;
    }
    quint32 wrTAISeconds() const { return wrTimecode >> 32;}
    quint32 wrTAINanoseconds() const { return (wrTimecode >> 2) & 0x3FFFFFFF;}
    /// Check time is in sync with local time, ntpd should be running.
    int wrTimeCheck() const;
    WrEpStatus & operator-=(const WrEpStatus& st) {
        wrUptime     -= st.wrUptime;
        wrLinkError  -= st.wrLinkError;
        wrSyncFail   -= st.wrSyncFail;
        wrCodeError  -= st.wrCodeError;
        wrOtherError -= st.wrOtherError;
        return *this;
    }
    bool hasWrEpStatus = false;
    qint64 acquiredAtMs = 0;
    qint32 acquireErrorMs = 0;
    quint16 wrStatusCode = 0; // Reg 0x60
    quint16 wrReserved = 0;   // Reg 0x61
    quint32 wrUptime = 0;     // Reg 0x62
    quint64 wrTimecode = 0;   // Reg 0x64
    quint32 wrLinkError = 0;  // Reg 0x68
    quint32 wrSyncFail = 0;   // Reg 0x6A
    quint32 wrCodeError = 0;  // Reg 0x6C
    quint32 wrOtherError = 0; // Reg 0x6E
};

#endif // WREPSTATUS_H
