//
//    Copyright 2013 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef REGOP_H
#define REGOP_H

#include <QtGlobal>
#include <QVector>
#include <QDebug>

namespace mlink {

class RegOp
{
public:
    RegOp() = default;
    RegOp(bool in_write, quint16 in_addr, quint16 in_data) : write(in_write), addr(in_addr), data(in_data) {}
    bool write = false;
    quint16 addr = 0;
    quint16 data = 0;
};

class RegRead : public RegOp
{
public:
    RegRead() = default;
    explicit RegRead(quint16 in_addr) : RegOp(false, in_addr, 0) {}
};

class RegWrite : public RegOp
{
public:
    RegWrite() = default;
    RegWrite(quint16 in_addr, quint16 in_data) : RegOp(true, in_addr, in_data) {}
};

class RegOpVector: public QVector<RegOp>
{
public:
    RegOpVector() = default;
    RegOpVector(std::initializer_list<RegOp> args):QVector(args) {}
};

class RegRead32 : public RegOpVector
{
public:
    RegRead32() = default;
    explicit RegRead32(quint16 in_addr) {
        operator << (RegRead(in_addr));
        operator << (RegRead(in_addr+1));
    }
};

class RegWrite32 : public RegOpVector
{
public:
    RegWrite32() = default;
    explicit RegWrite32(quint16 in_addr, quint32 in_data) {
        operator << (RegWrite(in_addr, in_data & 0xFFFF));
        operator << (RegWrite(in_addr+1, (in_data >> 16) & 0xFFFF));
    }
};

class RegRead64 : public RegOpVector
{
public:
    RegRead64() = default;
    explicit RegRead64(quint16 in_addr) {
        operator << (RegRead32(in_addr));
        operator << (RegRead32(in_addr+2));
    }
};

class RegWrite64 : public RegOpVector
{
public:
    RegWrite64() = default;
    explicit RegWrite64(quint16 in_addr, quint64 in_data) {
        operator << (RegWrite32(in_addr, in_data & 0xFFFFFFFF));
        operator << (RegWrite32(in_addr+2, (in_data >> 32) & 0xFFFFFFFF));
    }
};

} // namespace mlink


QDebug operator <<(QDebug &stream, const mlink::RegOp &v);

#endif // REGOP_H
