#include "DaqConfigDataBase.h"
#include "daq-config/DaqConfigDataBase.h"

#include <QCoreApplication>
#include <QDebug>
#include <QMetaType>
#include <QSqlRecord>
#include <QSqlError>
#include <QSettings>
#include <QVariant>

DaqConfigDataBase::DaqConfigDataBase(DaqConfigTypes t) :
    DaqConfig(t),
    configName("default")
{
    if(t == DaqConfigScheduler){
        table_name = "SchedulerConfig";
    }
    if(t == DaqConfigGenerator){
        table_name = "GeneratorConfig";
    }
    if(t == DaqConfigHrbAnalyzer){
        table_name = "AnalyzerConfig";
    }
}

bool DaqConfigDataBase::readSettings()
{
    runIndex = getIndex();
    if(names_val_list.size() == 0){
        if((m_vals.size() != 0) && (names_val_list.size() == 0)){
            for (QString str : m_vals.keys()) {
                names_val_list <<  str;
            }
        }
        else{
            qInfo() << "parametrs map is empty! Please, use set() method";
        }
        names_val = names_val_list.join(",");
        names_val.append(" ");
        names_val.prepend(" ");
    }

    qInfo() << "=====Read settings===";
    runIndex = getIndex();
    //---read setting from config, then overwrite, if DB is ok
    readSettings_fromConfig();

    QSqlQuery query(db);
    if(!db.isOpen()){
        qWarning() << QString("Fail to open DataBase %1. Configs loaded from ./config").arg(db.databaseName());
        return false;
    }

    if(configName == "default"){
        //---read default settings
        query.prepare(QString(
"           SELECT %1                           "
"             FROM %2                           "
"            WHERE runIndex  = :runIndex        "
"              AND configName = :configName     "
        ).arg(names_val).arg(table_name));
        (runIndex == "" ? query.bindValue(":runIndex", QString("")) : query.bindValue(":runIndex", runIndex));
        query.bindValue(":configName", "default");
        if (query.exec()){
            switch (query.size()) {
            case 0:{
                if (upload_zero_configurations()){
                    return true;
                }
                else{
                    return false;
                }
            }
            case 1:{
                query.next();
                update_parameters_from_query(query);
                return true;
            }
            default:{
                qWarning() << QString("Warning! doubled row in table %1. Please, check run_index %2 with DEFAULT config_name")
                              .arg(db.databaseName()).arg(runIndex);
                query.next();
                update_parameters_from_query(query);
                return true;
            }
            }
        }
        else{
            qWarning() << QString("Failed to make select from SchedulerConfig table: %1")
                          .arg( query.lastError().text());
            return false;
        }
    }
    else{
        //---read saved settings
        QList<QString> config_list = get_configurations_list();
        int sum=0;
        for(int i=0; i< config_list.size(); ++i){
            if(config_list[i] == configName){
                ++sum;
            }
        }
        QSqlQuery query(db);
        if(!db.isOpen()){
            qWarning() << QString("Fail to open DataBase %1").arg(db.databaseName());
            return  false;
        }
        query.prepare(QString(
"           SELECT %1                               "
"             FROM %2                               "
"            WHERE configName = :configName         "
        ).arg(names_val).arg(table_name));
        query.bindValue(":configName", configName);
        switch (sum) {
        case 0:{
            qWarning() << QString("Can't switch to configName %1, it doesn't exist").arg(configName);
            return false;
        }
        case 1:{
            if (query.exec()){
                qInfo() << QString("Switch to configuration %1").arg(configName);
                query.next();
                update_parameters_from_query(query);
                return true;
            }
            else{
                qWarning() << QString("Failed to make SELECT to SchedulerConfig table: %1")
                    .arg( query.lastError().text());
                return false;
            }
        }
        default:{
            qWarning() << QString("Doubled row in DB. Please, check ConfigName = %1").arg(configName);
            if (query.exec()){
                qInfo() << QString("Switch to configuration %1").arg(configName);
                query.next();
                update_parameters_from_query(query);
                return true;
            }
            else{
                qWarning() << QString("Failed to make SELECT to SchedulerConfig table: %1")
                    .arg( query.lastError().text());
                return false;
            }
        }
        }
    }
}

//---use it for febug print---
void DaqConfigDataBase::debug_vals_print()
{
    qInfo() << "===debug print===";
    for (QString str :  m_vals.keys()) {
        if(QMetaType::type(m_vals[str].typeName()) == QMetaType::Int){
            qInfo() << str << ":" << get<int>(str);
        }
        if(QMetaType::type(m_vals[str].typeName()) == QMetaType::QString){
            qInfo() << str << ":" << get<QString>(str);
        }
        if(QMetaType::type(m_vals[str].typeName()) == QMetaType::QDateTime){
            qInfo() << str << ":" << get<QDateTime>(str);
        }
        if(QMetaType::type(m_vals[str].typeName()) == QMetaType::Double){
            qInfo() <<  str << ":" << get<double>(str);
        }
    }
}

void DaqConfigDataBase::update_parameters_from_query(QSqlQuery query)
{
    QSqlRecord rec = query.record();
    for(int i=0; i< rec.count(); ++i){
        set(rec.fieldName(i), query.value(i) );
    }
}

bool DaqConfigDataBase::upload_zero_configurations()
{
    for (QString str : m_vals.keys()) {
        set(str,1);
    }
    configName = "default";
    runIndex = getIndex();
    if (writeSettings()){
        return true;
    }
    else{
        return false;
    }
}


bool DaqConfigDataBase::writeSettings()
{
    runIndex = getIndex();
    if(names_val_list.size() == 0){
        if((m_vals.size() != 0) && (names_val_list.size() == 0)){
            for (QString str : m_vals.keys()) {
                names_val_list <<  str;
            }
        }
        else{
            qInfo() << "parametrs map is empty! Please, use set() method";
        }
        names_val = names_val_list.join(",");
        names_val.append(" ");
        names_val.prepend(" ");
    }

    qInfo() << "=====Write settings===";

    //---anyone, save configs in .config
    writeSettings_toConfig();

    QSqlQuery query(db);
    if(!db.isOpen()){
        qWarning() << QString("Fail to open DataBase %1").arg(db.databaseName());
    }

    if(configName == "default"){
        //---try to find configs of this runIndex and "default" configName in DB
        query.prepare(QString(
"           SELECT id                               "
"             FROM %1                               "
"            WHERE runIndex  = :runIndex            "
"              AND configName = :configName         "
        ).arg(table_name));
        (runIndex == "" ? query.bindValue(":runIndex", QString("")) : query.bindValue(":runIndex", runIndex));
    }
    else{
        //---try to find configs of this configName in DB
        query.prepare(QString(
"           SELECT id                               "
"             FROM SchedulerConfig                  "
"            WHERE configName = :configName         "
        ));
    }
    query.bindValue(":configName", configName);
    if(query.exec()){
        switch (query.size()) {
        case 0:{
            //--- new configuration
            qInfo() << "new configuration";
            //---try to find free id
            int free_id = find_free_id();
            if(free_id != -1){
                if (insert_querry(free_id)){
                    return true;
                }
            }
            else{
                qWarning() << "Insert in DB fails. Cant find free id.";
                return false;
            }
            break;
        }
        case 1:{
            //---update old configuration
            query.next();
            int update_id = query.value(0).toInt();
            if (update_querry(update_id)){
                return true;
            }
            else{
                return false;
            }
        }
        default:{
            qWarning() << QString("Warning! doubled row in db %1. Please, check config_name = %2, run_index = %3")
                          .arg(db.databaseName()).arg(configName).arg(runIndex);
            return false;
        }
        }
    }
    else{
        qWarning() << QString("Failed to make select from SchedulerConfig table: %1")
                      .arg( query.lastError().text());
        return false;
    }
    return false;
}

int DaqConfigDataBase::find_free_id()
{
    QSqlQuery query(db);
    query.prepare(QString(
"       SELECT id                   "
"         FROM %1                   "
    ).arg(table_name));
    int free_id=0;
    int expected_id=0;
    int next_id=0;
    bool free_id_find = false;
    if(query.exec()){
        while ((query.next()) && (!free_id_find)) {
            next_id = query.value(0).toInt();
            if (next_id != expected_id){
                free_id = expected_id;
                free_id_find = true;
            }
            else{
                ++expected_id;
            }
        }
        if (!free_id_find){
            free_id = ++next_id;
        }
        return free_id;
    }
    else{
        qWarning() << QString("Failed to make select from SchedulerConfig table: %1")
            .arg( query.lastError().text());
        return -1;
    }
}

bool DaqConfigDataBase::update_querry(int update_id)
{
    QString names_val_set = "  ";
    for(int i=0; i< names_val_list.size(); ++i){
        QString str_help = QString(", %1 = :%1").arg(names_val_list[i]);
        names_val_set.append(str_help);
    }

    QSqlQuery query(db);
    query.prepare(QString(
"       UPDATE %1                                           "
"          SET runIndex = :runIndex,                        "
"              configName = :configName,                    "
"              dateTimeOfInsert = :dateTimeOfInsert         "
"              %2                                           "
"        WHERE id = :id                                     "
    ).arg(table_name).arg(names_val_set));
    query.bindValue(":id", update_id);
    (runIndex == "" ? query.bindValue(":runIndex", QString("")) : query.bindValue(":runIndex", runIndex));
    query.bindValue(":configName", configName);
    query.bindValue(":dateTimeOfInsert", QDateTime::currentDateTime());

    for(int i=0; i< names_val_list.size(); ++i){
        for (QString str : m_vals.keys()) {
            if(str == names_val_list.value(i)){
                if(QMetaType::type(m_vals[str].typeName()) == QMetaType::Int){
                    query.bindValue(QString(":%1").arg(names_val_list.value(i)), get<int>(str));
                }
                if(QMetaType::type(m_vals[str].typeName()) == QMetaType::QString){
                    query.bindValue(QString(":%1").arg(names_val_list.value(i)), get<QString>(str));
                }
                if(QMetaType::type(m_vals[str].typeName()) == QMetaType::QDateTime){
                    query.bindValue(QString(":%1").arg(names_val_list.value(i)), get<QDateTime>(str));
                }
                if(QMetaType::type(m_vals[str].typeName()) == QMetaType::Double){
                    query.bindValue(QString(":%1").arg(names_val_list.value(i)), get<double>(str));
                }
            }
        }
    }

    if (query.exec()){
        qInfo().noquote() << QString("Update configuration:") << endl
                          << QString("                      table name = %1, ").arg(table_name) << endl
                          << QString("                      updated id = %2,").arg(update_id) << endl
                          << QString("                       run_index = %3, ").arg(runIndex) << endl
                          << QString("                     config_name = %4  ").arg(configName);
        return true;
    }
    else{
        qWarning() << QString("Failed to make update to SchedulerConfig table: %1")
            .arg( query.lastError().text());
        return false;
    }
}

bool DaqConfigDataBase::insert_querry(int insert_id)
{
    QString names_val_insert_into;
    for(int i=0; i< names_val_list.size(); ++i){
        QString tmp = QString(" ,%1").arg(names_val_list[i]);
        names_val_insert_into.append(tmp);
    }

    QString names_val_values;
    for(int i=0; i< names_val_list.size(); ++i){
        QString str_help = QString(", :%1").arg(names_val_list[i]);
        names_val_values.append(str_help);
    }

    QSqlQuery query(db);
    query.prepare(QString(
"       INSERT INTO %1                       "
"                   (                        "
"                   id,                      "
"                   runIndex,                "
"                   configName,              "
"                   dateTimeOfInsert         "
"                   %2 )                     "
"            VALUES (                        "
"                   :id,                     "
"                   :runIndex,               "
"                   :configName,             "
"                   :dateTimeOfInsert        "
"                   %3 )                     "
    ).arg(table_name).arg(names_val_insert_into).arg(names_val_values));
    query.bindValue(":id", insert_id);
    (runIndex == "" ? query.bindValue(":runIndex", QString("")) : query.bindValue(":runIndex", runIndex));
    query.bindValue(":configName", configName);
    query.bindValue(":dateTimeOfInsert", QDateTime::currentDateTime());

    for(int i=0; i< names_val_list.size(); ++i){
        for (QString str : m_vals.keys()) {
            if(str == names_val_list.value(i)){
                if(QMetaType::type(m_vals[str].typeName()) == QMetaType::Int){
                    query.bindValue(QString(":%1").arg(names_val_list.value(i)), get<int>(str));
                }
                if(QMetaType::type(m_vals[str].typeName()) == QMetaType::QString){
                    query.bindValue(QString(":%1").arg(names_val_list.value(i)), get<QString>(str));
                }
                if(QMetaType::type(m_vals[str].typeName()) == QMetaType::QDateTime){
                    query.bindValue(QString(":%1").arg(names_val_list.value(i)), get<QDateTime>(str));
                }
                if(QMetaType::type(m_vals[str].typeName()) == QMetaType::Double){
                    query.bindValue(QString(":%1").arg(names_val_list.value(i)), get<double>(str));
                }
            }
        }
    }

    if (query.exec()){
        qInfo().noquote() << QString("Insert configuration:") << endl
                          << QString("                      table name = %1,").arg(table_name) << endl
                          << QString("                       insert id = %2,").arg(insert_id) << endl
                          << QString("                       run_index = %3,").arg(runIndex) << endl
                          << QString("                     config_name = %4").arg(configName);
        return true;
    }
    else{
        qWarning() << QString("Failed to make insert to SchedulerConfig table: %1")
            .arg( query.lastError().text());
        return false;
    }
}

QList<QString> DaqConfigDataBase::get_configurations_list()
{
    QList<QString> config_list;
    config_list.clear();
    QSqlQuery query(db);
    query.prepare(QString(
"       SELECT configName                   "
"         FROM %1                           "
"        WHERE configName != ''             "
"          AND configName != :configName    "
"     ORDER BY dateTimeOfInsert             "
    ).arg(table_name));
    query.bindValue(":configName", "default");
    if (query.exec()){
        switch (query.size()) {
        case 0:{
            qInfo() << "configuration list is empty";
            break;
        }
        default:{
            while (query.next()) {
                config_list.append(query.value(0).toString());
            }
            break;
        }
        }
    }
    else{
        qWarning() << QString("Failed to make SELECT to SchedulerConfig table: %1")
            .arg( query.lastError().text());
    }

    query.prepare(QString(
"       SELECT *                            "
"         FROM %1                           "
"        WHERE configName = :configName     "
    ).arg(table_name));
    query.bindValue(":configName", "default");
    if (query.exec()){
        switch (query.size()) {
        case 0:{
            qInfo() << "there id no DEFAULT config name";
            break;
        }
        default:{
            config_list.prepend("default");
            break;
        }
        }
    }
    else{
        qWarning() << QString("Failed to make SELECT to %1 table: %2").arg(table_name)
            .arg( query.lastError().text());
    }

    return config_list;
}

QMap<QString, QString> DaqConfigDataBase::get_parameters_map()
{
    QMap<QString, QString> parameters_list;
    for (QString str : m_vals.keys()) {
        parameters_list[str].append(get<QString>(str));
    }
    return parameters_list;
}


bool DaqConfigDataBase::create_new_config(QString new_name)
{
    QList<QString> config_list = get_configurations_list();
    new_name = new_name.trimmed();
    if(new_name == "") return false;
    int sum=0;
    for(int i=0; i< config_list.size(); ++i){
        qInfo() << config_list[i];
        if(config_list[i] == new_name){
            ++sum;
        }
    }
    switch (sum) {
    case 0:{
        //---insert row
        QString tmp_config_name = configName;
        configName = "default";
        if (!readSettings()){
            return false;
        }
        //---try to find free id
        int free_id = find_free_id();
        if(free_id != -1){
            configName = new_name;
            if(!insert_querry(free_id)){
                return false;
            }
            configName = tmp_config_name;
            if (!readSettings()){
                return false;
            }
        }
        else{
            qWarning() << "Insert in DB fails. Cant find free id.";
            return false;
        }
        return true;
    }
    case 1:{
        qInfo() << "This configuration name already exists";
        return false;
    }
    default:{
        qInfo() << "This configuration name already exists";
        qWarning() << QString("Doubled row in DB. Please, check row with configName = %1").arg(new_name);
        return false;
    }
    }
}

bool DaqConfigDataBase::create_and_open_new_config(QString new_name)
{
    QList<QString> config_list = get_configurations_list();
    new_name = new_name.trimmed();
    if(new_name == "") return false;
    int sum=0;
    for(int i=0; i< config_list.size(); ++i){
        qInfo() << config_list[i];
        if(config_list[i] == new_name){
            ++sum;
        }
    }
    switch (sum) {
    case 0:{
        //---insert row
        configName = "default";
        if (!readSettings()){
            return false;
        }
        //---try to find free id
        int free_id = find_free_id();
        if(free_id != -1){
            configName = new_name;
            if(!insert_querry(free_id)){
                return false;
            }
        }
        else{
            qWarning() << "Insert in DB fails. Cant find free id.";
            return false;
        }
        return true;
    }
    case 1:{
        qInfo() << "This configuration name already exists";
        return false;
    }
    default:{
        qInfo() << "This configuration name already exists";
        qWarning() << QString("Doubled row in DB. Please, check row with configName = %1").arg(new_name);
        return false;
    }
    }
}


bool DaqConfigDataBase::rename_config(QString name, QString rename_to)
{
    name = name.trimmed();
    if(name == "") return false;
    rename_to = rename_to.trimmed();
    if(rename_to == "") return false;
    if(name == rename_to){
        qWarning() << QString("Nothing to rename. name = %1, rename_to = %2").arg(name).arg(rename_to);
        return true;
    }
    QList<QString> config_list = get_configurations_list();
    int sum=0;
    for(int i=0; i< config_list.size(); ++i){
        qInfo() << config_list[i];
        if(config_list[i] == name){
            ++sum;
        }
    }
    switch (sum) {
    case 0:{
        qWarning() << QString("Rename fails. There is no config name %1 in DB").arg(name);
        return false;
    }
    case 1:{
        QSqlQuery query(db);
        query.prepare(QString(
"           UPDATE %1                                     "
"              SET configName = :configName_rename        "
"            WHERE configName = :configName_old           "
        ).arg(table_name));
        query.bindValue(":configName_rename", rename_to);
        query.bindValue(":configName_old", name);
        if (query.exec()){
            qInfo().noquote() << QString("Update configuration:") << endl
                              << QString("                      table name = %1,").arg(table_name) << endl
                              << QString("                  old configName = %2,").arg(name) << endl
                              << QString("                  new configName = %3").arg(rename_to);
            return true;
        }
        else{
            qWarning() << QString("Failed to make update to SchedulerConfig table: %1")
                .arg( query.lastError().text());
            return false;
        }
    }
    default:{
        qWarning() << QString("Doubled row in DB. Please, check row with configName = %1").arg(name);
        QSqlQuery query(db);
        query.prepare(QString(
"           UPDATE %1                                    "
"              SET configName = :configName_rename       "
"            WHERE configName = :configName_old          "
        ).arg(table_name));
        query.bindValue(":configName_rename", rename_to);
        query.bindValue(":configName_old", name);
        if (query.exec()){
            qInfo().noquote() << QString("Update configuration:") << endl
                              << QString("                      table name = %1,").arg(table_name) << endl
                              << QString("                  old configName = %2,").arg(name) << endl
                              << QString("                  new configName = %3").arg(rename_to);
            return true;
        }
        else{
            qWarning() << QString("Failed to make update to SchedulerConfig table: %1")
                .arg( query.lastError().text());
            return false;
        }
    }
    }
}

bool DaqConfigDataBase::clone_config(QString name, QString name_of_clone)
{
    name = name.trimmed();
    if(name == "") return false;
    name_of_clone = name_of_clone.trimmed();
    if(name_of_clone == "") return false;
    if(name == name_of_clone){
        qWarning() << QString("Clone config fails! Duplicate config names are prohibited (%1 = %2) ").arg(name).arg(name_of_clone);
        return false;
    }
    QList<QString> config_list = get_configurations_list();

    //---if name of clone already exists, then fail to clone
    for(int i=0; i< config_list.size(); ++i){
        if(config_list[i] == name_of_clone){
            qWarning() << QString("Clone fails. Config name = %1 already exists in DB").arg(name_of_clone);
            return false;
        }
    }

    int sum2=0;
    for(int i=0; i< config_list.size(); ++i){
        if(config_list[i] == name){
            ++sum2;
        }
    }
    switch (sum2) {
    case 0:{
        qWarning() << QString("Clone fails. Table name = %1. There is no config name = %1 in DB").arg(table_name).arg(name);
        return false;
    }
    case 1:{
        //---try to find free id
        int free_id = find_free_id();
        if(free_id == -1){
            qWarning() << "Insert in DB fails. Cant find free id.";
            return false;
        }
        QString names_val_insert_into;
        for(int i=0; i< names_val_list.size(); ++i){
            QString tmp = QString(" ,%1").arg(names_val_list[i]);
            names_val_insert_into.append(tmp);
        }

        QString names_val_select;
        for(int i=0; i< names_val_list.size(); ++i){
            QString str_help = QString(", :%1").arg(names_val_list[i]);
            names_val_select.append(str_help);
        }
        QSqlQuery query(db);
        query.prepare(QString(
"           INSERT INTO %1                          "
"                       (id,                        "
"                       runIndex,                   "
"                       configName,                 "
"                       dateTimeOfInsert            "
"                       %2)                         "
"                SELECT :free_id,                   "
"                       runIndex,                   "
"                       :configName_of_clone,       "
"                       :dateTimeOfInsert           "
"                       %3                          "
"                  FROM SchedulerConfig             "
"                 WHERE configName = :configName    "
        ).arg(table_name).arg(names_val_insert_into).arg(names_val_select));
        query.bindValue(":free_id", free_id);
        query.bindValue(":configName", name);
        query.bindValue(":configName_of_clone", name_of_clone);
        query.bindValue(":dateTimeOfInsert", QDateTime::currentDateTime());
        if (query.exec()){
            qInfo().noquote() << QString("Clone configuration:") << endl
                              << QString("                      table name = %1,").arg(table_name) << endl
                              << QString("               master configName = %2,").arg(name) << endl
                              << QString("                clone configName = %3").arg(name_of_clone);
            return true;
        }
        else{
            qWarning() << QString("Failed to make INSERT INTO to SchedulerConfig table: %1")
                .arg( query.lastError().text());
            return false;
        }
        }
     default:{
        break;
    }
    }
    return true;
}

bool DaqConfigDataBase::delete_config(QString name)
{
    name = name.trimmed();
    if(name == "") return false;
    if(name == configName){
        qWarning() << QString("Prohibited to delete current config: %1").arg(name);
        return false;
    }
    QList<QString> config_list = get_configurations_list();
    int sum=0;
    for(int i=0; i< config_list.size(); ++i){
        if(config_list[i] == name){
            ++sum;
        }
    }
    switch (sum) {
    case 0:{
        qInfo() << QString("There is no configName %1 to delete").arg(name);
        return true;
    }
    case 1:{
        QSqlQuery query(db);
        query.prepare(QString(
"           DELETE FROM %1                               "
"                 WHERE configName = :configName         "
        ).arg(table_name));
        query.bindValue(":configName", name);
        if (query.exec()){
            qInfo() << QString("Configuration was deleted, configName = %1").arg(name);
            return true;
        }
        else{
            qWarning() << QString("Failed to make DELETE FROM to SchedulerConfig table: %1")
                .arg( query.lastError().text());
            return false;
        }
    }
    default:{
        qWarning() << QString("Doubled row in DB. ConfigName = %1 will be deleted").arg(name);
        QSqlQuery query(db);
        query.prepare(QString(
"           DELETE FROM %1                           "
"           WHERE configName = :configName           "
        ).arg(table_name));
        query.bindValue(":configName", name);
        if (query.exec()){
            qInfo() << QString("Configuration was deleted, configName = %1").arg(name);
            return true;
        }
        else{
            qWarning() << QString("Failed to make DELETE FROM to SchedulerConfig table: %1")
                .arg( query.lastError().text());
            return false;
        }
    }
    }
}

bool DaqConfigDataBase::switch_to_config(QString name)
{
    name = name.trimmed();
    if(name == ""){
        qInfo() <<  QString("Config name can't be empty");
        return false;
    }
    if(name == configName){
        qInfo() <<  QString("Config already switched to %1").arg(name);
        return false;
    }
    else{
        configName = name;
        if (readSettings()){
            return true;
        }
    }
    return false;
}

void DaqConfigDataBase::readSettings_fromConfig()
{
    QSettings &s = getSettings();
    for(int i=0; i< names_val_list.size(); ++i){
        for (QString str : m_vals.keys()) {
            if(str == names_val_list.value(i)){
                if(QMetaType::type(m_vals[str].typeName()) == QMetaType::Int){
                    set(str, s.value(str).toInt());
                }
                if(QMetaType::type(m_vals[str].typeName()) == QMetaType::QString){
                    set(str, s.value(str).toString());
                }
                if(QMetaType::type(m_vals[str].typeName()) == QMetaType::QDateTime){
                    set(str, s.value(str).toDateTime());
                }
                if(QMetaType::type(m_vals[str].typeName()) == QMetaType::Double){
                    set(str, s.value(str).toDouble());
                }
            }
        }
    }
}

void DaqConfigDataBase::writeSettings_toConfig()
{
    QSettings &s = getSettings();
    for(int i=0; i< names_val_list.size(); ++i){
        for (QString str : m_vals.keys()) {
            if(str == names_val_list.value(i)){
                if(QMetaType::type(m_vals[str].typeName()) == QMetaType::Int){
                    s.setValue(str, get<int>(str));
                }
                if(QMetaType::type(m_vals[str].typeName()) == QMetaType::QString){
                    s.setValue(str, get<QString>(str));
                }
                if(QMetaType::type(m_vals[str].typeName()) == QMetaType::QDateTime){
                    s.setValue(str, get<QDateTime>(str));
                }
                if(QMetaType::type(m_vals[str].typeName()) == QMetaType::Double){
                    s.setValue(str, get<double>(str));
                }
            }
        }
    }
    s.sync();
}

