#ifndef DAQCONFIGDATABASE_H
#define DAQCONFIGDATABASE_H

#include <QObject>
#include <QSqlQuery>
#include "DaqConfig.h"
#include "DaqDataBase.h"

class DaqConfigDataBase  :  public DaqConfig
{
public:
    DaqConfigDataBase(DaqConfigTypes t);
    bool readSettings();
    bool writeSettings();
    QString get_current_config_name() {return configName;}
    bool create_new_config(QString new_name);
    bool create_and_open_new_config(QString new_name);
    bool rename_config(QString name, QString rename_to);
    bool clone_config(QString name, QString name_of_clone);
    bool delete_config(QString name);
    bool switch_to_config(QString name);
    QList<QString> get_configurations_list();
    QMap<QString, QString> get_parameters_map();

    template< typename T >
    void set( const QString& key, const T& val ) {
        m_vals[key] = QVariant::fromValue( val );
    }

    template< typename T >
    T get( const QString& key, const T& defVal = T() ) {
        if( m_vals.contains( key ) ) {
            return unpack< T >( m_vals[ key ], defVal );
        }
        return defVal;
    }

    template< typename T >
    T unpack( const QVariant& var, const T& defVal = T() ) {
        if( var.isValid() && var.canConvert< T >() ) {
            return var.value< T >();
        }
        return defVal;
    }

private:
    bool update_querry(int update_id);
    bool insert_querry(int insert_id);
    void readSettings_fromConfig();
    void writeSettings_toConfig();
    int find_free_id();
    bool upload_zero_configurations();
    void update_parameters_from_query(QSqlQuery query);
    void debug_vals_print();

    DaqDataBase scheduler_db;
    QSqlDatabase db = scheduler_db.getConnection();
    QString configName;
    QString runIndex;

    QMap<QString, QVariant> m_vals;
    QString names_val;
    QStringList names_val_list;
    QString table_name;
};


#endif // DAQCONFIGDATABASE_H
