#ifndef ADDPNPDIALOG_H
#define ADDPNPDIALOG_H

#include <QDialog>

#include "pnp-server/PNPMessage.h"

class ProgramDescription;
class QTableWidget;
class QTableWidgetItem;
class QComboBox;
class QPushButton;

namespace Ui {
class AddClientDialog;
}

class AddPnpDialog : public QDialog
{
    Q_OBJECT

public:
    explicit AddPnpDialog(QVector<ProgramDescription> progDescrs,
                          bool multi_select = true,
                          QWidget *parent = nullptr);
    ~AddPnpDialog();
    QString getProgType() const;
    QString getProgHost() const;
    QString getProgIndex() const;
    QVector<ProgramDescription> getProgDescr() const;
    void set_program_type_filter(QStringList filter);
    void set_function_filter(std::function<bool (const ProgramDescription &)> f);

public slots:
    void gotClosedProgram(const ProgramDescription &closedDescr);
    void gotProgramDescription(ProgramDescription);

private slots:
    void on_comboBoxType_currentIndexChanged(const QString &type);
    void on_comboBoxHost_currentIndexChanged(const QString &host);
    void on_comboBoxIndex_currentIndexChanged(const QString &index);
    void on_leftTableWidget_itemDoubleClicked(QTableWidgetItem *item);
    void on_rightTableWidget_itemDoubleClicked(QTableWidgetItem *item);
    void on_leftTableWidget_itemSelectionChanged();
    void on_rightTableWidget_itemSelectionChanged();
    void on_pushButtonAdd_clicked();
    void on_pushButtonRemove_clicked();

private:
    void populate_left_table();
    void populate_all_filter();
    void populate_program_filter(QComboBox *box);
    void filter_item_changed();
    void update_detail_label(ProgramDescription *row_struct);
    void set_current_index(QComboBox *box, QString &item);
    void init_table(QTableWidget *table);
    void table_append_row(QTableWidget *table, const ProgramDescription &pd);
    void push_button_add_enable(bool enable);
    void push_button_remove_enable(bool enable);
    void change_button_color(QPushButton *push_button, QColor color);
    QUuid row_to_uuid(QTableWidget *table, int row);
    ProgramDescription *row_to_progdescr(QTableWidget *table, int row);
    void check_add_button_enable();
    void check_ok_button_enable();
    int find_table_proper_row(QTableWidget *table, const ProgramDescription &pd);
    int find_table_row(QTableWidget *table, const ProgramDescription &pd);
    bool is_prog_match_filter(const ProgramDescription &pd) const;
    bool is_prog_match_filter(const ProgramDescription &pd,
                              QString type, QString host, QString index) const;
    void add_prog(int row);
    void remove_prog(int row);
    void set_background_row_color(int row, QColor color) const;

    Ui::AddClientDialog *ui;
    QMap<QUuid, ProgramDescription> progs;
    QMap<QUuid, ProgramDescription> selected_progs;
    QString program_type_filter;
    QString host_name_filter;
    QString program_index_filter;
    bool multi_select;
    QStringList global_type_filter;
    std::function<bool(const ProgramDescription&)> filter_func;
    QUuid detailed_program_uuid;
};

#endif // ADDPNPDIALOG_H
