//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#include "qxw/MultiDoubleSpinBox.h"
#include "ui_MultiDoubleSpinBox.h"

MultiDoubleSpinBox::MultiDoubleSpinBox(QWidget *parent) :
        QWidget(parent),
        ui(new Ui::MultiDoubleSpinBox)
{
    ui->setupUi(this);
    pointCheckBoxes.push_back(ui->checkBox_1);
    pointCheckBoxes.push_back(ui->checkBox_2);
    pointCheckBoxes.push_back(ui->checkBox_3);
    pointCheckBoxes.push_back(ui->checkBox_4);
    pointCheckBoxes.push_back(ui->checkBox_5);
    pointCheckBoxes.push_back(ui->checkBox_6);
    pointCheckBoxes.push_back(ui->checkBox_7);
    pointCheckBoxes.push_back(ui->checkBox_8);
    pointSpinBoxes.push_back(ui->doubleSpinBox_1);
    pointSpinBoxes.push_back(ui->doubleSpinBox_2);
    pointSpinBoxes.push_back(ui->doubleSpinBox_3);
    pointSpinBoxes.push_back(ui->doubleSpinBox_4);
    pointSpinBoxes.push_back(ui->doubleSpinBox_5);
    pointSpinBoxes.push_back(ui->doubleSpinBox_6);
    pointSpinBoxes.push_back(ui->doubleSpinBox_7);
    pointSpinBoxes.push_back(ui->doubleSpinBox_8);
}

MultiDoubleSpinBox::~MultiDoubleSpinBox()
{
    delete ui;
}

void MultiDoubleSpinBox::changeEvent(QEvent *e)
{
    QWidget::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MultiDoubleSpinBox::setSuffix ( const QString & suffix )
{
    for(int i=0; i < numInputs(); i++) {
        pointSpinBoxes[i]->setSuffix(suffix);
    }
}

int MultiDoubleSpinBox::numInputs() const
{
    return std::min(pointSpinBoxes.size(), pointCheckBoxes.size());
}

void MultiDoubleSpinBox::setRange ( double minimum, double maximum )
{
    for(int i=0; i < numInputs(); i++) {
        pointSpinBoxes[i]->setRange(minimum, maximum);
    }
}

void MultiDoubleSpinBox::setValues(const QVector<double> &data, const QVector<int> &states)
{
    for(int i=0; i < numInputs(); i++) {
        if(i < data.size())
            pointSpinBoxes[i]->setValue(data[i]);
        else
            pointSpinBoxes[i]->setValue(0);
        bool state = i < states.size() && states[i];
        pointCheckBoxes[i]->setChecked(state);
    }
}

QVector<double> MultiDoubleSpinBox::getValues() const
{
    QVector<double> v;
    for(int i=0; i<numInputs(); i++) {
        const QDoubleSpinBox *spinBox = pointSpinBoxes.at(i);
        double value = spinBox->value();
        v.push_back(value);
    }
    return v;
}

QVector<int> MultiDoubleSpinBox::getStates() const
{
    QVector<int> v;
    for(int i=0; i<numInputs(); i++) {
        const QCheckBox *checkBox = pointCheckBoxes.at(i);
        v.push_back(checkBox->isChecked()?1:0);
    }
    return v;
}
