//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef WAVEFORMDATA_H
#define WAVEFORMDATA_H

#include <vector>
#include <string>

#include <qxw/qxw_global.h>

class point_t
{
public:
    point_t():x(0),y(0){}
    point_t(double x,double y):x(x),y(y){}
    double x;
    double y;
};

class QXW_EXPORT WaveformData {
public:
    WaveformData(const std::string &dataName = std::string(), int dataChannel = 0);
    WaveformData(const std::vector<double> &vy, const std::string &dataName = std::string(), int dataChannel = 0);
    WaveformData(const std::vector<double> &vx, const std::vector<double> &vy, const std::string &dataName = std::string(), int dataChannel = 0);
    WaveformData(const WaveformData&w) {copyObject(w);}
    void simulate(unsigned int length = 20);
    void print() const;
    void setName(const std::string &str) { name = str; }

    WaveformData& operator=(const WaveformData&w) {if (&w!=this) copyObject(w); return *this;}

    unsigned int c; // channel
    std::vector<point_t> p; // points
    std::vector<double> t; // time marks
    std::string name;

private:
    void copyObject(const WaveformData&w) {c=w.c; p=w.p; t=w.t; name=w.name;}
};

typedef std::vector<WaveformData> WaveformDataAll;

class WaveformDataSimulator
{
public:
    WaveformDataAll run();
};

#endif // WAVEFORMDATA_H
